local MSG = require "api_msg"

local module_name = ngx.var.module_name
local object_name = ngx.var.object_name
local method_name = ngx.var.method_name

if not module_name or module_name == "" or not object_name or object_name == "" then
	MSG.ERROR( "ERROR: Invalid request module", { reason = "api:check", module = module_name, method = object_name } )
	return
end

local args = {}
local args_get = ngx.req.get_uri_args()
local is_rpc = true

if module_name == "firmware" and (object_name == "upgrade" or object_name == "fpgaUp") then
	ngx.log( ngx.INFO, "For upload file(s) processing" )
elseif module_name == "pic" and object_name == "add" then
	ngx.log( ngx.INFO, "img file(s) processing")
else
	ngx.req.read_body()
	local args_post = ngx.req.get_post_args() or {}

	local req_headers = ngx.req.get_headers()
	args.head = { platform = req_headers and req_headers.platform or "" }
	for k, v in pairs(req_headers or {}) do
		args.head[k] = tostring(v):lower()
	end
	local content_type = req_headers and (req_headers["Content-Type"] or req_headers["content-type"]) or nil
	if content_type and content_type:match("application/json") then
		--Is JSON
		local cjson = require "cjson"
		local parsed_data={}
		for k,v in pairs(args_post) do
			local item=cjson.decode(k)
			if item then
				table.insert(parsed_data, item)
			end
		end

		if #parsed_data == 1 then
		--Just one json. (likely)
			args_post = parsed_data[1]
		else 
		--Unlikely
			args_post = parsed_data
		end
	end

	local k,v
	
	local arrIdx
	for k,v in pairs(args_post) do
		if type(k) == "number" then
			arrIdx = nil
		else
			arrIdx = k:match("(.*)%[%]")
		end
		if arrIdx then
			args[arrIdx] = v
		else
			args[k] = v
		end
	end
end

--Merge get arguments to post arguments.
for k,v in pairs(args_get) do
	if type(k) == "number" then
		arrIdx = nil
	else
		arrIdx = k:match("(.*)%[%]")
	end
	if arrIdx then
		args[arrIdx] = v
	else
		args[k] = v
	end
end

local module = require "api_module"
if module[module_name] then
	is_rpc = false
end

if not (module_name == "users" and object_name == "authorize" ) and (ngx.var.check_session == "true") then
	local check_session = module.users.session
	local is_ok, err = check_session(args)
	if not is_ok then
		return
	end
end

if not is_rpc then
	local method = module[module_name] and module[module_name][object_name]
	if not method then
		MSG.ERROR( "ERROR: Invalid request method", { reason = "api:method", module = module_name, method = object_name } )
	else
		method(args)
	end
	return 
end

local CO = require "api_coservice"
local proxy = CO.GetProxy("/" .. tostring(object_name), tostring(module_name))
if not proxy then
	MSG.ERROR( "Failure to connect " .. module_name .. " service" )
	return 
end

local function createResult(is_ok, ...)
	local data = {...}
	if not is_ok or not data[1] then
		MSG.ERROR( tostring(data[2] or data[1] or "systemError") )
		return
	end
	--NOTE: 如果有多个返回值这里要处理一下
	if #data == 1 then
		data = data[1]
	end
	MSG.OK("", { data = data})
end

createResult(proxy[method_name](args))
proxy:Destroy()
