local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local upload = require "resty.upload"

-- Include the global version info.
-- TODO: improve this.
local origPath = package.path
package.path = "/usr/share/lua/5.3/?.lua;" .. package.path
require "version"
package.path = origPath

local STORE_PATH = "/tmp"

local dset = {}

local function API_get(red, args)
	MSG.OK( "", {
		data = {
			firmwareVersion = _G.FIRMWARE_VERSION or "0.0.0",
			softwareVersion = _G.SOFTWARE_VERSION or "0.0.0"
		}
	})
	return true
end


local function checkPartType(res)
	if type(res) == "table" then
		local idr = res[1]:lower()
		if idr == "content-disposition" then
			local fieldname, filename = res[2]:match('name=%"(.*)%".*filename=%"(.*)%"')
			if not fieldname then
				fieldname = res[2]:match('name=%"(.*)%"')
			end
			if not fieldname then
				return nil
			end
			if not filename then
				return "field", fieldname
			else
				return "file", fieldname, filename
			end
		elseif idr == "content-type" then
			return idr, res[2] or ""
		else
			return nil
		end
	else
		return nil
	end
end

local function getUploadFile(args)
	args = args or {}
	local form,err = upload:new(8192)
	if not form then
		return nil, "uploadError"
	end
	form:set_timeout(5*1000)        --5secs timeout

	local fieldType, fieldName
	local srcFileName
	local curFile

	while true do
		local typ, res, err = form:read()
		if not typ then
			if curFile then
				curFile:close()
			end
			return nil, "getUploadFileError"
		end

		if typ == "header" then
			local ftype,fname,filename = checkPartType(res)
			if ftype == "field" then
				fieldType = ftype
				fieldName = fname
				args[fieldName] = ""
			elseif ftype == "file" then
				fieldType = ftype
				fieldName = fname
				srcFileName = filename
				if srcFileName == "" then
					return nil, "invalidUploadName"
				end
				if not srcFileName:match("%.[Bb][Ii][Nn]$") then
					srcFileName = srcFileName .. ".bin"
				end

			elseif ftype == "content-type" then
				if fieldType ~= "file" then
					if curFile then
						curFile:close()
					end
					return nil, "invalidUploadType"
				else
					local openerrormsg
					curFile, openerrormsg = io.open( STORE_PATH .. "/" .. srcFileName, "wb" )
					if not curFile then
						return nil, "invalidUploadCreate"
					end
				end
			else
				filedType = nil
				fieldName = nil
				if curFile then
					curFile:close()
					curFile = nil
				end
			end
		elseif typ == "body" then
			if fieldType == "field" then
				args[fieldName] = args[fieldName] .. res
			elseif fieldType == "file" then
				if not curFile then
					return nil, "uploadError"
				end
				curFile:write( res )
			end
		elseif typ == "part_end" then
			filedType = nil
			fieldName = nil
			if curFile then
				curFile:close()
				curFile = nil
			end
		elseif typ == "eof" then
			break
		end
	end

	if curFile then
		curFile:close()
	end

	if not srcFileName then
		return nil, "noneUpload"
	end

	return srcFileName
end

local function API_upgrade(red, args)
	local srcFileName, err = getUploadFile(args)
	if not srcFileName then
		return false, err or "uploadFileError"
	end

	-- local proxy = CO.GetProxy( "/record", "streamer", "local", 5 ) --Need a long timeout option
	-- if proxy then
	-- 	proxy.StopAll()
	-- 	proxy:Destroy()
	-- end

	proxy = CO.GetProxy( "/", "systemctrl", "local", 30 ) --This need 30seconds timeout, for purpose.
	if not proxy then
		os.execute( "/bin/rm -f " .. STORE_PATH .. "/" .. srcFileName )
		MSG.ERROR( "serverError" )
		return true
	end

	local r, done, reason, msg = proxy.ProcessFirmware( STORE_PATH .. "/" .. srcFileName )
	if not r or not done then
		proxy:Destroy()
		os.execute( "/bin/rm -f " .. STORE_PATH .. "/" .. srcFileName )
		MSG.ERROR( "upgradeServerError")
		return true
	end
	
	proxy.Reboot(2)
	proxy:Destroy()
	MSG.OK()
	return true
end

local function API_fpgaUp(red, args)
	local srcFileName, err = getUploadFile(args)
	if not srcFileName then
		return false, err or "uploadFileError"
	end

	local proxy = CO.GetProxy( "/", "systemctrl", "local", 300 ) --This need 5minutes timeout, for purpose.
	if not proxy then
		os.execute( "/bin/rm -f " .. STORE_PATH .. "/" .. srcFileName )
		MSG.ERROR( "serverError" )
		return true
	end

	local r, done, reason, msg = proxy.ProcessFirmware( STORE_PATH .. "/" .. srcFileName, "FPGA" )
	if not r or not done then
		proxy:Destroy()
		os.execute( "/bin/rm -f " .. STORE_PATH .. "/" .. srcFileName )
		MSG.ERROR( "upgradeFPGAError")
		return true
	end
	proxy.Reset()
	proxy:Destroy()
	MSG.OK()
	return true
end
--------------------------------------


dset["get"] = API_get
dset["upgrade"] = API_upgrade
--dset.APIS["fpgaUp"] = API_fpgaUp

--------------------------------------
return dset
