local _dbg = require "coaf.debug"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"

--For nginx using, the timeout unit is ms, so the timeoutUnit value is 1000
adapter.timeoutUnit = 1000
client.InitSocket(nil, adapter)

local _M = {}

--------------------------------

function _M.GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local funcs = {
	"Update", "Update2",
	"GetLinkDetail", "ChangeConnection",-- "audio_set", 
}

local params = {
	Update2 = {
		chnId = 1,
		new = {
            aacType = "G711",
            resample = "fastest",
            sampling = 48000,
            channels = 2,
            bitrate = 64000,
            law = "ULAW",
            source = {
                enSamplerate = 48000,
                enBitwidth = 16,
                audioSource = "hdmi",
                u32EXFlag = 0,
                enWorkmode = "I2S_SLAVE",
                u32ChnCnt = 2,
                u32FrmNum = 30,
                u32ClkChnCnt = 2,
                u32PtNumPerFrm = 1024,
                u32ClkSel = 0,
                enSoundmode = "STEREO",
            },
        }
	},
	GetLinkDetail = {
		ifname = "eth0"
	},
	ChangeConnection = {
		ifname = "eth0",
		cfg = {
			address = "192.168.0.111"
		}
	},
	GetMediaSW = {
		stream = "main"
	},
	Update = {
		stream = "main",
		new = {
			type = "H264",
		}	
	},
	SetVqeVolume ={
		chnId = 1, vol = 5
	},
	setVqeAttr = {
		tbl = {
			bHpfOpen = 0,
			bAecOpen = 0,
			bAnrOpen = 0,
			bRnrOpen = 0,
			bAgcOpen = 1,
			bEqOpen = 0,
			bHdrOpen = 0,
			s32WorkSampleRate = 16000,
			s32FrameSample = 1024,
			enWorkstate = 0,
			stAgcCfg = {
				bUsrMode = 1,
				s8TargetLevel = -2,
				s8NoiseFloor = -40,
				s8MaxGain = 0,
				s8AdjustSpeed = 10,
				s8ImproveSNR = 2,
				s8UseHighPassFilt = 0,
				s8OutputMode = 0,
				s16NoiseSupSwitch = 1,
				s32Reserved = 1,
			}
		}
	},
	DelOSDItem = {
		chnId = 1,
		idx = 1
	},
	UpdateConfig = {
		chnId = 1,
		cfg = {
			source = {
				enWorkmode = "I2S_SLAVE", --NOTE: 3521D 不能改，硬件决定
				enSamplerate = 48000,
				enBitwidth = 16,
				enSoundmode = "MONO",
				-- enSoundmode = "STEREO",
				u32EXFlag = 0,
				u32FrmNum = 30,
				u32PtNumPerFrm = 1024,
				u32ClkChnCnt = 2,
				u32ChnCnt = 2,
				u32ClkSel = 0
			}, -- 音频输入源
			vqeVolume = 0,
	
			sampleSize = 160,
			channels = 1,
			-- sampling = 48000,
			sampling = 8000,
			-- aacType = "AAC",
			aacType = "G711",
			aacFormat = "RAW",
			bitrate = 64000,
		}
    },
	Exit = {},
	Rtsp_exit = {
		stream = "main"
	},
	audio_set = {},
	set_crop = {
		u32Height = 0,
		u32Width = 0,
		s32X = 0,
		s32Y = 0,
	},
	get_crop = {},
	get_video_attr = {
		stream = "main"
	},
	video_set_update2 = {
		stream = "main",
		new = {
			type = "H264",
			ve = {
				-- u32PicHeight = 1080,
				-- u32PicWidth = 1920,
			},
		}
	},
}

local functype = funcs[tonumber(arg[1])]
local mtype = "/codec"

local param = params[functype]
if functype == "video_set_update2" then
	functype = "video_set_update"
elseif functype == "Update" or functype == "Update2" then
	functype = "Update"
elseif functype == "UpdateConfig" then
	mtype = "/audio"
end
local proxy = _M.GetProxy(mtype, "codec")
print("functype = ", functype)
if proxy then
    local is_ok, skip = proxy[functype](param)
    print("is_ok = ", is_ok, "skip = ", skip)
    proxy:Destroy()
	if functype == "GetMediaSW" then
		print(skip, skip.GetMediaSource)
	end
end

return _M
