package.path = package.path .. ";./chips/?.lua"
require "himpp"
require "kmCore"

local _dbg = require "coaf.debug"
local coaf = require "coaf.core"
local pt = require ("pl.pretty")
local vpss = require "soc.vpss"

local encoder = require "soc.encoder"
local ain = require "soc.ain"

local mod_Streamer, mod_record

local codec = {
    doc = "",
    enc_dict = {},
    hiEncSource_dict = {},
}


local index_tbl = {
    main = 1, sub = 2, --mjpeg = 3, 
    preview = 4,
}

--NOTE: 这里根据配置文件，创建不同码流
--NOTE： 为了以后拓展为 可动态选择 音频源、视频源 来创建码流，可以在配置文件中
--       增加 音频源id，视频源id属性，然后这里根据相应的 音视频源id 参数 ，找到相应的 音视频源
--       再根据配置文件中的编码参数 来进行编码。
local function createNewChannel(stream, index, attr, reInit)
    local encSource = codec.hiEncSource_dict[stream]
    if encSource then
        encSource:Stop()
    end

    local enc = codec.enc_dict[stream]
    if enc then
        enc:Destroy()
        codec.enc_dict[stream] = nil
        encoder.channels[enc.id] = nil
    end
    collectgarbage()
    if not attr.enable then
        return true
    end

    enc = encoder.CreateChannel( index )
    if not enc then
        _dbg.Error("CODEC", "Create venc channel failed!")
        return false
    end

    enc:BindSourceChannel(codec.vi_vpss_list[index])

    enc.encodingOpt.type = attr.type:upper()
    table.update(enc.encodingOpt.attr, attr.ve)

    enc.bitrateOpts.mode = enc.encodingOpt.type .. attr.rc.mode:upper()
    table.update(enc.bitrateOpts.attr, attr.rc)

    local is_ok = enc:Start()
    if not is_ok then
        _dbg.Error("ENCODE", "!!!!!!!Create VENC Channel Failed!!!!!!")
        return false
    end
    enc:SetColor2Grey(attr.grey)

    codec.enc_dict[stream] = enc

    if not encSource or reInit then
        encSource = KM_STREAMING.HISI.Encoder.createNew(index - 1)
        if not encSource then
            _dbg.Error( "STREAM", "Fail to create Hi-encoder source! " )
            return false
        end
        if not encSource:Init(false, 4*1024*1024, nil) then
            _dbg.Error( "STREAM", "Fail to initialize Hi-encoder source !" )
            return false
        end
    else
        encSource:Start()
    end

    codec.hiEncSource_dict[stream] = encSource
    return true
end

function codec.Init()
    encoder.Init()
    codec.vpss = vpss.CreateChannel(1)
    if not codec.vpss then
        _dbg.Warn( "CODEC", "Fail to create OUTPUT vpss channel, so OUTPUT encoding streams may be wrong!" )
    else
        codec.vpss:Init()
    end

    codec.vpss:Start()
    codec.vi_vpss_group = codec.vpss:GetVPSSGroup()
    if not codec.vi_vpss_group then
        _dbg.Warn( "CODEC", "Fail to get OUTPUT capture VPSS channel, so OUTPUT encoding streams may be wrong!" )
        codec.vi_cap = nil
    else
        codec.vi_vpss_list = {
            codec.vi_vpss_group:GetChannel(0),
            codec.vi_vpss_group:GetChannel(1),
            codec.vi_vpss_group:GetChannel(2),
            codec.vi_vpss_group:GetChannel(3),
        }
    end

    for stream, v in pairs(_G.codec_cfg) do
        createNewChannel(stream, index_tbl[stream], v)
    end

    codec.vpss:setCrop()
end

function codec.GetMediaSW(stream)
    if stream and codec.mediaSW_dict[stream] and codec.syncSource_dict[stream] then
        return codec.syncSource_dict[stream], codec.mediaSW_dict[stream]
    end
end

function codec.Exit()
    for i, v in pairs(codec.hiEncSource_dict) do
        v:Stop()
        codec.hiEncSource_dict[i] = nil
    end

    encoder.Destroy()
end

local static_params = {
    u32MaxPicWidth = true,
    u32MaxPicHeight = true,
    u32BufSize = true,
    bByFrame = true,
    u32Profile = true,
    -- u32RefNum = true,
    -- u32BFrameNum = true,
}

function codec.Update(stream, new, must)
    mod_record = mod_record or require "mod_record"
    mod_Streamer = mod_Streamer or require "mod_Streamer"
    local enc = codec.enc_dict[stream]
    -- if not enc then
    --     return false
    -- end

    local copy = table.copy(_G.codec_cfg[stream])
    if not table.compare(copy, new) then
        if not (new.ve and new.ve.u32PicWidth == 1) and not must then
            return true
        end
    end

    local reCreate = false
    local reInit = false
    --NOTE: 检验是否需要销毁通道重新创建
    for k in pairs(static_params) do
        reInit = reInit or (new.type ~= nil and new.type ~= _G.codec_cfg[stream].type)
        if reInit or (new.enable ~= nil and new.enable ~= _G.codec_cfg[stream].enable)
        or (new.ve and new.ve[k] ~= nil and new.ve[k] ~= _G.codec_cfg[stream].ve[k]) then
            reCreate = true
            break
        end
    end

    local reCreateNDI = new.rc and new.rc.fr32DstFrmRate and _G.codec_cfg[stream].rc.fr32DstFrmRate ~= new.rc.fr32DstFrmRate
    if reCreate then
        mod_Streamer.Destroy(stream)
        mod_record.StopRecord(stream)
    end

    if new.ve and new.ve.u32PicHeight and new.ve.u32PicHeight ~= 1 then
        if copy.ve.u32PicHeight ~= 1 then
            new.ve.u32PicWidth = math.min(new.ve.u32PicWidth, _G.capture_cfg[1].stCapRect[3])
            new.ve.u32PicHeight = math.min(new.ve.u32PicHeight, _G.capture_cfg[1].stCapRect[4])
        end
    end

    table.update(copy, new)

    copy.ve.u32BufSize = copy.ve.u32MaxPicWidth * copy.ve.u32MaxPicHeight * 2

    local encode_cfg = {
        type = copy.type:upper(),
        attr = copy.ve
    }
    local bitrate_cfg = {
        mode = encode_cfg.type .. copy.rc.mode:upper(),
        attr = copy.rc
    }

    local is_ok
    if reCreate or must or not enc then
        is_ok = createNewChannel(stream, index_tbl[stream], copy, reInit)
    else
        codec.hiEncSource_dict[stream]:Stop()
        is_ok = enc:UpdateEncoding(encode_cfg, bitrate_cfg)
        if new.grey ~= nil and new.grey ~= _G.codec_cfg[stream].grey then
            local num = enc:SetColor2Grey(new.grey)
            is_ok = is_ok and num == 0
        end
        codec.hiEncSource_dict[stream]:Start()
    end

    --NOTE：如果设置的参数不生效，要及时设置回正常的，不然会影响海思 VENC 系统，导致后续的
    --      调用会出现  0012 或其他错误。

    if not is_ok then
        if reCreate then
            createNewChannel(stream, index_tbl[stream], _G.codec_cfg[stream])
        else
            enc:UpdateEncoding({
                type = _G.codec_cfg[stream].type:upper(),
                attr = _G.codec_cfg[stream].ve
            }, {
                mode = _G.codec_cfg[stream].type:upper() .. _G.codec_cfg[stream].rc.mode:upper(),
                attr = _G.codec_cfg[stream].rc
            })
        end
        return false, "argumentError"
    end

    if reCreate then
        mod_Streamer.Init(stream)
        mod_record.StartRecord(stream, 2)
    end
    
    table.update(_G.codec_cfg[stream], new)
    _G.codec_cfg:__TRIGGER(true)
    codec.vpss:resizeVpss(stream)
    if not reCreate and reCreateNDI then
        mod_Streamer.ReStartNDI()
    end
    return true
end

function codec.GetInfo(stream)
    -- local enc = codec.enc_dict[stream]
    -- if enc then
    --     enc:GetAttr()
    -- end
    if stream then
        local copy = table.copy(_G.codec_cfg[stream])
        if copy then
            copy.capSize = _G.capture_cfg[1].stDestSize
        end
        return copy
    end
    local copy_main = table.copy(_G.codec_cfg.main)
    local copy_sub = table.copy(_G.codec_cfg.sub)
    copy_main.capSize = _G.capture_cfg[1].stDestSize
    copy_sub.capSize = {
        math.min(_G.capture_cfg[1].stDestSize[1], 1920),
        math.min(_G.capture_cfg[1].stDestSize[2], 1080),
    }
    return {
        main = copy_main,
        sub = copy_sub,
    }
end

function codec.GetRealBitrate(stream)
    local encSource = codec.hiEncSource_dict[stream]
    if not encSource then
        return
    end
    return encSource:GetVideoBiteRate()
end

function codec.SetCrop(_, cfg)
    local vpssGroup = codec.vi_vpss_group
    if not vpssGroup then
        _dbg.Error("Codec","Not have vpssGroup, Can not Set Crop")
        return
    end
    local copy = table.copy(_G.vpss_cfg)
    table.update(copy, cfg)
    copy.s32X = tonumber(copy.s32X)
    copy.s32Y = tonumber(copy.s32Y)
    copy.u32Width = tonumber(copy.u32Width)
    copy.u32Height = tonumber(copy.u32Height)
    if not copy.s32X or not copy.s32Y or not copy.u32Width or not copy.u32Height or type(copy.enable) ~= "boolean"
    or copy.s32X < 0 or copy.s32Y < 0 or copy.u32Height < 0  or copy.u32Width < 0 or (copy.s32X + copy.u32Width > 100) or (copy.s32Y + copy.u32Height > 100) then
        return false, "Invalid arguments!"
    end

    local is_ok = codec.vpss:setCrop(copy)
    if is_ok then
        table.update(_G.vpss_cfg, cfg)
        _G.vpss_cfg:__TRIGGER(true)
    end
    return is_ok, not is_ok and "Set Crop Failed!"
end

function codec.GetCrop()
    return _G.vpss_cfg
end

function codec.WidthIsBigger()
    local isWidthBigger = _G.capture_cfg[1].stDestSize[1] > _G.capture_cfg[1].stDestSize[2]
    local exchange = _G.capture_cfg[1].rotate == 90 or _G.capture_cfg[1].rotate == 270
    if exchange then
        isWidthBigger = not isWidthBigger
    end
    return isWidthBigger
end

function codec.RotateVPSSAndVenc()
    for stream, enc in pairs(codec.enc_dict) do
        codec.Update(stream, _G.codec_cfg[stream], true)
    end
end

function codec.getVpssAndPicInfo()
    local result = {}
    for k, v in pairs(codec.enc_dict) do
        local index = index_tbl[k]
        result[index] = {
            {
                vpssGroup = vi_vpss_group,
                vpssChannel = codec.vi_vpss_list[index],
                enc = v,
                type = "region",
                target = "encoder",
                encObj = {
                    vencChannel = v.vencChannel,
                    u32PicWidth = _G.codec_cfg[k].ve.u32PicWidth,
                    u32PicHeight = _G.codec_cfg[k].ve.u32PicHeight,
                }
            }
        }
    end
    return result
end

function codec.GetAttr(stream)
    local enc = codec.enc_dict[stream]
    if not enc then
        return
    end

    return enc.vencChannel:GetAttr()
end

codec.Rule = {
    GetRealBitrate = {
        parameters = {
            {key = "stream", type = "string"},
        },
        is_open = true
    },
    SetCrop = {
        parameters = {
            {key = "chnId", type = "number"},
            {key = "cfg", type = "table", must = true},
        },
        is_open = true
    },
    GetCrop = {
        parameters = {
        },
        is_open = true
    },
    Exit = {
        parameters = {},
        is_open = true
    },
    GetInfo = {
        parameters = {
            {key = "stream", type = "string"},
        },
        is_open = true
    },
    Update = {
        parameters = {
            {key = "stream", type = "string", must = true},
            {key = "new", type = "table", must = true},
            {key = "must", type = "boolean"},
        },
        is_open = true
    },
    GetAttr = {
        is_open = true,
        parameters = {
            {key = "stream", type = "string", must = true},
        }
    },
}

return codec