local DC = require "supertable.declare"
local CONF = require "supertable.core"
local mo_OSD

local pt = require "pl.pretty"
local index_tbl = {
	main = 1, sub = 2
}
local function OnResolutionChanged(cfg)
	mo_OSD = mo_OSD or require "mod_OSD"
	if cfg.enable then
		local chnName = tostring(cfg.__proto.name)
		local chnId = index_tbl[chnName]
		if chnId then
			mo_OSD.ResolutionChanged(chnId)
		end
	end
end

local _M = {
	config_file = "/data/configs/codec_config.conf"
}

local codec_template = DC.OBJ({
	["main"] = DC.OBJ{
		type = DC.STR("H264"),
		enable = DC.BOOL(true),
		grey = DC.BOOL(false),
		ve = DC.OBJ{
			u32PicWidth = DC.INT(1920),
			u32PicHeight = DC.INT(1080),

			-- 以下是静态属性
			-- 264 265 mjpeg
			u32MaxPicWidth = DC.INT(1920),
			u32MaxPicHeight = DC.INT(1080),
			u32BufSize = DC.INT(1920*1080*2),
			bByFrame = DC.BOOL(true),

			u32Profile = DC.INT(0),  -- 264  265 --265只能为0
			-- u32RefNum = DC.INT(0),    -- 264 265 --3521D 没有这个参数
			-- u32BFrameNum = DC.INT(0),   -- 264  265 --3521D 没有这个参数
		},

		rc = DC.OBJ{
			mode = DC.STR("CBR"),
			u32Gop = DC.INT(60),
			u32StatTime = DC.INT(1),
			u32SrcFrmRate = DC.INT(30),
			fr32DstFrmRate = DC.INT(30),

			u32MaxQp = DC.INT(48),   --vbr
			u32MinQp = DC.INT(10),  --vbr
			u32MaxBitRate = DC.INT(8000),   --vbr
			u32MinIQp = DC.INT(20), --vbr

			u32BitRate = DC.INT(4000), --cbr 
			u32FluctuateLevel = DC.INT(1), -- cbr
		}
	}: __changed( OnResolutionChanged ),

	["sub"] = DC.OBJ{
		type = DC.STR("H264"),
		enable = DC.BOOL(true),
		grey = DC.BOOL(false),
		ve = DC.OBJ{
			u32PicWidth = DC.INT(1920),
			u32PicHeight = DC.INT(1080),
			u32MaxPicWidth = DC.INT(1920),
			u32MaxPicHeight = DC.INT(1080),
			u32BufSize = DC.INT(1920*1080*2),
			bByFrame = DC.BOOL(true),
			u32Profile = DC.INT(0),  -- 264  265
			-- u32RefNum = DC.INT(0),    -- 264 265  --3521D 没有这个参数
			-- u32BFrameNum = DC.INT(0),   -- 264  265  --3521D 没有这个参数
		},
		rc = DC.OBJ{
			mode = DC.STR("CBR"),
			u32Gop = DC.INT(60),
			u32StatTime = DC.INT(1),
			u32SrcFrmRate = DC.INT(30),
			fr32DstFrmRate = DC.INT(30),

			u32MaxQp = DC.INT(48),   --vbr
			u32MinQp = DC.INT(10),  --vbr
			u32MaxBitRate = DC.INT(8000),   --vbr
			u32MinIQp = DC.INT(20), --vbr

			u32BitRate = DC.INT(4000), --cbr 
			u32FluctuateLevel = DC.INT(1), -- cbr
		}
	}: __changed( OnResolutionChanged ),

	-- ["mjpeg"] = DC.OBJ{
	-- 	type = DC.STR("MJPEG"),
	-- 	enable = DC.BOOL(true),
	-- 	ve = DC.OBJ{
	-- 		u32PicWidth = DC.INT(1920),
	-- 		u32PicHeight = DC.INT(1080),
	-- 		u32MaxPicWidth = DC.INT(1920),
	-- 		u32MaxPicHeight = DC.INT(1080),
	-- 		u32BufSize = DC.INT(1920*1080*2),
	-- 		bByFrame = DC.BOOL(true),
	-- 	},

	-- 	rc = DC.OBJ{
	-- 		mode = DC.STR("CBR"),
	-- 		u32StatTime = DC.INT(1),
	-- 		u32SrcFrmRate = DC.INT(30),
	-- 		fr32DstFrmRate = DC.INT(30),

	-- 		u32MaxQfactor = DC.INT(50),   --vbr  -- 1-99
	-- 		u32MinQfactor = DC.INT(30),  --vbr  -- 1-max
	-- 		u32MaxBitRate = DC.INT(8000),   --vbr

	-- 		u32BitRate = DC.INT(4000), --cbr 
	-- 		u32FluctuateLevel = DC.INT(1), -- cbr
	-- 	}
	-- }
}) : __changed( function(conf)
	conf:TAB_SAVE()
end)

local default = {
	main = {
		type = "H264", enable = true,
		grey = false,
		ve = {
			u32PicWidth = 1,
			u32PicHeight = 1,

			-- 以下是静态属性
			-- 264 265 mjpeg
			u32MaxPicWidth = 3840,
			u32MaxPicHeight = 2168,
			u32BufSize = 3840*2168*2,
			bByFrame = true,

			u32Profile = 1,  -- 264  265
			-- u32RefNum = 0,    -- 264 265 --3521D没有这个参数
			-- u32BFrameNum = 0,   -- 264  265  --3521D没有这个参数
		},
		rc = {
			mode = "CBR",
			u32Gop = 60,
			u32StatTime = 1,
			u32SrcFrmRate = 60,
			fr32DstFrmRate = 60,

			u32MaxQp = 48,   --vbr
			u32MinQp = 10,  --vbr
			u32MinIQp = 29, --vbr
			u32MaxBitRate = 6000,   --vbr

			u32BitRate = 6000, --cbr 
			u32FluctuateLevel = 1, -- cbr
		},
	},
	sub = {
		type = "H264", enable = true,
		ve = {
			u32PicWidth = 1280,
			u32PicHeight = 720,

			-- 以下是静态属性
			-- 264 265 mjpeg
			u32MaxPicWidth = 3840,
			u32MaxPicHeight = 2168,
			u32BufSize = 3840*2168*2,
			bByFrame = true,

			u32Profile = 0,  -- 264  265
			-- u32RefNum = 0,    -- 264 265
			-- u32BFrameNum = 0,   -- 264  265
		},
		rc = {
			mode = "CBR",
			u32Gop = 60,
			u32StatTime = 1,
			u32SrcFrmRate = 60,
			fr32DstFrmRate = 30,

			u32MaxQp = 48,   --vbr
			u32MinQp = 10,  --vbr
			u32MaxBitRate = 8000,   --vbr
			u32MinIQp = 20, --vbr

			u32BitRate = 4000, --cbr 
			u32FluctuateLevel = 1, -- cbr
		},
	},
	preview = {
		type = "H264", enable = true,
		ve = {
			u32PicWidth = 960,
			u32PicHeight = 540,

			-- 以下是静态属性
			-- 264 265 mjpeg
			u32MaxPicWidth = 960,
			u32MaxPicHeight = 968,
			u32BufSize = 960*968*2,
			bByFrame = true,

			u32Profile = 0,  -- 264  265
			-- u32RefNum = 0,    -- 264 265
			-- u32BFrameNum = 0,   -- 264  265
		},
		rc = {
			mode = "CBR",
			u32Gop = 60,
			u32StatTime = 1,
			u32SrcFrmRate = 30,
			fr32DstFrmRate = 30,

			u32MaxQp = 48,   --vbr
			u32MinQp = 10,  --vbr
			u32MaxBitRate = 8000,   --vbr
			u32MinIQp = 20, --vbr

			u32BitRate = 1024, --cbr 
			u32FluctuateLevel = 1, -- cbr
		},
	},
}

--------------------------------------------
function _M.Load()
	local conf = CONF.Create({}, codec_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure
		conf:TAB_BIND( default )
	end

	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------
return _M