local DC = require "supertable.declare"
local CONF = require "supertable.core"
local is_ok, FEATURES = pcall(require, "features")

local _M = {
	config_file = "/data/configs/streams.conf"
}

--------------------------------------------------------

local streams_template = DC.OBJ{
	main = DC.ARR(
        DC.OBJ{
            name = DC.STR(""),
            type = DC.STR("rtmp_push"),
            enable = DC.BOOL(false),
            bindAudio = DC.INT(1),
        }
    ),
    sub = DC.ARR(
        DC.OBJ{
            name = DC.STR(""),
            type = DC.STR("rtmp_push"),
            enable = DC.BOOL(false),
            bindAudio = DC.INT(1),
        }
    ),
 } : __changed( function(conf)
	conf:TAB_SAVE()
end)

local default = {
    main = {
        {
            bindAudio = 1,
            name = "RTSP",
            type = "rtsp",
            enable = true,
            session = "ch01",
            port = 554,
            httpTunnelPort = 8554,
            auth = false,
            no_adts = 0,
            ssm = 0,
            multicast_addr = "224.0.0.1",
            multicast_ttl = 127,
            multicast_video_port = 3101,
            multicast_audio_port = 3102,
        },
        {
            bindAudio = 2,
            name = "SRT",
            type = "SRT",
            enable = true,
            address = "",
            connectionMode = "Listener",
            encryption = 0,
            passphrase = "",
            latency = 125,
            bandwidth = 5,
            payloadSize = 1316,
            listenerPort = 8083,
            srt_stream_id = ""
        }
    },
    sub = {
    },
}

if is_ok and FEATURES.FEATURE_DEFAULT_RTSP_AUTH then
    default.main[1].auth = true
end

if is_ok and not FEATURES.FEATURE_NO_NDI then
    table.insert(default.main, 1, {
        name = "NDI-HX",
        type = "ndi_hx",
        enable = true,
        ttl = "1",
        NDIname = "Chan1",
        group = "",
        connection = "tcp",
        netmask = "255.255.0.0",
        netprefix = "239.255.0.0",
        discovery_server = "",
        bindAudio = 1,
    })
end

-------------------------------------------------------
function _M.Load()
	local conf = CONF.Create({}, streams_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
    if not r then
		conf:TAB_BIND( default )
    end
	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------

return _M


