local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local client = require "coaf.client"
-- local vscreen = require "textScreen"
local OSD_IMAGE_LOC = "/data/storage/img"

local MAX_AREAS = 8 --海思自身限制，最多8个
--<3>[   rgn] [Func]:RgnAttachToChn [Line]:5671 [Info]:<3>[   rgn] Chn(Mod:8,Dev:0,Chn:0, Num:8, MaxNum:8)'s RegionNum is too much!

local _M = {
	initialized = false,
	mo_CodecChannel = nil,
	channels = {
		[1] = {
			osdGroups = {},
			withRemote = false,
			remoteCli = nil,
			remoteProxy = nil
		},
		[2] = {
			osdGroups = {},
			withRemote = false,
			remoteCli = nil,
			remoteProxy = nil
		}
	},

	-- vs = vscreen.New()
}

--Load saved text.
-- _M.vs:Load()
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)
local function GetProxy(objPath, service, loc, timeout, addr)
	local proxy
	local cli

	cli = client.New( service, loc, addr, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local function GetIMGSize(fileID)
	local proxy = GetProxy("/pics", "picManager")
	if not proxy then
		_dbg.Error("Can not Create picManager proxy in line 58")
		return
	end
	local is_ok, size_w, size_h, rgba = proxy.GetIMGDesc(fileID)
	proxy:Destroy()
	if not is_ok or not size_w then
		return 
	end
	return size_w, size_h, rgba
end


----------------------------------------------
local osdgroup_meta = {}

function osdgroup_meta:Init( width, height )
	if self.started then
		self.osdObj:Reset()
	end

	self.items = {}

	self.osdObj:ChangeSize( width, height )
	self.width = width
	self.height = height
	self.started = true
end

function osdgroup_meta:Destroy()
	if self.started then
		self.osdObj:Destroy()
		self.osdObj = nil
	end
	collectgarbage()
end

function osdgroup_meta:UpdateOSDArea( areaId, glbCfg, itemCfg )
	if self.width <=0 or self.height <=0 then
		_dbg.Warn( "OSD", "The stream of " .. self.streamId .. " with invalid size! So could not update the areas." )
		return
	end

	local text
	local osd_type = itemCfg.type or itemCfg.osd_type

	self.osdObj:DropArea(areaId)
	if not osd_type or osd_type == "none" or osd_type == "" or not itemCfg.enable or not glbCfg.enable then
		return
	elseif osd_type == "time" then
		text = '%Y/%m/%d %H:%M:%S'
	elseif osd_type == "date" then
		text = '%Y/%m/%d'
	elseif osd_type == "clock" then
		text = '%H:%M:%S'
	elseif osd_type == "image" then
		text = "image"
	elseif osd_type == "text" then
		text = itemCfg.text
	end

	self.osdObj:ChangeAreaType(areaId, osd_type )

	if not text or text == "" then
		self.osdObj:EnableArea(areaId, false)
		return
	else
		self.osdObj:EnableArea(areaId, true)
	end

	-- local screenW = glbCfg.gridW or 40
	-- local screenH = glbCfg.gridH or 25

	-- --Don't be too small
	-- if screenW < 16 then screenW = 16 end
	-- if screenH < 9 then screenH = 9 end

	local thisAreaW, thisAreaH, rgba
	local xOffset = 16
	local yOffset = 16

	if osd_type ~= "image" then
		local screenGridW, screenGridH
		local isBold = 0
		local isItalic = 0
		if itemCfg.style and itemCfg.style == "bold" then
			isBold = 1
		end
		local color = itemCfg.color or 1
	
		local lineCnt = 0
		local font, fontSize = string.lower(itemCfg.font or 0), 32
		font = (_M.FontMap and _M.FontMap[font]) or _M.DefaultFontIndex or 0
		if itemCfg.size <= 2 then
			fontSize = math.floor(self.width / (80 - math.abs(itemCfg.size - 2) * 20))
		else
			fontSize = itemCfg.size
		end
		screenGridW = fontSize
		screenGridH = screenGridW + 1  -- 这个是行间距 默认1

		self.osdObj:SetAreaFont( areaId, { font = font, size=fontSize, bold=isBold, italic=isItalic, outline=math.min(itemCfg.outline, 2) } )
		self.osdObj:SetAreaFontColor( areaId, color )
		self.osdObj:SetAreaOutlineColor( areaId, itemCfg.outlineColor )
		self.osdObj:SetAreaLineHeight( areaId, screenGridH )
		self.osdObj:SetAreaText( areaId, text )
		for flag in text:gmatch("(\n)") do
			lineCnt = lineCnt + 1
		end
		thisAreaW, thisAreaH = self.osdObj:TellAreaSize( areaId )
		thisAreaH = (lineCnt+1)*screenGridH
		--Should return back restricted size.
		thisAreaW, thisAreaH = self.osdObj:ChangeAreaSize( areaId, thisAreaW, thisAreaH )
	else
		thisAreaW, thisAreaH, rgba = GetIMGSize(itemCfg.fileID)
		if not thisAreaW then
			_dbg.Error("OSD", "获取图片宽高失败！图片ID：" .. itemCfg.fileID)
			return
		end
		self.osdObj:ChangeAreaSize( areaId, thisAreaW, thisAreaH)
	end

	local x, y
	if itemCfg.position == "left-top" then
		x = xOffset
		y = yOffset
	elseif itemCfg.position == "top-center" then
		x = math.floor((self.width - thisAreaW)/2)
		y = yOffset
	elseif itemCfg.position == "right-top" then
		x = self.width - thisAreaW - xOffset
		y = yOffset
	elseif itemCfg.position == "left-bottom" then
		x = xOffset
		y = self.height - yOffset - thisAreaH
	elseif itemCfg.position == "bottom-center" then
		x = math.floor((self.width - thisAreaW)/2)
		y = self.height - yOffset - thisAreaH
	elseif itemCfg.position == "right-bottom" then
		x = self.width - thisAreaW - xOffset
		y = self.height - yOffset - thisAreaH
	elseif itemCfg.position == "center" then
		x = math.floor((self.width - thisAreaW)/2)
		y = math.floor((self.height - thisAreaH)/2)
	else
		local posX = itemCfg.x or itemCfg.customX or 0
		local posY = itemCfg.y or itemCfg.customY or 0
		-- x = xOffset + posX * screenGridW
		-- y = yOffset + posY * screenGridH
		x = posX
		y = posY
	end

	if osd_type ~= "image" then
		self.osdObj:SetAreaPositionType( areaId, itemCfg.position )
	end

	self.osdObj:SetAreaPosition( areaId, x, y )

	if osd_type == "image" then
		self.osdObj:SetBackGroundFlag(areaId, false)
		local is_ok, msg = self.osdObj:LoadFile(areaId, {
			filename = rgba,
			w = thisAreaW,
			h = thisAreaH,
		})
	end
end

function osdgroup_meta:UpdateOSD(itemId)
	local glbCfg = _G.osd_cfg.channels[ self.streamId ].config
	local i
	for i=1, MAX_AREAS do
		if not itemId or itemId == i then
			local itemCfg = _G.osd_cfg.channels[ self.streamId ].items[i]
			if itemCfg then
				self:UpdateOSDArea( i-1, glbCfg, itemCfg )
			end
		end
	end

	self.osdObj:Update()
end

osdgroup_meta.__index = osdgroup_meta

----------------------------------------------

function _M.Init()
	_M.mo_CodecChannel = require "codec"
	-- mod_license.Init()
end

----------------------------------------------

function _M.Destroy()
	_M.FreeOSDs()
end

----------------------------------------------
local pt = require "pl.pretty"
--NOTE:这里只是为了拿通道宽高
local function whenResolutionChanged(chn)
	local codecChan = _M.mo_CodecChannel
	local osdChan = _M.channels[chn]
	if not codecChan or not osdChan then
		_dbg.Error( "OSD", "OnResolutionChanged: Fail to get CODEC or OSD channel of " .. chn )
		return
	end

	local osdDetails = codecChan:getVpssAndPicInfo()[chn]
	if not osdDetails then
		_dbg.Error( "OSD", "OnResolutionChanged: Fail to get CODEC channel's OSD detail for channel " .. chn )
		return
	end

	local picW, picH
	for i, item in ipairs(osdDetails) do
		local j, fbGroup
		for j,fbGroup in ipairs(osdChan.osdGroups) do
			if fbGroup.flag == item.flag then
				if item.type == "fb" then
					--picW = item.picFormat.width
					--picH = item.picFormat.height
				elseif item.type == "region" then
					if item.target == "output" then
						--picW = item.outObj.syncDetail.width
						--picH = item.outObj.syncDetail.height
					elseif item.target == "encoder" then
						picW = item.encObj.u32PicWidth == 1 and _G.capture_cfg[1].stCapRect[3] or item.encObj.u32PicWidth
						picH = item.encObj.u32PicHeight == 1 and _G.capture_cfg[1].stCapRect[4] or item.encObj.u32PicHeight
					else
						break
					end
				else
					break
				end

				fbGroup.width = picW
				fbGroup.height = picH
				fbGroup.osdObj:ChangeSize(picW,picH)
				fbGroup:UpdateOSD()
				break
			end
		end
	end
end

function _M.ResolutionChanged(chn)
	whenResolutionChanged(chn)
end

function _M.ConstructOSDs(chnId)
	local chn
	for chn=1, 2 do
		if not chnId or chnId == chn then
			local tbl = _M.mo_CodecChannel.getVpssAndPicInfo()
			local osdDetails = tbl[chn]
			if not osdDetails then
				_dbg.Error( "OSD", "Fail to get CODEC channel's OSD detail for channel " .. chn )
			else
				--NOTE: 此处好像不需要
				-- codecChan:OnResolutionChanged( whenResolutionChanged, chn )
				local itemCnt = 0
				for i, item in ipairs(osdDetails) do
					if item.type == "fb" then
						-- local fbOsd = require( "mod_OSD_adapter" ).createNew("fb", 
						-- 											{ 	device=item.device, 
						-- 												width = item.picFormat.width, 
						-- 												height = item.picFormat.height 
						-- 											})
						-- if not fbOsd then
						-- 	_dbg.Error( "OSD", "Fail to create FB osd instance for channel " .. chn )
						-- else
						-- 	_dbg.Info( "OSD", "Created FB osd instance for channel " .. chn )
						-- 	local fbGroup = {
						-- 		streamId = chn,
						-- 		osdObj = fbOsd,
						-- 		flag = item.flag,
						-- 		vs = _M.vs --vscreen.New()
						-- 	}
						-- 	setmetatable( fbGroup, osdgroup_meta )
						-- 	fbGroup:Init( item.picFormat.width, item.picFormat.height )
						-- 	table.insert( _M.channels[chn].osdGroups, fbGroup )
						-- end
					elseif item.type == "region" then
						local rgOsd 
						local picW, picH
						if item.target == "output" then
							-- picW = item.outObj.syncDetail.width
							-- picH = item.outObj.syncDetail.height
							-- if item.vpssChn then
							-- 	rgOsd = require("mod_OSD_adapter").createNew( "hi_region", 
							-- 				{ 	overlay_type = "OVERLAYEX", 
							-- 					channel = item.vpssChn, 
							-- 					base_id = (chn-1)*32 + itemCnt*8,
							-- 					width = picW,
							-- 					height = picH
							-- 				}
							-- 			)
							-- else
							-- 	rgOsd = require("mod_OSD_adapter").createNew( "hi_region", 
							-- 				{ 	overlay_type = "OVERLAYEX", 
							-- 					channel = item.outObj.voChannel, 
							-- 					base_id = (chn-1)*32 + itemCnt*8,
							-- 					width = picW,
							-- 					height = picH
							-- 				}
							-- 			)
							-- end
						elseif item.target == "encoder" then
							picW = item.encObj.u32PicWidth
							picH = item.encObj.u32PicHeight
							if picW == 1 then
								if chn == 1 then
									picW = _G.capture_cfg[1].stCapRect[3]
									picH = _G.capture_cfg[1].stCapRect[4]
								else
									picW = math.min(_G.capture_cfg[1].stCapRect[3], 1920)
									picH = math.min(_G.capture_cfg[1].stCapRect[4], 1080)
								end
							end
							rgOsd = require("mod_OSD_adapter").createNew( "hi_region", 
										{ 	overlay_type = "OVERLAY", 
											channel = item.encObj.vencChannel,
											base_id = (chn-1)*32 + itemCnt*8,
											width = picW,
											height = picH
										}
									)
						end
						
						if not rgOsd then
							_dbg.Info( "OSD", "Fail to create REGION osd instance for channel", chn, ", target=", item.target, "note that this may be not a serious problem, if the region is duplicated." )
						else
							itemCnt = itemCnt + 1
							_dbg.Info( "OSD", "Created REGION osd instance for channel ", chn, ", target=", item.target )
							local rgGroup = {
								streamId = chn,
								flag = item.flag,
								osdObj = rgOsd,
								-- vs = _M.vs --vscreen.New()
							}
							setmetatable( rgGroup, osdgroup_meta )
							rgGroup:Init( picW, picH )
							table.insert( _M.channels[chn].osdGroups, rgGroup )
						end
					elseif item.type == "remote" then
						-- _M.channels[chn].withRemote = true
						-- if not _M.channels[chn].remoteCli then
						-- 	_M.channels[chn].remoteCli = client.New( "codec", "CO" )
						-- end
						-- if _M.channels[chn].remoteCli then
						-- 	_dbg.Info( "OSD", "Created remote controller client for channel " .. chn )
						-- 	_M.channels[chn].remoteProxy = _M.channels[chn].remoteCli:CreateProxy("/config/osd")
						-- else
						-- 	_dbg.Error( "OSD", "Fail to create remote controller client for channel " .. chn .. ", it should create later" )
						-- end
					else
						_dbg.Error( "OSD", "Unsupported OSD type of ", item.type, " for channel " .. chn )
					end
				end --For each OSD item.
			end --if got osd details
		end
	end --for Codec channels enum.

	for chn = 1, 2 do
		_M.UpdateOSDConfig(chn)
	end
	_M.initialized = true
end

function _M.FreeOSDs(chnId)
	local chn
	for chn=1,2 do
		if not chnId or chnId == chn then
			local j, group
			for j,group in ipairs(_M.channels[chn].osdGroups) do
				if group then
					group:Destroy()
				end
			end
			_M.channels[chn].osdGroups = {}
	
			if _M.channels[chn].remoteProxy then
				_M.channels[chn].remoteProxy:Destroy()
				_M.channels[chn].remoteProxy = nil
			end
			if _M.channels[chn].remoteCli then
				_M.channels[chn].remoteCli:Destroy()
				_M.channels[chn].remoteCli = nil
			end
			_M.channels[chn].withRemote = false
		end
	end
	_M.initialized = false
end

function _M.UpdateOSDConfig( chnId )
	local chn
	for chn=1,2 do
		if not chnId or chnId == chn then
			for j,group in ipairs(_M.channels[chn].osdGroups) do
				group:UpdateOSD()
			end
		end
	end
end

local default_item = {
	type = "none",
	position = "left-top",
	customX = 0,
	customY = 0,
	size = 0,
	style = "normal",
	outline = 1,
	color = 1,
	outlineColor = 0,
	text = "",
	fileID = "",
	enable = false,
}

local function GetPicUrl(fileID)
	local proxy = GetProxy("/pics", "picManager")
	if not proxy then
		_dbg.Error("Cannot create picManager proxy in line 540")
		return "", ""
	end
	local skip, _, _, _, picUrl, name = proxy.GetIMGDesc(fileID)
	proxy:Destroy()
	if skip then
		return picUrl, name
	end
	return "", ""
end

--@@ 更新某个叠加项
function _M.UpdateOSDItem(chnId, idx, cfg)
	chnId = tonumber(chnId)
	if not chnId or chnId > 2 or chnId < 1 then
		return false
	end
	if cfg.type == "image" then
		if cfg.fileID ~= "" then
			local url = GetPicUrl(cfg.fileID)
			if url == "" or not url then
				return false, "invalidPicID"
			end
		else
			return false, "invalidPicID"
		end
	end
	table.merge(_G.osd_cfg.channels[chnId].items[idx], cfg)
	_G.osd_cfg:__TRIGGER(true)
	return true
end

--@@ 更新主码流或者子码流的整体配置
function _M.UpdateChannel(chnId, cfg)
	chnId = tonumber(chnId)
	if not chnId or chnId > 2 or chnId < 1 then
		return false
	end
	table.update(_G.osd_cfg.channels[chnId].config, cfg)
	_G.osd_cfg:__TRIGGER(true)
	return true
end

function _M.AddOSDItem(chnId, cfg)
	chnId = tonumber(chnId)
	if not chnId or chnId > 2 or chnId < 1 then
		return false
	end

	if #(_G.osd_cfg.channels[chnId].items) + 1 > MAX_AREAS then
		return false
	end

	local copy = table.copy(default_item)
	table.update(copy, cfg)
	if copy.type == "image" then
		if copy.fileID ~= "" then
			local url = GetPicUrl(copy.fileID)
			if url == "" or not url then
				return false, "invalidPicID"
			end
		else
			return false, "invalidPicID"
		end
	end
	table.insert(_G.osd_cfg.channels[chnId].items, copy)
	_G.osd_cfg:__TRIGGER(true)
	return true
end

function _M.DelOSDItem(chnId, idx)
	chnId = tonumber(chnId)
	if not chnId or chnId > 2 or chnId < 1 then
		return false
	end

	local group = _M.channels[chnId].osdGroups[1]
	if not group then
		return true
	end

	group.osdObj:DropArea(idx - 1)
	table.remove(_G.osd_cfg.channels[chnId].items, idx)
	_G.osd_cfg:__TRIGGER(true)
	return true
end

function _M.GetInfo(chnId)
	local result
	if chnId then
		result = table.copy(_G.osd_cfg.channels[chnId])
		for _, v in ipairs(result.items) do
			if v.type == "image" and v.fileID then
				v.picUrl, v.name = GetPicUrl(v.fileID)
			end
		end
	else
		result = table.copy(_G.osd_cfg.channels)
		for _, vv in ipairs(result) do
			for _, v in ipairs(vv.items) do
				if v.type == "image" and v.fileID then
					v.picUrl, v.name = GetPicUrl(v.fileID)
				end
			end
		end
	end
	return result
end

function _M.RemoveOSDPic(id)
	local remove = {}
	for chnId, vv in ipairs(_G.osd_cfg.channels) do
		remove[chnId] = remove[chnId] or {}
		for index, v in ipairs(vv.items) do
			if v.type == "image" and v.fileID == id then
				if vv.enable and v.enable then
					return false
				else
					table.insert(remove[chnId], 1, index)
				end
			end
		end
	end

	for chnId, vv in ipairs(remove) do
		for _, index in ipairs(vv) do
			table.remove(_G.osd_cfg.channels[chnId].items, index)
		end
	end
	_G.osd_cfg:__TRIGGER(true)
	return true
end

_M.doc = ""
_M.Rule = {
	RemoveOSDPic = {
		is_open = true,
		parameters = {
			{key = "id", type = "number", must = true},
		},
	},
	UpdateOSDItem = {
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "idx", type = "number", must = true},
			{key = "cfg", type = "table", must = true},
		},
		is_open = true,
	},
	UpdateChannel = {
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "cfg", type = "table", must = true},
		},
		is_open = true,
	},
	AddOSDItem = {
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "cfg", type = "table", must = true},
		},
		is_open = true
	},
	DelOSDItem = {
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "idx", type = "number", must = true},
		},
		is_open = true
	},
	GetInfo = {
		parameters = {
			{key = "chnId", type = "number"},
		},
		is_open = true
	}
}

return _M

