local codec = require "codec"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local streams = require "mod_Streams"
local event = require "coaf.event"
package.path = package.path .. ";/usr/local/bin/network/?.lua;"
local funcs = require "nm_funcs"
local mod_audioEnc = require "mod_audioEnc"

require "kmCore"

local rtsp_pusher = {} --存放 RTSP 创建的 pusher，避免重复创建
local MAX_SUPPORT_STREAMS = 8
local NDIWebUrlTimer

local _M = {
    main = {},
    sub = {},
    preview = {},
    NDITallyStatus = {},
    initialized = {
        main = false, sub = false, preview = false,
    },
}

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)
local function GetProxy(objPath, service, loc, timeout, addr)
	local proxy
	local cli

	cli = client.New( service, loc, addr, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local old_ip = ""
local function UpdateNDIWebUrl(kmObj)
    local ip_info = funcs.GetLinkDetail("eth0")
	if not ip_info then
		_dbg.Info("NDI", "GetLinkDetail ERROR")
		return
	end

    local proxy = GetProxy("/webDaemon", "systemctrl")
    if proxy then
        local is_ok, port_cfg = proxy.GetInfo()
        ip_info.address = ip_info.address .. ":" .. port_cfg.port
        proxy:Destroy()
    end

    local ip = ip_info.address 
	if not ip or ip == "" then
		_dbg.Info("NDI", "can not get ip")
		return
	end

	if ip == old_ip then
		return
	end

    kmObj:SetWebUrl("http://" .. ip)
    old_ip = ip
end

local tally_changed = false
local leds_dict = {
    red = "off", green = "off"
}
local function SetTally()
    coaf.ScheduleDelayedTask(function()
        if not tally_changed then
            return
        end
        tally_changed = false
        local ZynqProxy = GetProxy("/io_leds", "codec", "CO")
        if not ZynqProxy then
            _dbg.Error("ZYNQLOG", "Can't create zynq proxy! Set tally led failed!")
            return
        end

        for led, status in pairs(leds_dict) do
            local is_ok = ZynqProxy.TallyLightControl(0, led, status)
            if not is_ok then
                _dbg.Error("ZYNQLOG", "Call TallyLightControl failed! ")
            end
        end
        ZynqProxy:Destroy()
    end, 0.01)
end

local lock = {}
local function StartNDIEvent(kmObj)
    if not _M.NDITallyTicket then
        _M.NDITallyTicket = _G.streamer_event:Subscribe("SetTallyLED", function(_, tbl)
            _M.NDITallyStatus[tbl.led] = tbl.state
            local old_red, old_green = leds_dict.red, leds_dict.green
            if _M.NDITallyStatus.TALLY == "on" then
                if _M.NDITallyStatus.MAIN == "on" then
                    leds_dict.red = "on"
                    leds_dict.green = "off"
                else
                    leds_dict.red = "off"
                    leds_dict.green = "on"
                end
            else
                leds_dict.red = "off"
                leds_dict.green = "off"
            end
            if old_red ~= leds_dict.red or old_green ~= leds_dict.green then
                tally_changed = true
            end
            SetTally()
        end)
    end

    if not _M.NDIIframeTicket then
        _M.NDIIframeTicket = _G.streamer_event:Subscribe("UpdateIFrame", function(_, tbl)
            local stream = tbl.stream
            if lock[stream] or lock.all then
                _dbg.Error("NDIEvent", "Lock and return !")
                return
            end
            lock[stream] = true
            local mainVenc = codec.enc_dict.main
            local subVenc = codec.enc_dict.sub
            if stream == "all" or stream == "main" then
                local skip = mainVenc.vencChannel:RequestIDR(true)
                -- _dbg.Info("NDIEvent", "Request Main Stream IDR:",skip)
            end

            if stream == "all" or stream == "sub" then
                local skip = subVenc.vencChannel:RequestIDR(true)
                -- _dbg.Info("NDIEvent", "Request Main Stream IDR:",skip)
            end
            lock[stream] = nil
        end)
    end

    NDIWebUrlTimer = coaf.NewTimer( function()
        UpdateNDIWebUrl(kmObj)
    end, 0.01, 10)
    if NDIWebUrlTimer then NDIWebUrlTimer:start() end
end

local function StopNDIEvent()
    if _M.NDITallyTicket then
        _G.streamer_event:Unsubscribe(_M.NDITallyTicket)
        _M.NDITallyTicket = nil
    end

    if _M.NDIIframeTicket then
        _G.streamer_event:Unsubscribe(_M.NDIIframeTicket)
        _M.NDIIframeTicket = nil
    end

    if NDIWebUrlTimer then 
        NDIWebUrlTimer:stop()
        NDIWebUrlTimer = nil
        old_ip = ""
    end
end

local function checkStreamConflict(config, skip, stream, id)
    for s, v in pairs(_G.streamer_cfg) do
        for i, vv in ipairs(v) do
            if not (s == stream and id and id == i) then
                if config.type == "rtsp" and config.port and vv.port == config.port --RTSP
                or config.type == "SRT" and vv.listenerPort == config.listenerPort and vv.connectionMode == config.connectionMode and config.listenerPort and config.connectionMode --SRT
                or (skip and vv.type == "ndi_hx" and config.type == "ndi_hx") --ndihx
                or (config.type == "hls" and vv.type == "hls" and (config.session == vv.session or not config.session or config.session == "")) -- hls
                then
                    return s, i
                end
            end
        end
    end

    if config.type == "SRT" and config.listenerPort < 1024 then
        return true
    end
end

local function FetchStream(cfg, old)
    if cfg.type == "ndi_hx" then
        return _M.NDIService
    end
    if cfg.type ~= "rtsp" then
        return old
    end

    if rtsp_pusher[cfg.port] 
        --and rtsp_pusher[cfg.port].counter < 4 --NOTE:如果这里开放，可能会重置 rtsp_puhser[cfg.port]
    then
        rtsp_pusher[cfg.port].counter = rtsp_pusher[cfg.port].counter + 1
        return rtsp_pusher[cfg.port].stream
    end
end

local function DellocateRTSPServer(port)
	if rtsp_pusher[port] then
		if rtsp_pusher[port].counter > 0 then
			rtsp_pusher[port].counter = rtsp_pusher[port].counter - 1
		end
	end
end

local function createSyncSwitch(stream, audioId, skipSync)
    local encSource = codec.hiEncSource_dict[stream]
    if not encSource then
        _dbg.Error("STREAMER", "Can’t get encSource! stream:", stream)
        return
    end
    local audioSource
    if audioId then
        audioSource = mod_audioEnc.channels[audioId]
    end

    local mediaSW = KM_STREAMING.SwitchSource.createNew()
    mediaSW:SelectTrack("video", encSource:GetMediaSource())
    if audioSource then
        mediaSW:SelectAudioEncoder( audioSource:GetEncoder() )
    end
 
    local afterSwSync
    if not skipSync then
        afterSwSync = KM_STREAMING.SyncSource.createNew()
        afterSwSync:SetBypass()
        afterSwSync:BindMediaSource(mediaSW:GetMediaSource())
    else
        afterSwSync = mediaSW
    end
    return afterSwSync, mediaSW
end

local stream_meta = {}
function stream_meta:Init(id, stream, cfg, audioChange)
    if not id or not cfg or not stream then
        _dbg.Error("Init stream failed, not have id or cfg", id, cfg, stream)
        return
    end

    if self.status and (self.status ~= "destroy" and self.status ~= "stop" and self.status ~= "error") then
        _dbg.Error("Init stream failed, already Init and never destroy or stop, stream = ", stream, "id = ", id)
        return
    end

    self.id = id
    self.bindStream = stream
    self.config = cfg

    if not cfg.enable then
        self.status = "destroy"
        return true
    end

    if audioChange or not self.mediaSource then
        if self.mediaSW then
            self.mediaSW:SelectAudioNull()
        end
        local syncSource, syncSource_sub
        syncSource, self.mediaSW = createSyncSwitch(stream, self.config.bindAudio)
        if not syncSource then
            _dbg.Error("Init stream failed, not init codec before, can not get syncSource.")
            return
        end
        self.mediaSource = syncSource:GetMediaSource()
        if self.config.type == "ndi_hx" then
            syncSource_sub, self.mediaSW_sub = createSyncSwitch("sub", self.config.bindAudio)
            self.mediaSource_sub = syncSource_sub:GetMediaSource()
        end
    end

    if self.config.type ~= "rtsp" then
        self.group = stream .. id
    end

    local fetech_stream = FetchStream(self.config, self.stream)
    local streamObj
    streamObj, self.sessionId, self.targetAddr = streams.createStream(self.config, self.mediaSource, self.group, fetech_stream, self)
    self.stream = streamObj

    if not self.stream then
        self.status = "error"
        return
    end

    if not fetech_stream then
        if cfg.type == "rtsp" then
            rtsp_pusher[cfg.port] = {
                counter = 1,
                stream = streamObj
            }
        elseif cfg.type == "ndi_hx" then
            _M.NDIService = streamObj
        end
    end

    self:Start()
    mod_audioEnc.UpdateDispatcherRef(self.config.bindAudio, self.mediaSW_sub and 2 or 1)
    return true
end

function stream_meta:Start()
    if self.status ~= "start" and self.stream then
        if self.stream.StartSession then
            self.stream:StartSession(self.sessionId)
        elseif self.stream.Start then --RTSP
            self.stream:Start()
            -- self:DoSpecialFunc("EnableAuth", self.config.auth)
        elseif self.stream.StartAllSessions then
            self.stream:StartAllSessions()
            self.stream:HandleEvent("streamer", "SetTallyLED")
            self.stream:HandleEvent("streamer", "UpdateIFrame")
            StartNDIEvent(self.stream)
        end
        self.status = "start"
    end
end

function stream_meta:Stop()
    if (self.status ~= "stop" and self.status ~= "destroy") and self.stream then
        if self.config.type == "rtsp" then
            DellocateRTSPServer(self.config.port)
            self.stream:RemoveSession(self.config.session)
        elseif self.config.type == "ndi_hx" then
            -- self.stream:RemoveSession(self.bindStream == "main" and 0 or 1)
            self.stream:StopAllSessions()
            StopNDIEvent()
        else
            self.stream:RemoveSession(self.sessionId)
        end

        mod_audioEnc.UpdateDispatcherRef(self.config.bindAudio, self.mediaSW_sub and -2 or -1)
        self.status = "stop"
    end
end

function stream_meta:Destroy()
    if self.status == "destroy" then
        _dbg.Info("Streamer", "Not need to Destroy, Already Destroy")
        return
    end

    if self.stream then
        if self.config.type == "rtsp" then
            DellocateRTSPServer(self.config.port)
            self.stream:RemoveSession(self.config.session)
        elseif self.config.type == "ndi_hx" then
            -- self.stream:RemoveSession(self.bindStream == "main" and 0 or 1) 
            self.stream:StopAllSessions()
            self.stream:RemoveAllSessions()
            StopNDIEvent()
        else
            self.stream:RemoveSession(self.sessionId)
        end
        self.mediaSW:SelectAudioNull()
        if self.mediaSW_sub then
            self.mediaSW_sub:SelectAudioNull()
        end

        mod_audioEnc.UpdateDispatcherRef(self.config.bindAudio, self.mediaSW_sub and -2 or -1)
    end

    self.sessionId = nil
    self.mediaSource = nil
    self.mediaSource_sub = nil
    self.status = "destroy"
    collectgarbage()
end

function stream_meta:Update(cfg)
    self:Stop()

    if checkStreamConflict(cfg, nil, self.bindStream, self.id) then
        return false, "conflictError"
    end

    local audioChange = cfg.bindAudio and cfg.bindAudio ~= self.config.bindAudio

    local default = streams.defaultStreamCfg[self.config.type:lower()]
    for k, v in pairs(default) do
        if self.config[k] == nil then
            self.config[k] = v
        end
    end
    local old = table.copy(self.config)
    table.update(self.config, cfg)

    if not self:Init(self.id, self.bindStream, self.config, audioChange) then
        self.config = old
        return false, "argumentError"
    end

    table.update(_G.streamer_cfg[self.bindStream][self.id], cfg)
    _G.streamer_cfg:__TRIGGER(true)
    return true
end

function stream_meta:GetInfo()
    local tbl = table.copy(self.config)
    tbl.status = self.status
    if self.status == "start" then
        if tbl.type == "rtsp" then
            tbl.addressUrl = string.format( "rtsp://%%HOST%%:%s/%s", tbl.port, tbl.session)
        elseif tbl.type == "SRT" then
            if tbl.connectionMode ~= "" and tbl.listenerPort > 0 then
                if tbl.address == "" then
                    tbl.addressUrl = string.format( "srt://%%HOST%%:%s", tbl.listenerPort)
                else
                    tbl.addressUrl = string.format( "srt://%s:%s", tbl.address, tbl.listenerPort)
                end
            end
        elseif tbl.type == "rtmp" then
            tbl.addressUrl = self.targetAddr
        elseif tbl.type == "hls" then
            tbl.addressUrl = string.format("http://%%HOST%%/hls/%s/playlist.m3u8", tbl.session)
        elseif tbl.type == "ts" then
            tbl.addressUrl = string.format("udp://%s:%s", tbl.address, tbl.port)
        else
            tbl.addressUrl = ""
        end
    else
        tbl.addressUrl = "offline"
    end
    return tbl
end

function stream_meta:DoSpecialFunc(name, ...)
    if not self.stream or not self.stream[name] or type(self.stream[name]) ~= "function" then
        _dbg.Warn("STREAMER", "No the special function:", name, "streamer: ", self.type)
        return
    end

    return self.stream[name](self.stream, ...)
end

stream_meta.__index = stream_meta

function _M.Init(skip)
    for stream, list in pairs(_G.streamer_cfg) do
        -- if not _M.initialized[stream] then
            -- _M.initialized[stream] = true
            if not skip or stream == skip then
                for id, config in ipairs(list) do
                    if id <= MAX_SUPPORT_STREAMS then
                        local old = _M[stream][id]
                        if not old then
                            _M[stream][id] = setmetatable({}, stream_meta)
                        end
                        _M[stream][id]:Init(id, stream, config)
                    end
                end
            end
        -- end
    end
    if not _M.preview.syncSource then
        _M.preview.syncSource = createSyncSwitch("preview")
    end
end

function _M.UpdateStreamer(stream, id, cfg)
    local meta = _M[stream] and _M[stream][id]
    if not meta then
        return false, "argumentError"
    end

    if _M.UpdateingStreamer then
        return false, "busyError"
    end
    _M.UpdateingStreamer = true

    cfg.type = nil
    local is_ok = meta:Update(cfg)
    _M.UpdateingStreamer = nil
    return is_ok, not is_ok and "modifyError"
end

function _M.AddStreamer(stream, cfg)
    local id = #(_G.streamer_cfg[stream] or {}) + 1
    if not _M[stream] or id > MAX_SUPPORT_STREAMS then
        _dbg.Error("AddStreamer failed, not have stream:", stream, "or id not in maxN, id:", id)
        return false, "limitError"
    end

    local config = streams.GetDefaultStreamConfig(cfg)
    if not config then
        _dbg.Error("Streamer", "AddStreamer failed, can not get config.")
        return false, "invalidTypeError"
    end

    config.name = cfg.name
    config.type = cfg.type
    config.enable = cfg.enable or false
    config.bindAudio = cfg.bindAudio or 1
    if not mod_audioEnc.channels[config.bindAudio] then
        return false, "invalidAudioIDError"
    end
    if checkStreamConflict(config, true, stream) then
        return false, "conflictError"
    end

    table.insert(_G.streamer_cfg[stream], config)

    _M[stream][id] = setmetatable({}, stream_meta)
    _M[stream][id]:Init(id, stream, config)
    _G.streamer_cfg:__TRIGGER(true)
    
    return true
end

function _M.DeleteStreamer(stream, id)
    if not _M[stream] or not _M[stream][id] then
        return true
    end

    if id == 1 and _M[stream][id].config.type == "ndi_hx" then
        return false, "deleteStreamError"
    end

    _M[stream][id]:Destroy()
    table.remove(_M[stream], id)

    table.remove(_G.streamer_cfg[stream], id)
    _G.streamer_cfg:__TRIGGER(true)
    return true
end

function _M.GetHLSM3U8(groupId)
    -- local stream, idx = groupId:match("(%a+)(%d+)")
    -- idx = tonumber(idx)
    -- if not stream or not idx or not _M[stream] or not _M[stream][idx] or not _M[stream][idx].fileInfo then
    --     return
    -- end

    local meta
    for _, vv in ipairs({_M.main, _M.sub}) do
        for idx, v in ipairs(vv) do
            if v.createSession == groupId then
                meta = v
                break
            end
        end
    end

    if not meta then
        return
    end

    local files = meta.fileInfo.files

    if not files or #files < 1 then
        return
    end

    local result = {
        "#EXTM3U",
		"#EXT-X-VERSION:3",
		"#EXT-X-ALLOW-CACHE:YES",
		"#EXT-X-MEDIA-SEQUENCE:", --Concat later.
		"#EXT-X-TARGETDURATION:", --Concat later.
    }

    local seq = files[1].sequence or 0
    result[4] = result[4] .. seq
    if seq == 0 then
        table.insert(result, "#EXT-X-DISCONTINUITY")
    end

    local max_dur = 0
    for i = 1, #files do
        table.insert(result, "#EXTINF:" .. math.floor(files[i].duration/1000) .. "." .. string.format("%03d", files[i].duration%1000))
        max_dur = math.max(files[i].duration, max_dur)
		table.insert(result, files[i].file)
    end

    if max_dur <= 0 or max_dur < meta.config.segmentTime then
		max_dur = meta.config.segmentTime
	end
    result[5] = result[5] .. math.ceil(max_dur / 1000)
    return result
end

function _M.Destroy(stream, audioId)
    if stream == "preview" then
        return
    end
    local list = {}
    if not stream then
        list = {"main", "sub"}
    else
        table.insert(list, stream)
    end

    for _, v in ipairs(list) do
        for _, meta in pairs(_M[v]) do
            if not audioId or meta.config.bindAudio == audioId then
                meta:Destroy()
            end
        end
        -- _M.initialized[v] = false
    end

    collectgarbage()
    return true
end

function _M.UpdateRTSPUser(index, username, password)
    local meta = _G.media_users[index]
    if not meta then
        return false, "modifyUserError"
    end

    for _, vv in ipairs({_M.main, _M.sub}) do
        for idx, v in ipairs(vv) do
            if v.type == "rtsp" then
                v.stream:DelAuth(username)
                v.stream:AddAuth(username, password)
            end
        end
    end

    meta.username = username
    meta.password = password
    _G.media_users:__TRIGGER(true)
    return true
end

function _M.AddRTSPUser(username, password)
    if #_G.media_users >= 16 then
        return false, "createUserError"
    end

    for i, v in ipairs(_G.media_users) do
        if v.username == username then
            return false, "createUserError"
        end
    end

    for _, vv in ipairs({_M.main, _M.sub}) do
        for idx, v in ipairs(vv) do
            if v.config.type == "rtsp" then
                v.stream:AddAuth(username, password)
            end
        end
    end

    table.insert(_G.media_users, {
        username = username,
        password = password
    })

    _G.media_users:__TRIGGER(true)
    return true
end

function _M.DeleteRTSPUser(index)
    if not _G.media_users[index] then
        return false, "delUserError"
    end

    for _, vv in ipairs({_M.main, _M.sub}) do
        for idx, v in ipairs(vv) do
            if v.config.type == "rtsp" then
                v.stream:DelAuth(_G.media_users[index].username)
            end
        end
    end
    table.remove(_G.media_users, index)
    _G.media_users:__TRIGGER(true)
    return true
end

function _M.GetInfo(stream, id)
    if _M[stream] and _M[stream][id] then
        return _M[stream][id]:GetInfo()
    end

    local tbl = { main = {}, sub = {} }
    for stream, v in pairs({main = _M.main, sub = _M.sub}) do
        for id, vv in ipairs(v) do
            table.insert(tbl[stream], vv:GetInfo())
        end
    end
    return tbl
end

function _M.GetMediaUsers()
    return _G.media_users
end

function _M.GetStreamObject(stream, bindAudio)
    if _M[stream] and _M[stream].syncSource then
        return _M[stream].syncSource:GetMediaSource()
    end
    local x = createSyncSwitch(stream, bindAudio)
    return x:GetMediaSource()
end

function _M.ReStartNDI()
    for id, meta in pairs(_M.main) do
        if meta.config.type == "ndi_hx" then
            meta:Stop()
            meta:Init(meta.id, meta.bindStream, meta.config)
            break
        end
    end
end

_M.doc = ""
_M.Rule = {
    GetHLSM3U8 = {
        is_open = true,
        parameters = {
            {key = "groupId", type = "string", must = true},
        }
    },
    GetMediaUsers = {
        is_open = true,
        parameters = {}
    },
    GetInfo = {
        is_open = true,
        parameters = {key = "id", type = "number"}
    },
    AddStreamer = {
        parameters = {
            {key = "stream", type = "string", must = true},
            {key = "cfg", type = "table", must = true},
        },
        is_open = true
    },
    DeleteStreamer = {
        parameters = {
            {key = "stream", type = "string", must = true},
            {key = "id", type = "number", must = true},
        },
        is_open = true
    },
    UpdateStreamer = {
        parameters = {
            {key = "stream", type = "string", must = true},
            {key = "id", type = "number", must = true},
            {key = "cfg", type = "table", must = true},
        },
        is_open = true
    },
    UpdateRTSPUser = {
        is_open = true,
        parameters ={
            {key = "index", type = "number", must = true},
            {key = "username", type = "string", must = true},
            {key = "password", type = "string", must = true},
        }
    },
    AddRTSPUser = {
        is_open = true,
        parameters ={
            {key = "username", type = "string", must = true},
            {key = "password", type = "string", must = true},
        }
    },
    DeleteRTSPUser = {
        is_open = true,
        parameters ={
            {key = "index", type = "number", must = true},
        }
    },
}

return _M