require "himpp"
require "kmCore"

local coaf = require "coaf.core"
local ain = require "soc.ain"
local _dbg = require "coaf.debug"
local run_daemon = require "coaf.run_daemon" 
local mod_Streamer, mod_record

local MAX_CHANNELS = 4
local idx = 1

local _M = {
    channels = {},
    dispatcher = {},
    dispatcher_ref = {},
    aiChannel = {},
    outSink = {},
    audioCrossEncChannel = {},
    audioEng = {}
}

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)

local function GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 5 ) --5 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local function createCommon(aiPubAttr, idx)
    _M.aiChannel[idx] = ain.CreateChannel(idx)
    if not _M.aiChannel[idx] then
        _dbg.Error( "DISPATCHER", "Fail to create channel for AI !!!")
        return
    end

    if not _M.aiChannel[idx]:SetAudioOption(aiPubAttr) then
        return
    end
    _M.aiChannel[idx]:Start()

    _M.audioEng[idx] = KM_STREAMING.AudioEngine.Encoder.createNew()
    if not _M.audioEng[idx] then
        _dbg.Error("DISPATCHER", "Cannot create audioEng")
        return 
    end

    _M.audioCrossEncChannel[idx] = _M.audioEng[idx]:CreateEncodingChannel( { type="HISIAI", device= tostring(idx - 1) }, { type="CROSS-PCM", bufferSize = 512000, channels=2 } )
    if not _M.audioCrossEncChannel[idx] then
        _dbg.Error( "DISPATCHER", "Fail to create audio encoding channel!" )
    else
        _M.audioCrossEncChannel[idx]:SetParameters({
            source_fixed_channels = aiPubAttr.u32ChnCnt,
            source_fixed_sample_size = aiPubAttr.u32PtNumPerFrm,
            source_fixed_sampling = aiPubAttr.enSamplerate,
            sampleSize = aiPubAttr.u32PtNumPerFrm,
            channels = aiPubAttr.u32ChnCnt,
            sampling = aiPubAttr.enSamplerate,
            bitrate = 64000
        })
    end

    _dbg.Info( "DISPATCHER", "audioEncChannel SetParameters~!" )

    _M.dispatcher[idx] = KM_STREAMING.AudioEngine.CreateAudioDispatcher( "NDI-Audio" .. idx )
    _M.dispatcher_ref[idx] = 0
    if not _M.dispatcher[idx] then
        _dbg.Error( "DISPATCHER", "Fail to create audio Dispatcher!" )
    end

    _M.dispatcher[idx]:Set(aiPubAttr.enSamplerate, aiPubAttr.u32ChnCnt, aiPubAttr.u32PtNumPerFrm, 16)
    
    if not _M.dispatcher[idx]:SelectAudioSourceAsInput( _M.audioCrossEncChannel[idx]:GetSource() ) then
        _dbg.Error( "DISPATCHER", "SelectInputSource: Fail to select audio source as input for dispatcher")
    end
    
    _M.outSink[idx] = KM_STREAMING.AudioEngine.CreateAudioSink( "CROSS", {buffer = 1024*1024} )
    if not _M.outSink[idx] then
        _dbg.Error( "DISPATCHER", "SelectInputSource: Fail to select audio source as input for dispatcher")
        return false
    end
    
    --把sink挂载到派发器的输出
    _M.dispatcher[idx]:AddOutputSink( _M.outSink[idx] )
    
    --_M.dispatcher[idx]:Start()

    _dbg.Info("DISPATCHER", "Create Dispatcher Success!!!!!!!")
end

local channel_meta = {}
function channel_meta:Init()
    if not _M.audioEng[self.channelMap] then
        _dbg.Error( "AudioEnc", "Fail to create audio engine!" )
        return
    end

    local attr
    local fixType = "AAC"
    if self.aacType == "AAC" then
        attr = {
            source_fixed_sampling = _G.aenc_cfg.source.enSamplerate,
			source_fixed_channels = _G.aenc_cfg.source.u32ChnCnt,
			source_fixed_sample_size = _G.aenc_cfg.source.u32PtNumPerFrm,
            type = "AAC",
            sampling = self.sampling,
            channels = self.channels,
            resample = self.resample,
            sampleSize = 1024,
            bitrate = self.bitrate
        }
    else
        fixType = "PCMU"
        attr = {
			source_fixed_sampling = _G.aenc_cfg.source.enSamplerate,
			source_fixed_channels = _G.aenc_cfg.source.u32ChnCnt,
			source_fixed_sample_size = _G.aenc_cfg.source.u32PtNumPerFrm,
			sampleSize = 480,
			g711Law = "PCMU",
			bitrate = self.bitrate,
            resample = self.resample,
            channels = 1,
            sampling = 8000
        }
    end

    self.audioEncChannel = _M.audioEng[self.channelMap]:CreateEncodingChannel(
        {type = "CROSS"}, {type = fixType, bufferSize = 512000, channels = attr.channels}
    )

    if not self.audioEncChannel then
        _dbg.Error( "AudioEnc", "Fail to create audio encoding channel!" )
        return
    end

    self.audioEncChannel:SetParameters(attr)

    KM_STREAMING.AudioEngine.ConnectSinkToSource(_M.outSink[self.channelMap], self.audioEncChannel:GetSource() )

    self.audioEncChannel:Start()
    _dbg.Info( "AudioEnc", "audioEncChannel Start~! id = ", self.id )
    return true
end

function channel_meta:DestroyEncChannel()
    if self.audioEncChannel then
        self.audioEncChannel:Stop()
        KM_STREAMING.AudioEngine.DisconnectSinkToSource(self.audioEncChannel:GetSource())
        self.audioEncChannel:Destroy()
        self.audioEncChannel = nil
        collectgarbage()
    end
end

function channel_meta:Destroy()
    mod_Streamer.Destroy(nil, self.id)
    self:DestroyEncChannel()
    collectgarbage()
end

function channel_meta:GetEncoder()
    if self.audioEncChannel then
        return self.audioEncChannel:GetEncoder()
    end
    return nil
end

function channel_meta:SetVqeVolume(vol)
    if self.aiChannel then
        self.aiChannel:SetVqeVolume(vol)
    end
end

function channel_meta:Update(cfg)
    if not self.audioEncChannel then
        return self:init()
    end

    local reCreate = false
    local resample, aacType, channelMap = self.resample, self.aacType, self.channelMap
    if cfg.resample ~= nil and cfg.resample ~= self.resample or cfg.aacType ~= nil and cfg.aacType ~= self.aacType
    or cfg.channels ~= nil and cfg.channels ~= self.channels or cfg.channelMap ~= nil and cfg.channelMap ~= self.channelMap then
        resample = cfg.resample or self.resample
        aacType = cfg.aacType or self.aacType
        channelMap = cfg.channelMap or self.channelMap
        reCreate = true
    end

    if reCreate then
        self:DestroyEncChannel()
    end

    local attr
    if aacType == "AAC" then
        attr = {
            aacType = aacType,
            source_fixed_sampling = _G.aenc_cfg.source.enSamplerate,
			source_fixed_channels = _G.aenc_cfg.source.u32ChnCnt,
			source_fixed_sample_size = _G.aenc_cfg.source.u32PtNumPerFrm,
            type = "AAC",
            sampling = cfg.sampling or self.sampling,
            channels = cfg.channels or self.channels,
            sampleSize = 1024,
            resample = resample,
            bitrate = cfg.bitrate or self.bitrate
        }
    else
        aacType = "PCMU"
        attr = {
            aacType = aacType,
			source_fixed_sampling = _G.aenc_cfg.source.enSamplerate,
			source_fixed_channels = _G.aenc_cfg.source.u32ChnCnt,
			source_fixed_sample_size = _G.aenc_cfg.source.u32PtNumPerFrm,
			sampleSize = 480,
			g711Law = cfg.g711Law or self.g711Law,
			bitrate = cfg.bitrate or self.bitrate,
            resample = resample,
            channels = 1,
            sampling = 8000
        }
        cfg.channels = 1
        cfg.sampling = 8000
    end

    if not self.audioEncChannel then
        self.audioEncChannel = _M.audioEng[channelMap]:CreateEncodingChannel(
            {type = "CROSS"}, {type = aacType, bufferSize = 512000, channels = attr.channels}
        )
    end

    if not self.audioEncChannel then
        _dbg.Error( "AudioEnc", "Fail to create audio encoding channel in function Update!" )
        return
    end

    self.audioEncChannel:SetParameters(attr)

    if reCreate then
        KM_STREAMING.AudioEngine.ConnectSinkToSource(_M.outSink[channelMap], self.audioEncChannel:GetSource() )
    end

    self.audioEncChannel:Start()

    return true
end

channel_meta.__index = channel_meta
--------------------------------------------------------------

function _M.CreateChannel(id)
    id = tonumber(id)
    if not id or id > MAX_CHANNELS then
        _dbg.Error("AudioEnc", "Faile to create audio channel, Invalid id:", id)
        return
    end

    if _M.channels[id] then
        return _M.channels[id]
    end

    local one = {
        id = id,
        vqeVolume = _G.aenc_cfg.audioEncode[id].vqeVolume or 0,
        resample = _G.aenc_cfg.audioEncode[id].resample or "fastest",
        sampleSize = _G.aenc_cfg.audioEncode[id].sampleSize or 1024,
        channels = _G.aenc_cfg.audioEncode[id].channels or 3,
        sampling = _G.aenc_cfg.audioEncode[id].sampling or 48000,
        aacType = _G.aenc_cfg.audioEncode[id].aacType or "AAC",
        aacFormat = _G.aenc_cfg.audioEncode[id].aacFormat or "RAW",
        bitrate = _G.aenc_cfg.audioEncode[id].bitrate or 64000,
        channelMap = _G.aenc_cfg.audioEncode[id].channelMap or 1
    }

    local r = setmetatable(one, channel_meta)
    _M.channels[id] = r

    return r
end

function _M.Init()
    run_daemon.Init()
    for i = 1, ain.MAX_CHANNELS do
        createCommon(_G.aenc_cfg.source, i)
    end

    local fixMap
    if _G.aenc_cfg.source.audioSource == "Line-IN" then
        -- fixMap = true
    end

    for id, v in ipairs(_G.aenc_cfg.audioEncode) do
        if fixMap then v.channelMap = 1 end
        local meta = _M.CreateChannel(id)
        meta:Init()
    end
    mod_Streamer = require "mod_Streamer"
    mod_record = require "mod_record"
	local task = run_daemon.Create({
		name = "InitAudioSelect",
		run = "/usr/local/bin/cocall.lua",
		args = {"CO", "codec", "/", "SelectAudioSource", _G.aenc_cfg.source.audioSource == "Line-IN" and "linein" or "hdmi"},
	})
	if task then
		task:Start()
	end

    local proxy
    local thread = require "coaf.thread"
    _M.dbTimer = thread.NewTimer(function()
        local data = {}
        for _, v in ipairs(_M.audioCrossEncChannel) do
            local info = v:GetAudioDB()
            table.insert(data, tonumber(info[0] or 0) + 20)
            table.insert(data, tonumber(info[1] or 0) + 20)
        end

        if not proxy then
            proxy = GetProxy("/websocket", "webSocketSvc")
        else
            proxy.update_data("audio", data)
        end
    end, 1, 0.05):start()
end

function _M.AddAudioEncode(cfg)
    if #_G.aenc_cfg.audioEncode >= MAX_CHANNELS then
        return false, "Limit channel!"
    end
    local default = {
        name = "Channel-1",
        vqeVolume = 0,
        resample = "fastest",
        sampleSize = 1024,
        channels = 2,
        sampling = 48000,
        aacType = "AAC",
        aacFormat = "RAW",
        bitrate = 64000,
        g711Law = "PCMU",
        channelMap = 1,
    }

    table.update(default, cfg)
    table.insert(_G.aenc_cfg.audioEncode, default)
    local meta = _M.CreateChannel(#_G.aenc_cfg.audioEncode)
    meta:Init()
    _G.aenc_cfg:__TRIGGER(true)
    return true
end

function _M.RemoveAudioEncode(chnId)
    if not _G.aenc_cfg.audioEncode[chnId] then
        return true
    end

    for stream, v in pairs(_G.streamer_cfg) do
        for kk, vv in ipairs(v) do
            if vv.bindAudio == chnId then
                return false, "Can't delete this. Some streams are using this Audio Encode"
            end
        end
    end

    local proxy = GetProxy("/record", "record")
    if not proxy then
        return false, "SystemError"
    end

    local is_ok, data = proxy.GetInfo()
    if not is_ok or not data then
        return false, "SystemBuzy"
    end

    if data.bindAudio == chnId then
        return false, "Can't delete this. Record is using this Audio Encode"
    end

    local meta = _M.channels[chnId]
    if meta then
        meta:Destroy()
    end

    table.remove(_M.channels, chnId)
    table.remove(_G.aenc_cfg.audioEncode, chnId)
    _G.aenc_cfg:__TRIGGER(true)

    for stream, v in pairs(_G.streamer_cfg) do
        for kk, vv in ipairs(v) do
            if vv.bindAudio > chnId then
                vv.bindAudio = vv.bindAudio - 1
            end
        end
    end
    _G.streamer_cfg:__TRIGGER(true)

    for _, v in pairs(mod_record.recordSvc) do
        for _, vv in pairs(v.recorder or {}) do
            if vv.bindAudio and vv.bindAudio > chnId then
                vv.bindAudio = vv.bindAudio - 1
            end
        end
    end

    if data.bindAudio > chnId then
        proxy.Update({
            cfg = {
                bindAudio = data.bindAudio - 1
            }
        })
    end
    proxy:Destroy()
    return true
end

function _M.Update(chnId, new)
    local meta = _M.channels[chnId]
    if not meta then
        return false
    end

    local old = table.copy(_G.aenc_cfg.audioEncode[chnId])
    if not table.compare(old, new) then
        return true
    end

    mod_Streamer.Destroy(nil, chnId)
    mod_record.StopRecord("main")
    mod_record.StopRecord("sub")

    local is_ok = meta:Update(new)
    if not is_ok then
        return false, "argumentError"
    end
    
    table.update(_G.aenc_cfg.audioEncode[chnId], new)
    _G.aenc_cfg:__TRIGGER(true)

    for _, v in ipairs({"vqeVolume","resample","sampleSize","channels","sampling","aacType","aacFormat","bitrate","channelMap"}) do
        if new[v] ~= nil then
            meta[v] = new[v]
        end
    end

    mod_Streamer.Init()
    mod_record.StartRecord("main", 2)
    mod_record.StartRecord("sub", 2)
    return true
end

function _M.UpdateSource(audioSource)
    if audioSource ~= _G.aenc_cfg.source.audioSource then
        local task = run_daemon.Create({
            name = "InitAudioSelect",
            run = "/usr/local/bin/cocall.lua",
            args = {"CO", "codec", "/", "SelectAudioSource", audioSource == "Line-IN" and "linein" or audioSource},
        })
        if task then
            task:Start()
        end

        -- if audioSource == "Line-IN" then
        --     for chnId, v in pairs(_M.channels) do
        --         v:Update({
        --             channelMap = 1
        --         })
        --     end
        -- end
        _G.aenc_cfg.source.audioSource = audioSource
        _G.aenc_cfg:__TRIGGER(true)
    end
    return true
end

function _M.GetInfo(chnId)
    if chnId then
        return _G.aenc_cfg.audioEncode[chnId]
    end

    return _G.aenc_cfg
end

function _M.GetAudioEnc(chnId)
    return _M.channels[chnId]
end

function _M.UpdateDispatcherRef(chnId, num)
    local meta = _M.channels[chnId]
    if not meta then
        return
    end

    local started = _M.dispatcher_ref[meta.channelMap] > 0
    _M.dispatcher_ref[meta.channelMap] = _M.dispatcher_ref[meta.channelMap] + (num or 1)
    if _M.dispatcher_ref[meta.channelMap] > 0 then
        if not started then
            _M.dispatcher[meta.channelMap]:Start()
        end
    else
        if started then
            _M.dispatcher[meta.channelMap]:Close()
        end
    end
end

function _M.SetVqeVolume(chnId, vol)
    local meta = _M.channels[chnId]
    if not meta then
        return
    end

    _G.aenc_cfg.audioEncode[chnId].vqeVolume = vol
    _G.aenc_cfg:__TRIGGER(true)

    meta:SetVqeVolume(vol)
    return true
end

function _M.Destroy()
    for _, v in pairs(_M.channels) do
        v:Destroy()
    end

    for i, v in pairs(_M.dispatcher) do
        v:Close()
        v:RemoveInput()
        KM_STREAMING.AudioEngine.FreeAudioSink(_M.outSink[i])
        KM_STREAMING.AudioEngine.FreeAudioDispatcher(v)
    end

    for _, v in pairs(_M.audioCrossEncChannel) do
        v:Stop()
        v:Destroy()
    end

    for _, v in pairs(_M.aiChannel) do
        v:Destroy()
    end

    if _M.dbTimer then _M.dbTimer:stop() end
end

function _M.UpdateAIDev(enSamplerate)
    if enSamplerate == _G.aenc_cfg.source.enSamplerate then 
        return
    end

    for _, v in pairs(_M.aiChannel) do
        if v:SetAudioOption({enSamplerate = enSamplerate}) then
            _G.aenc_cfg.source.enSamplerate = enSamplerate
        end
    end
    _G.aenc_cfg:__TRIGGER(true)
    _M.Destroy()
    _M.Init()
    mod_Streamer.Init()
end

_M.doc = ""
_M.Rule = {
    UpdateAIDev = {
        is_open = true,
        parameters = {
            {key = "enSamplerate", type = "number", must = true},
        }
    },
    AddAudioEncode = {
        is_open = true,
        parameters = {
            {key = "cfg", type = "table", must = true},
        }
    },
    RemoveAudioEncode = {
        is_open = true,
        parameters = {
            {key = "chnId", type = "number", must = true},
        }
    },
    UpdateSource = {
        is_open = true,
        parameters = {
            {key = "audioSource", type = "string", must = true},
        }
    },
    GetAudioEnc = {
        is_open = true,
        parameters = {
            {key = "chnId", type = "number", must = true},
        }
    },
    Update = {
        is_open = true,
        parameters = {
            {key = "chnId", type = "number", must = true},
            {key = "new", type = "table", must = true}
        }
    },
    SetVqeVolume = {
        parameters = {
            {key = "chnId", type = "number", must = true},
            {key = "vol", type = "number", must = true},
        },
        is_open = true
    },
    GetInfo = {
        parameters = {
            {key = "chnId", type = "number"}
        },
        is_open = true
    }
}

return _M