require "kmCore"	--Media core. [ got KM_STREAMING global ]
----------------------------------------------------
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"
local run_daemon = require "coaf.run_daemon" 
local mod_audioEnc
local function stratDaemonTask(args, name, run)
	if not name then
		table.insert(args, 1, "/record")
		table.insert(args, 1, "record")
		table.insert(args, 1, "local")
	end
	name = name or args[4]
	run = run or "/usr/local/bin/cocall.lua"
	local task = run_daemon.Create({
		name = name,
		run = run,
		args = args,
	})
	if task then
		task:Start()
	end
end

local os_date = os.date
local os_time = os.time
local string_format = string.format
local Gprint = _G.print
local print = function(...)
	local now = os_date("*t")
	local tbl = {...}
	table.insert(tbl, string_format("%02d:%02d:%02d", now.hour, now.min, now.sec))
	Gprint(table.unpack(tbl))
end

local mod_streamer
----------------------------------------------------
local _M = {
	allObjects = {},
	recordProxy = nil,
	recordEventsProxy = nil,
	recorderRegistered = nil,
	recordGroup = nil,
	recordSvc = {
		main = {},
		sub = {}
	}
}

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)

local function GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 5 ) --5 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

----------------------------------------------------
-- The signal events for recording service.
function _M.NotifyStartRecord( bindAudio, filesys, stream, seqId, format, path, fileprefix, limit_type, limit ) 
	-- _M.recordProxy = _M.recordProxy or GetProxy("/record", "record")
	print( "*** StartRecord:", filesys, stream, seqId, format, path, fileprefix, limit_type, limit )
	local streamIdx = 0
	if stream == "sub" or stream == "1" then
		streamIdx = 1
	end

	if not _M.recorderRegistered then
		coaf.ScheduleDelayedTask( function()
			print( "***WARNING: The recorder is pending initialization, wait!" )
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "started" )
			else
				print("===============AckStartRecord started failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "started" )
		end)
		-- stratDaemonTask({"AckStartRecord", seqId, "started"})
		return
	end

	if not _G.codec_cfg[stream].enable then
		coaf.ScheduleDelayedTask( function()
			print( "***WARNING: The stream of '" .. stream .. "' is disabled, so could not start recording!" )
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "not-start" )
			else
				print("===============AckStartRecord not-start failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "not-start" )
		end)
		-- stratDaemonTask({"AckStartRecord", seqId, "not-start"})
		return
	end

	local thisSvc = _M.recordSvc[stream]
	local mediaSource = mod_streamer.GetStreamObject(stream, tonumber(bindAudio))
	if not mediaSource then
		coaf.ScheduleDelayedTask( function()
			print( "*** ERROR: Invalid IAV source, so could not start the recording of stream '" .. stream .. "'" )
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "no-source" )
			else
				print("===============AckStartRecord no-source failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "no-source" )
		end)
		-- stratDaemonTask({"AckStartRecord", seqId, "no-source"})
		return
	end

	thisSvc.recorder = thisSvc.recorder or {}
	if thisSvc.recorder[path .. "/" .. seqId] then --It's already started.
		coaf.ScheduleDelayedTask( function()
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "started" )
			else
				print("===============AckStartRecord started failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "started" )
		end)
		-- stratDaemonTask({"AckStartRecord", seqId, "started"})
		return
	end

	thisSvc.recorder[path .. "/" .. seqId] = _M.createNew( filesys, format, streamIdx )
	if not thisSvc.recorder[path .. "/" .. seqId] then
		coaf.ScheduleDelayedTask( function()
			print( "*** ERROR: Fail to create recorder for stream '" .. stream .. "', format=" .. format )
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "start-fail" )
			else
				print("===============AckStartRecord start-fail failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "start-fail" )
		end)
		-- stratDaemonTask({"AckStartRecord", seqId, "start-fail"})
		return
	end

	if not thisSvc.recorder[path .. "/" .. seqId]:BindInputMediaSource( mediaSource, tonumber(bindAudio) ) then
		coaf.ScheduleDelayedTask( function()
			print( "*** ERROR: Fail to bind media source for recording stream '" .. stream .. "', format=" .. format )
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "start-fail" )
			else
				print("===============AckStartRecord start-fail failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "start-fail" )
		end)
		-- stratDaemonTask({"AckStartRecord", seqId, "start-fail"})
		return
	end

	thisSvc.recorder[path .. "/" .. seqId]:SetRecordOptions( path, seqId, fileprefix, limit_type, limit )

	coaf.ScheduleDelayedTask( function()
		if not thisSvc.recorder[path .. "/" .. seqId]:StartRecord() then
			print( "*** ERROR: Fail to start recording for stream '" .. stream .. "', format=" .. format )
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "start-fail" )
			else
				print("===============AckStartRecord start-fail failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "start-fail" )
			return
		else
			local proxy = GetProxy("/record", "record")
			if proxy then
				proxy.AckStartRecord( seqId, "started" )
			else
				print("===============AckStartRecord started failed! seqId = ", seqId)
			end
			-- _M.recordProxy.AckStartRecord( seqId, "started" )
		end
	end )
	-- if not thisSvc.recorder[path .. "/" .. seqId]:StartRecord() then
	-- 	stratDaemonTask({"AckStartRecord", seqId, "start-fail"})
	-- else
	-- 	stratDaemonTask({"AckStartRecord", seqId, "started"})
	-- end
end

function _M.NotifyStopRecord( stream, seqId )
	-- _M.recordProxy = _M.recordProxy or GetProxy("/record", "record")
	print( "*** StopRecord:", stream, seqId, "in line 150" )
	local thisSvc = _M.recordSvc[stream]
	if thisSvc and thisSvc.recorder then
		for k, v in pairs(thisSvc.recorder) do
			if k:match(seqId) then
				-- v:StopRecord()
				_M.free(v)
				thisSvc.recorder[k] = nil
			end
		end
	end
	print( "*** StopRecord:", stream, seqId, "prepaer to run delayedTask in line 161" )
	-- coaf.ScheduleDelayedTask( function()
		print( "*** StopRecord:", stream, seqId, "in ScheduleDelayedTask" )
		local proxy = GetProxy("/record", "record")
		if proxy then
			proxy.AckStopRecord( seqId, "stopped" )
		else
			print("===============AckStartRecord stopped failed! seqId = ", seqId)
		end
		-- _M.recordProxy.AckStopRecord( seqId, "stopped" )
		print( "*** StopRecord:", stream, seqId, "end of ScheduleDelayedTask" )
	-- end)
	-- coaf.ScheduleDelayedTask(function()
		-- stratDaemonTask({"AckStopRecord", seqId, "stopped"})
	-- end, 0.5)
end

----------------------------------------------------
function _M.Init()
	run_daemon.Init()
	mod_streamer = require "mod_Streamer"
	mod_audioEnc = require "mod_audioEnc"
	-- _M.recordProxy = GetProxy("/record", "record")
	-- if not _M.recordProxy then
	-- 	print( "***ERROR: Fail to get recording proxy!" )
	-- 	return false
	-- end
	return true
end

----------------------------------------------------
local indexFlag = 0
function _M.createNew( fileSys, fileType, streamIdx )
	-- _M.recordProxy = _M.recordProxy or GetProxy("/record", "record")
	-- if not _M.recordProxy then
	-- 	print( "***ERROR: Un-initialized record module, invalid DBUS client session!" )
	-- 	return nil
	-- end

	local OBJ = {
		fileSys = fileSys or "vfat",
		fileType = fileType or "ts",
		streamIdx = streamIdx,
		filePrefix = "REC",
		recordSess = nil,
		mediaSource = nil,
		recordingStatus = "idle",
		location = nil,
		sequenceId = nil,
		limitionType = nil,
		limitValue = nil,
		lastError = nil,
		curSplitFile = nil,
		curSplitStatus = "idle",
		curFileSize = 0,
		curFileDur = 0
	}

	function OBJ:BindInputMediaSource( mediaSource, bindAudio )
		if self.recordingStatus == "recording" then
			print( "***ERROR: The media source is already bound and you must stop recording to bind again!" )
			return false
		end

		self.mediaSource = mediaSource
		self.bindAudio = bindAudio
		return true
	end

	function OBJ:SetRecordOptions( location, sequenceId, fileprefix, limitionType, limitValue )
		if self.recordingStatus == "recording" then
			print( "***ERROR: The recording is on work, please stop it first to set recording options." )
			return false
		end

		self.filePrefix = fileprefix
		self.location = location
		self.sequenceId = sequenceId
		self.limitionType = limitionType or "none"
		self.limitValue = limitValue or "0"
		return true
	end

	function OBJ:StartRecord()
		if self.recordingStatus == "recording" or self.recordingStatus == "preparing" then
			return true
		end

		if not self.mediaSource then
			self.lastError = "invalid_media_source"
			print( "***ERROR: Invalid media source bound, could not start recording" )
			return false
		end

		if not self.location or not self.sequenceId then
			self.lastError = "invalid_options"
			print( "***ERROR: Invalid recording options!" )
			return false
		end

		if not _M.recordGroup then
			_M.recordGroup = KM_STREAMING.Recorder.createNew("recorder")
			if not _M.recordGroup then
				self.lastError = "fail_create_recorder"
				print( "***ERROR: Fail to create recorder group" )
				return false
			end
		end


		self.recordSess = _M.recordGroup:AddSession(self.sequenceId .. '-' .. indexFlag, self.mediaSource, 4*1024*1024, 64*1024)
		if not self.recordSess then
			print( "***ERROR: Fail to create a record session for sequence:", self.sequenceId)
			self.lastError = "fail_create_record_session"
			return false
		end

		indexFlag = indexFlag + 1
		local multiple = false
		local maxSize = "4G" --NOTE:TODO:This is a limitation for current device.
		if self.fileSys == "ntfs" or self.fileSys:match("ext") then
			maxSize = "0"
		end
		local maxTime = "0"
		if self.limitionType:match( "time" ) then
			if self.limitionType:match( "loop" ) then
				multiple = true
			elseif self.limitionType:match( "multiple" ) then
				multiple = true
			elseif self.limitionType:match( "split" ) then
				multiple = true
			end
			maxTime = "" .. self.limitValue
		elseif self.limitionType:match( "size" ) then
			if self.limitionType:match( "loop" ) then
				multiple = true
			elseif self.limitionType:match( "multiple" ) then
				multiple = true
			elseif self.limitionType:match( "split" ) then
				multiple = true
			end
			maxSize = "" .. self.limitValue .. "K"
		end

		local filePath = self.location
		if #filePath > 0 and filePath:sub(-1) ~= "/" then
			filePath = filePath .. "/" .. self.sequenceId .. "/"
		else
			filePath = filePath .. self.sequenceId .. "/"
		end

		local muxrate = -1
		if self.fileType == "ts" or self.fileType == "TS" then
			-- local streams = require "streams"
			-- muxrate = streams.GetTsMuxrate( self.streamIdx )
		end

		_M.recordGroup:SetOption( self.recordSess, {
			path = filePath,
			type = self.fileType,
			prefix = self.filePrefix or "REC",
			multiple = multiple,
			maxSize = maxSize,
			maxTime = maxTime,
			muxrate = muxrate,
			ts_pts_pcr_delay = 500,
			ts_null_multiple = 1.2 --TODO: Here.
		})

		self.lastError = ""
		self.recordingStatus = "preparing"
		self.curSplitStatus = "idle"
		self.curSplitFile = nil
		self.curFileSize = 0
		self.curFileDur = 0

		_M.recordGroup:StartSession( self.recordSess )
		mod_audioEnc.UpdateDispatcherRef(self.bindAudio)
		return true
	end

	function OBJ:StopRecord()
		print("in line 296 , in function StopRecord")
		if self.recordingStatus == "recording" or self.recordingStatus == "preparing" then
			if self.recordingStatus == "recording" then
				coaf.ScheduleDelayedTask( function()
					print("!!!!!!!!!!!!!!!!!!!!!!!!!!! in ScheduleDelayedTask line 299, Stop the record")
					local proxy = GetProxy("/record", "record")
					if proxy then
						proxy.UpdateRecord( self.sequenceId, "end", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(self.curFileSize) or 0) )
					else
						print("===============UpdateRecord end failed!")
					end
					-- _M.recordProxy.UpdateRecord( self.sequenceId, "end", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(self.curFileSize) or 0) )
					print("!!!!!!!!!!!!!!!!!!!!!!!!!!! end of  ScheduleDelayedTask line 301, Stop the record")
				end, self.streamIdx == 0 and 0.01 or 0.1)
				-- stratDaemonTask({"UpdateRecord", self.sequenceId, "end", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(self.curFileSize) or 0)})
			end
			
			self.recordingStatus = "stopped"
			_M.recordGroup:StopSession( self.recordSess )
			mod_audioEnc.UpdateDispatcherRef(self.bindAudio, -1)
			self.recordSess = nil
			collectgarbage()
			stratDaemonTask({}, "/bin/sync", "/bin/sync")
			-- os.execute("/bin/sync")
			return true
		end
		return true
	end

	function OBJ:Check()
		if self.recordingStatus == "preparing" then
			local status, err = _M.recordGroup:GetSessionStatus( self.recordSess )
			if not status then
				print( "*** Get recording status fail:", err )
				return
			end

			local curSplitStatus, curSplitFile, curFileSize, curFileDur
			curSplitStatus = status.state
			curSplitFile = status.file
			curFileSize = status.size_kb
			curFileDur = status.time

			if curSplitStatus == "recording" then
				self.recordingStatus = "recording"
			else
				return
			end
		end

		if self.recordingStatus == "recording" then
			local status, err = _M.recordGroup:GetSessionStatus( self.recordSess )
			if not status then
				print( "*** Get recording status fail:", err )
				return
			end

			local curSplitStatus, curSplitFile, curFileSize, curFileDur
			curSplitStatus = status.state
			curSplitFile = status.file
			curFileSize = status.size_kb
			curFileDur = status.time
			
			local proxy = GetProxy("/record", "record")
			if curSplitFile and curSplitFile ~= "" and curFileSize and curFileDur then
				if self.curSplitFile ~= curSplitFile then
					if self.curSplitFile and self.curSplitFile ~= "" then
						-- coaf.ScheduleDelayedTask( function()
							print("!!!!!!!!!!!!!!!!!!!!!!!!!!! in line 354, Update Record to be end")

							if proxy then
								proxy.UpdateRecord( self.sequenceId, "end", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(self.curFileSize) or 0 ) )
							else
								print("===============UpdateRecord end failed!")
							end
							-- _M.recordProxy.UpdateRecord( self.sequenceId, "end", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(self.curFileSize) or 0 ) )
						-- end, self.streamIdx == 0 and 0.01 or 0.1)
						-- stratDaemonTask({"UpdateRecord", self.sequenceId, "end", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(self.curFileSize) or 0 )})
					end

					-- coaf.ScheduleDelayedTask( function()
						print("!!!!!!!!!!!!!!!!!!!!!!!!!!! in line 361, Update Record to be new")
						-- local proxy = GetProxy("/record", "record")
						if proxy then
							proxy.UpdateRecord( self.sequenceId, "new", curSplitFile, self.fileType, os_time(), math.floor(tonumber(curFileSize) or 0 ) )
						else
							print("===============UpdateRecord new failed!")
						end
						-- _M.recordProxy.UpdateRecord( self.sequenceId, "new", curSplitFile, self.fileType, os_time(), math.floor(tonumber(curFileSize) or 0 ) )
						-- print("!!!!!!!!!!!!!!!!!!!!!!!!!!! in line 363, Update Record to be new")
					-- end, self.streamIdx == 0 and 0.01 or 0.1)
					-- stratDaemonTask({"UpdateRecord", self.sequenceId, "new", curSplitFile, self.fileType, os_time(), math.floor(tonumber(curFileSize) or 0 )})
					self.curSplitFile = curSplitFile
					self.curSplitStatus = curSplitStatus
					self.curFileSize = curFileSize
					self.curFileDur = curFileDur
				end
			end
						
			if curSplitStatus == "recording" then
				local dur = curFileDur - self.curFileDur
				if dur < 0 or dur >= 1 then --NOTE: old dur >= 5
					self.curFileDur = curFileDur
					self.curFileSize = curFileSize
					-- stratDaemonTask({"UpdateRecord", self.sequenceId, "update", self.curSplitFile, 
					-- self.fileType, os_time(), math.floor(tonumber(curFileSize) or 0)})
					-- coaf.ScheduleDelayedTask( function()
						print("!!!!!!!!!!!!!!!!!!!!!!!!!!! in line 390, Update Record to be update")
						-- local proxy = GetProxy("/record", "record")
						if proxy then
							proxy.UpdateRecord( self.sequenceId, "update", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(curFileSize) or 0) )
						else
							print("===============UpdateRecord update failed!")
						end
						-- _M.recordProxy.UpdateRecord( self.sequenceId, "update", self.curSplitFile, self.fileType, os_time(), math.floor(tonumber(curFileSize) or 0) )
					-- end, self.streamIdx == 0 and 0.01 or 0.1)
				end
			elseif curSplitStatus ~= "preparing" and curSplitStatus ~= "starting" then
				if curSplitFile == self.curSplitFile and curFileSize > 0 and curFileDur > 0 then
					self.curFileDur = curFileDur
					self.curFileSize = curFileSize
				end

				self:StopRecord()
			end
			if proxy then
				proxy:Destroy()
			end
		end
	end

	table.insert( _M.allObjects, OBJ )
	return OBJ
end

----------------------------------------------------
function _M.free(obj)
	local k,v
	for k,v in ipairs( _M.allObjects ) do
		if v == obj then
			v:StopRecord()
			table.remove( _M.allObjects, k )
			break
		end
	end
end

----------------------------------------------------
function _M.Check()
	-- _M.recordProxy = _M.recordProxy or GetProxy("/record", "record")
	if not _M.recorderRegistered then
		local thisSvc = next(mod_streamer.main)
		if thisSvc then
			print( "*** Media is ready, register recording worker sink to record management service!" )
			coaf.ScheduleDelayedTask( function()
				local proxy = GetProxy("/record", "record")
				if proxy then
					proxy.RegisterWorker( "any" )
				else
					print("===============RegisterWorker any failed!")
				end
				-- _M.recordProxy.RegisterWorker( "any" )
			end)
			-- stratDaemonTask({"RegisterWorker", "any"})
			_M.recorderRegistered = true
		end
	end

	for k,v in ipairs( _M.allObjects ) do
		if v then
			v:Check()
		end
	end
end
----------------------------------------------------

function _M.StopRecord(stream, delay)
	if stream == "preview" then
		return
	end
	if not _M.recordProxy then
		return
	end
	coaf.ScheduleDelayedTask(function()
		local proxy = GetProxy("/record", "record")
		if proxy then
			proxy.ControlRecord(stream, false)
		else
			print("===============ControlRecord false failed!")
		end
		-- _M.recordProxy.ControlRecord(stream, false)
	end, tonumber(delay) or 0.01)
	-- stratDaemonTask({"ControlRecord", stream, false})
end

function _M.StartRecord(stream, delay)
	if not _M.recordProxy then
		return
	end
	coaf.ScheduleDelayedTask(function()
		local proxy = GetProxy("/record", "record")
		if proxy then
			proxy.ControlRecord(stream, true)
		else
			print("===============ControlRecord true failed!")
		end
		-- _M.recordProxy.ControlRecord(stream, true)
	end, tonumber(delay) or 0.01)
	-- stratDaemonTask({"ControlRecord", stream, true})
end

-- function _M.Terminate()
-- end

return _M

