require "himpp"
local _dbg = require "coaf.debug"
local coaf = require "coaf.core"
local run_daemon = require "coaf.run_daemon"
local codec

local VI = HiMPP.VI

local _M = {
	MAX_CHANNELS = 2,
	channels = {}
}

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)

local function GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 5 ) --5 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local function conversion(max, x, y, max_y)
	local str = "0x"
	if x == 128 then 
		x = "0100" 
	elseif x < 128 then
		x = string.format("%04X", math.floor((x / 128) * 256 ))
	else
		x = string.format("%04X", math.floor((x - 128) / 128 * (max - 256)) + 256 )
	end
	str = str .. x

	if y then
		if y == 128 then
			y = "0100"
		elseif y < 128 then
			y = string.format("%04X", math.floor((y / 128) * 256 ))
		else
			y = string.format("%04X", math.floor((y - 128) / 128 * (max_y - 256)) + 256 )
		end
		str = str .. y
	end

	return tonumber(str)
end

local function UpdateFPGA(brightness, contrast, saturation, use_daemon)
	if use_daemon then
		local task = run_daemon.Create({
			name = "UpdateFPGAColor1",
			run = "/usr/local/bin/cocall.lua",
			args = {"CO", "codec", "/", "write_reg", 0x200, 0x01, conversion(0x07ff, saturation)},
		})
		if task then
			task:Start()
		end

		task = run_daemon.Create({
			name = "UpdateFPGAColor2",
			run = "/usr/local/bin/cocall.lua",
			args = {"CO", "codec", "/", "write_reg", 0x200, 0x11, conversion(0x07ff, contrast, brightness, 0x0200)},
		})
		if task then
			task:Start()
		end
		return
	end
	local proxy = GetProxy("/", "codec", "CO")
	if proxy then
		proxy.write_reg(0x200, 0x01, conversion(0x07ff, saturation)) --饱和度。有效值 0x0000~0x07ff, 中值0x0100
		proxy.write_reg(0x200, 0x11, conversion(0x07ff, contrast, brightness, 0x0200))  --高 16 位为 对比度，低16位为亮度, 有效值 0x0000~0x07ff, 中值 0x0100
		proxy:Destroy()
		return true
	end
	return false
end

local streamMaxWH = {
	main = { w = 3840, h = 2160}, sub = { w = 1920, h = 1080}, preview = { w = 960, h = 540},
}
local function UpdateCodec(cfg)
	codec = codec or require "codec"
	for stream, v in pairs(_G.codec_cfg) do
		if stream ~= "preview" then
			-- if v.ve.u32PicWidth == 1 or v.ve.u32PicWidth > cfg.stDestSize[1] then
				local setWidth = v.ve.u32PicWidth == 1 and 1 or cfg.stDestSize[1]
				local setHeight = v.ve.u32PicHeight == 1 and 1 or cfg.stDestSize[2]
				codec.Update(stream, {
					ve = {
						u32PicWidth = math.min(setWidth, streamMaxWH[stream].w),
						u32PicHeight = math.min(setHeight, streamMaxWH[stream].h),
					},
					rc = {
						u32SrcFrmRate = cfg.sourceFrameRate,
						fr32DstFrmRate = math.min(v.rc.fr32DstFrmRate, cfg.sourceFrameRate or v.rc.fr32DstFrmRate),
					}
				})
			-- end
		end
	end

	codec.SetCrop(nil, _G.vpss_cfg)
end

-------------------------------------------------------
-- Channel meta methods.
-------------------------------------------------------
local channel_meta = {}
function channel_meta:Init()
	if self.initialized then
		return
	end

	self.vi_dev = VI.GetDevice(self.id - 1)
	if not self.vi_dev then
		_dbg.Error( "CAPTURE", "Fail to get VI input device 0" )
		return
	end

	self.vi_channel = self.vi_dev:GetChannel(self.id - 1)
	self.vi_channel:Disable()
	self.vi_dev:Disable()

	local r, errmsg = self.vi_dev:ConfigMode(self.attr.mode)
	if not r then
		_dbg.Error( "CAPTURE", "Fail to configure VI device to BT.1120 mode, with error:", errmsg, "set mode = ", self.attr.mode )
		self.vi_dev = nil
		return
	end

	r, errmsg = self.vi_channel:SetAttr( self.attr )
	if not r then
		_dbg.Error( "CAPTURE", "Fail to set VI channel attributes:", errmsg )
	end

	self.vi_channel:SetRotate(self.attr.rotate or 0)
	self.vi_channel:SetUserPic({
		u32BgColor = 0x0000ff
	})

	r = self.vi_dev:Enable()
	if r ~= 0 then
		_dbg.Error( "CAPTURE", "Fail to enable VI device, with error code:", string.format("%08X",r) )
	end

	self.initialized = true
end

-- Destroy channel.
function channel_meta:Destroy()
	if not self.initialized then
		return
	end

	self.vi_channel:Destroy()
	self.vi_dev:Disable()
	self.initialized = false
end

function channel_meta:Start()
	if not self.initialized then
		self:Init()
	end
	if self.initialized then
		local r = self.vi_channel:Enable()
		if r ~= 0 then
			_dbg.Error( "CAPTURE", "Fail to enable VI input channel for capture(ID=" .. self.id .. "), error:", string.format("%08X",r) )
		end
	end
end

function channel_meta:Stop()
	if self.initialized then
		self.vi_channel:Disable()
	end
end

function channel_meta:UpdateConfig(cfg)
	local same_frame = true
	if cfg.sourceFrameRate and cfg.sourceFrameRate ~= self.attr.sourceFrameRate then
		_dbg.Info("CAPTURE", "--------------------------------------------------Set sourceFrameRate = ", cfg.sourceFrameRate)
		_G.capture_cfg[self.id].sourceFrameRate = cfg.sourceFrameRate
		_G.capture_cfg:__TRIGGER(true)
		same_frame = false
	end

	cfg.stCapRect[3] = math.min(3840, cfg.stCapRect[3])
	cfg.stCapRect[4] = math.min(2160, cfg.stCapRect[4])
	cfg.stDestSize[1] = math.min(3840, cfg.stDestSize[1])
	cfg.stDestSize[2] = math.min(2160, cfg.stDestSize[2])
	if (cfg.stCapRect[3] == self.attr.stCapRect[3] 
		and cfg.stCapRect[4] == self.attr.stCapRect[4]) and same_frame then
		_dbg.Info("CAPTURE", "-------------------------------------------------- No Need Re Init VI and Return")
		return true
	end

	self:Stop()
	local old = table.copy( self.attr )
	table.update(old, cfg)
	local r, errmsg = self.vi_channel:SetAttr( old )
	self:Start()
	if not r then
		self.vi_channel:SetAttr( self.attr )
		_dbg.Error( "CAPTURE", "Fail to update VI channel attributes:", errmsg )
		return
	end

	table.update(_G.capture_cfg[self.id], cfg)
	_G.capture_cfg:__TRIGGER(true)

	UpdateCodec(_G.capture_cfg[self.id])
	return true
end

channel_meta.__index = channel_meta

--------------------------------------------------------------

function _M.Init()
	if not VI then
		_dbg.Error( "CAPTURE", "This chipset does not support video capture feature!" )
		return
	end

	_M.vi_attr = _G.capture_cfg

	for chnId, attr in pairs(_M.vi_attr) do
		local one = _M.CreateChannel(chnId)
		one:Init()
	end

	run_daemon.Init()
	UpdateFPGA(_M.vi_attr[1].brightness, _M.vi_attr[1].contrast, _M.vi_attr[1].saturation, true)
end

function _M.Destroy()
	for _, v in pairs(_M.channels) do
		v:Destroy()
	end
	_M.channels = {}
end

function _M.CreateChannel(chnId)
	chnId = chnId or 1
	if chnId < 1 or chnId > _M.MAX_CHANNELS then
		_dbg.Error( "CAPTURE", "Invalid channel ID passed. It's only support 1-" .. _M.MAX_CHANNELS .. " channels." )
		return nil
	end

	if _M.channels[chnId] then
		return _M.channels[chnId]
	end

	local chanObj = {
		id = chnId,
		attr = _M.vi_attr[chnId]
	}

	if not chanObj.attr then
		_dbg.Error( "CAPTURE", "CreateChannel failed! Please check chnId and VI model is support!" )
		return
	end

	local r = setmetatable( chanObj, channel_meta )
	_M.channels[ chnId ] = r
	return r
end

------------------------

function _M.UpdateVideoSource(chnId, new)
	if _G.capture_cfg[chnId] then
		_G.capture_cfg[chnId].videoSource = _G.capture_cfg[chnId].videoSource or new.videoSource
		_G.capture_cfg:__TRIGGER(true)
	end
	if 1 == 1 then  --NOTE: 第一版只有hdmi
		return true
	end

	-- if not _M.channels[chnId] or (source ~= "hdmi" and source ~= "NDI") then
	-- 	return false
	-- end
	--TODO: 切换视频源
	-- return true
end

function _M.Update(chnId, cfg)
	local meta = _M.channels[chnId]
	if not meta then
		_dbg.Error("CAPTURE", "SetConfig failed, please check the chnId is sure!")
		return
	end
	print("In Capture Update, cfg = ")
	local pt = require "pl.pretty"
	pt.dump(cfg)
	return meta:UpdateConfig({
		stCapRect = cfg.stCapRect,
		stDestSize = cfg.stDestSizeRect,
		sourceFrameRate = tonumber(cfg.frame_rate),
	})
end

function _M.GetInfo(chnId)
	if not chnId then
		local tbl = {}
		for k, v in pairs(_G.capture_cfg) do
			tbl[k] = {
				videoSource = v.videoSource,
				-- stCapRect = v.stCapRect,
				-- stDestSize = v.stDestSize,
				bMirror = v.bMirror,
				bFlip = v.bFlip,
				rotate = v.rotate,
				sourceFrameRate = v.sourceFrameRate,
				sourceSize = v.stDestSize[1] .. "x" .. v.stDestSize[2],
				brightness = v.brightness,
				contrast = v.contrast,
				saturation = v.saturation,
			}
		end
		return tbl
	end

	return {
		bMirror = _G.capture_cfg[chnId].bMirror,
		bFlip = _G.capture_cfg[chnId].bFlip,
		videoSource = _G.capture_cfg[chnId].videoSource,
		rotate = _G.capture_cfg[chnId].rotate,
		sourceFrameRate = _G.capture_cfg[chnId].sourceFrameRate,
		sourceSize = _G.capture_cfg[chnId].stDestSize[1] .. "x" .. _G.capture_cfg[chnId].stDestSize[2],
		brightness = _G.capture_cfg[chnId].brightness,
		contrast = _G.capture_cfg[chnId].contrast,
		saturation = _G.capture_cfg[chnId].saturation,
		-- stCapRect = _G.capture_cfg[chnId].stCapRect,
		-- stDestSize = _G.capture_cfg[chnId].stDestSize,
	}
end

function _M.EnablePic(enable)
	for _, v in pairs(_M.channels) do
		if enable then
			v.vi_channel:EnableUserPic()
		else
			v.vi_channel:DisableUserPic()
		end
	end
end

function _M.SetRotate(chnId, num, skip)
	chnId = chnId or 1
	if num ~= 0 and num ~= 90 and num ~= 180 and num ~= 270 then
		return false, "Error Number"
	end

	local meta = _M.channels[chnId]
	if not skip then
		meta:Stop()
	end
	local result = meta.vi_channel:SetRotate(num)
	_G.capture_cfg[chnId].rotate = num
	_G.capture_cfg:__TRIGGER(true)
	if not skip then
		meta:Start()
	end

	codec = codec or require "codec"
	codec.RotateVPSSAndVenc(chnId)
	return result
end

function _M.UpdateDynamic(chnId, cfg)
	chnId = tonumber(chnId) or 1
	local meta = _M.channels[chnId]
	if not meta then
		return false, "Invalid Channle id"
	end

	local config = table.copy(meta.attr)
	table.update(config, {
		rotate = cfg.rotate,
		bMirror = cfg.bMirror,
		bFlip = cfg.bFlip
	})

	local failed = true
	meta:Stop()
	if _M.SetRotate(chnId, config.rotate, true) then
		failed = false
	else
		_dbg.Error("CAPTURE", "Set rorate failed! Num = ", config.rotate)
	end

	if meta.vi_channel:SetAttr(config) then
		_G.capture_cfg[chnId].bMirror = config.bMirror
		_G.capture_cfg[chnId].bFlip = config.bFlip
		failed = false
	else
		_dbg.Error("CAPTURE", "Set attr failed! bMirror = ", config.bMirror, "bFlip = ", config.bFlip)
	end
	meta:Start()

	_G.capture_cfg:__TRIGGER(true)
	return not failed, failed and "Invalid arguments!"
end

function _M.UpdateColor(chnId, brightness, contrast, saturation)
	if not UpdateFPGA(brightness, contrast, saturation) then
		return false, "setError"
	end
	_G.capture_cfg[chnId].brightness = brightness
	_G.capture_cfg[chnId].contrast = contrast
	_G.capture_cfg[chnId].saturation = saturation
	_G.capture_cfg:__TRIGGER(true)
	return true
end

_M.doc = ""
_M.Rule = {
	UpdateColor = {
		is_open = true,
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "brightness", type = "number", must = true},
			{key = "contrast", type = "number", must = true},
			{key = "saturation", type = "number", must = true},
		}
	},
	UpdateDynamic = {
		is_open = true,
		parameters = {
			{key = "chnId", type = "number"},
			{key = "cfg", type = "table", must = true}
		}
	},
	SetRotate = {
		is_open = true,
		parameters = {
			{key = "chnId", type = "number"},
			{key = "num", type = "number", must = true}
		}
	},
	EnablePic = {
		is_open = true,
		parameters = {
			{key = "enable", type = "boolean"}
		},
	},
	UpdateVideoSource = {
		is_open = true,
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "new", type = "table", must = true}
		},
	},
	Update = {
		is_open = true,
		parameters = {
			{key = "chnId", type = "number", must = true},
			{key = "cfg", type = "table", must = true}
		},
	},
	GetInfo = {
		parameters = {
			{key = "chnId", type = "number"},
		},
		is_open = true
	},
}

return _M

