require "himpp"
local _dbg = require "coaf.debug"

local SYS = HiMPP.SYS
local VENC = HiMPP.VENC
local codec

local _M = {
    MAX_CHANNELS = 4,
    channels = {},
}

function _M.Init()
    -- empty
	codec = require "codec"
end

function _M.Destroy()
    for i=1, _M.MAX_CHANNELS do
		if _M.channels[i] then
			_M.channels[i]:Destroy()
		end
	end
	_M.channels = {}
end

local channel_meta = {}
function channel_meta:Init()
    if self.initialized then
		return
	end

	if not self.srcChannel then
		_dbg.Error( "ENCODER", "No valid video source channel bound with encoder " .. self.id )
		return
	end

	VENC.ForceFreeChannel(self.id - 1)
    local attr = {
        encoding = self.encodingOpt,
		bitrate = self.bitrateOpts
    }
	if tonumber(attr.encoding.attr.u32PicHeight) == 1 or tonumber(attr.encoding.attr.u32PicWidth) == 1 then
		if self.id == 2 then
			attr.encoding.attr.u32PicHeight = math.min(_G.capture_cfg[1].stDestSize[2], 1080)
			attr.encoding.attr.u32PicWidth = math.min(_G.capture_cfg[1].stDestSize[1], 1920)
		else
			attr.encoding.attr.u32PicHeight = _G.capture_cfg[1].stDestSize[2]
			attr.encoding.attr.u32PicWidth = _G.capture_cfg[1].stDestSize[1]
		end
	end

	local isWidthBigger = codec.WidthIsBigger()
	local fixPicWidth = attr.encoding.attr.u32PicWidth
	local fixPicHeight = attr.encoding.attr.u32PicHeight
	local fixMaxWidth = attr.encoding.attr.u32MaxPicWidth
	local fixMaxHeight = attr.encoding.attr.u32MaxPicHeight
	if isWidthBigger ~= (attr.encoding.attr.u32PicWidth > attr.encoding.attr.u32PicHeight) then
		local tmp = attr.encoding.attr.u32PicWidth
		attr.encoding.attr.u32PicWidth = attr.encoding.attr.u32PicHeight
		attr.encoding.attr.u32PicHeight = tmp
	end

	if isWidthBigger ~= (attr.encoding.attr.u32MaxPicWidth > attr.encoding.attr.u32MaxPicHeight) then
		local tmp = attr.encoding.attr.u32MaxPicWidth
		attr.encoding.attr.u32MaxPicWidth = attr.encoding.attr.u32MaxPicHeight
		attr.encoding.attr.u32MaxPicHeight = tmp
	end

    local venc_chan = VENC.GetChannel(self.id - 1)
    if not venc_chan then
        _dbg.Error( "ENCODER", "Fail to get VENC channel of " .. self.id )
		return nil
    end

	local fixSrcFrmRate = attr.bitrate.attr.u32SrcFrmRate
	local fixDstFrmRate = attr.bitrate.attr.fr32DstFrmRate
	local fixMaxBitrate = attr.bitrate.attr.u32MaxBitRate
	local fixBitrate = attr.bitrate.attr.u32BitRate
	if _G.capture_cfg[1].stDestSize[1] == 3840 then
		attr.bitrate.attr.u32SrcFrmRate = 30
		attr.bitrate.attr.fr32DstFrmRate = math.min(attr.bitrate.attr.fr32DstFrmRate, 30)
		if attr.bitrate.mode:match("H265") then
			attr.bitrate.attr.u32MaxBitRate = math.min(20 * 1024, attr.bitrate.attr.u32MaxBitRate)
			attr.bitrate.attr.u32BitRate = math.min(20 * 1024, attr.bitrate.attr.u32BitRate)
		end
	else
		attr.bitrate.attr.u32SrcFrmRate = math.ceil(_G.capture_cfg[1].sourceFrameRate == 0 and 60 or _G.capture_cfg[1].sourceFrameRate)
		attr.bitrate.attr.fr32DstFrmRate = math.min(attr.bitrate.attr.fr32DstFrmRate, attr.bitrate.attr.u32SrcFrmRate)
	end
	print("dump the create venc attr : ")
	local pt = require "pl.pretty"
	pt.dump(attr)

    local r, errmsg = venc_chan:Create(attr)

	attr.bitrate.attr.u32SrcFrmRate = fixSrcFrmRate
	attr.bitrate.attr.fr32DstFrmRate = fixDstFrmRate
	attr.bitrate.attr.u32MaxBitRate = fixMaxBitrate
	attr.bitrate.attr.u32BitRate = fixBitrate
	attr.encoding.attr.u32PicWidth = fixPicWidth
	attr.encoding.attr.u32PicHeight = fixPicHeight
	attr.encoding.attr.u32MaxPicWidth = fixMaxWidth
	attr.encoding.attr.u32MaxPicHeight = fixMaxHeight

	if not r then
		_dbg.Error( "ENCODER", "Fail to create VENC channel of " .. self.id .. ", with error:", errmsg )
        -- venc_chan:Destroy()
		return
	end

    r, errmsg = SYS.Bind( self.srcChannel, venc_chan )
	if not r then
		_dbg.Error( "ENCODER", "Fail to bind video source channel to encoder, for encoding channel " .. self.id .. ", reason:", errmsg )
		venc_chan:Destroy()
		return
	end

	venc_chan:EnableIDR(true)
    self.vencChannel = venc_chan
	self.initialized = true
    return true
end

function channel_meta:Destroy()
    if not self.initialized then
        return
    end

    self.vencChannel:Stop()
    self.srcChannel:Disable()

    SYS.UnBind( self.srcChannel, self.vencChannel )

    self.vencChannel:Destroy()
	self.vencChannel = nil
	self.initialized = false
end

function channel_meta:Start()
	if not self.initialized then
		if not self:Init() then
            return
        end
	end

    self.srcChannel:Enable()
    self.vencChannel:Start()
    self.started = true
    return true
end

function channel_meta:Stop()
	if self.initialized then
		self.vencChannel:Stop()
		self.srcChannel:Disable()
		self.started = nil
	end
end

--NOTE:这个方法仅用于修改动态属性，如果需要修改静态属性，那么需要销毁此通道(Destroy)重新创建(Init)
function channel_meta:UpdateEncoding(encoding_cfg, bitrate_cfg)
    if not self.vencChannel then
        return
    end

    local attr = {
        encoding = encoding_cfg,
        bitrate = bitrate_cfg
    }

	if tonumber(attr.encoding.attr.u32PicHeight) == 1 or tonumber(attr.encoding.attr.u32PicWidth) == 1 then
		if self.id == 2 then
			attr.encoding.attr.u32PicHeight = math.min(_G.capture_cfg[1].stDestSize[2], 1080)
			attr.encoding.attr.u32PicWidth = math.min(_G.capture_cfg[1].stDestSize[1], 1920)
		else
			attr.encoding.attr.u32PicHeight = _G.capture_cfg[1].stDestSize[2]
			attr.encoding.attr.u32PicWidth = _G.capture_cfg[1].stDestSize[1]
		end
	end

	local isWidthBigger = codec.WidthIsBigger()
	local fixPicWidth = attr.encoding.attr.u32PicWidth
	local fixPicHeight = attr.encoding.attr.u32PicHeight
	local fixMaxPicHeight = attr.encoding.attr.u32MaxPicHeight
	local fixMaxPicWidth = attr.encoding.attr.u32MaxPicWidth
	if isWidthBigger ~= (attr.encoding.attr.u32PicWidth > attr.encoding.attr.u32PicHeight) then
		local tmp = attr.encoding.attr.u32PicWidth
		attr.encoding.attr.u32PicWidth = attr.encoding.attr.u32PicHeight
		attr.encoding.attr.u32PicHeight = tmp

		if isWidthBigger then
			attr.encoding.attr.u32MaxPicHeight = math.min(fixMaxPicHeight, fixMaxPicWidth )
			attr.encoding.attr.u32MaxPicWidth = math.max(fixMaxPicHeight, fixMaxPicWidth)
		else
			attr.encoding.attr.u32MaxPicHeight = math.max(fixMaxPicHeight, fixMaxPicWidth)
			attr.encoding.attr.u32MaxPicWidth = math.min(fixMaxPicHeight, fixMaxPicWidth )
		end
	end

	local fixSrcFrmRate = attr.bitrate.attr.u32SrcFrmRate
	local fixDstFrmRate = attr.bitrate.attr.fr32DstFrmRate
	local fixMaxBitrate = attr.bitrate.attr.u32MaxBitRate
	local fixBitrate = attr.bitrate.attr.u32BitRate
	if _G.capture_cfg[1].stDestSize[1] == 3840 then
		attr.bitrate.attr.u32SrcFrmRate = 30
		attr.bitrate.attr.fr32DstFrmRate = math.floor(math.min(attr.bitrate.attr.fr32DstFrmRate, 30))
		if attr.bitrate.mode:match("H265") then
			attr.bitrate.attr.u32MaxBitRate = math.min(20 * 1024, attr.bitrate.attr.u32MaxBitRate)
			attr.bitrate.attr.u32BitRate = math.min(20 * 1024, attr.bitrate.attr.u32BitRate)
		end
	else
		attr.bitrate.attr.u32SrcFrmRate = math.ceil(_G.capture_cfg[1].sourceFrameRate == 0 and 60 or _G.capture_cfg[1].sourceFrameRate)
		attr.bitrate.attr.fr32DstFrmRate = math.floor(math.min(attr.bitrate.attr.fr32DstFrmRate, attr.bitrate.attr.u32SrcFrmRate))
	end
    
    self:Stop()
	local pt = require "pl.pretty"
	print("=================== in encoder Update !!!!! Self.id =", self.id, "Attr:")
	pt.dump(attr)
    local r, errmsg = self.vencChannel:SetAttr(attr)
    self:Start()
	
	attr.bitrate.attr.u32SrcFrmRate = fixSrcFrmRate
	attr.bitrate.attr.fr32DstFrmRate = fixDstFrmRate
	attr.bitrate.attr.u32MaxBitRate = fixMaxBitrate
	attr.bitrate.attr.u32BitRate = fixBitrate
	attr.encoding.attr.u32PicWidth = fixPicWidth
	attr.encoding.attr.u32PicHeight = fixPicHeight
	attr.encoding.attr.u32MaxPicWidth = fixMaxPicWidth
	attr.encoding.attr.u32MaxPicHeight = fixMaxPicHeight

    if r then
        return true
    end

    _dbg.Error( "ENCODER", "Fail to set encoding parameters, for channel " .. self.id .. ", reason:", errmsg )
end

function channel_meta:GetAttr()
    local x = self.vencChannel:GetAttr()
    local pt = require "pl.pretty"
    pt.dump(x)
    return x
end

function channel_meta:SetColor2Grey(en)
    if self.vencChannel then
        return self.vencChannel:SetColor2Grey(en)
    end
end

function channel_meta:GetColor2Grey()
    if self.vencChannel then
        return self.vencChannel:GetColor2Grey()
    end
end

function channel_meta:RequestIDR(now)
	if self.vencChannel then
		self.vencChannel:RequestIDR(now)
	end
end

function channel_meta:BindSourceChannel(srcChan)
	if not self.initialized then
		self.srcChannel = srcChan
	else
		_dbg.Warn( "ENCODER", "Current encoding channel is already initialized, so can't bind video source channel again!" )
	end
end
channel_meta.__index = channel_meta


function _M.CreateChannel(chanId, srcChan)
	chanId = chanId or 1
	if chanId < 1 or chanId > _M.MAX_CHANNELS then
		_dbg.Error( "ENCODER", "Invalid encoding channel ID passed. It's only support 1-" .. _M.MAX_CHANNELS .. " channels." )
		return nil
	end

	if _M.channels[chanId] then
		_dbg.Info( "ENCODER", "The requesting channel (ID=" .. chanId .. ") is already created, returns the exist channel object." )
		return _M.channels[chanId]
	end

	local chanObj = {
		id = chanId,
		srcChannel = srcChan,
		vencChannel = nil,
        osdRegion = nil,

		encodingOpt = {
			type = "H264",
			-- type = "H265",
			attr = {
				u32MaxPicWidth = 3840, --静态属性
				u32MaxPicHeight = 2160,--静态属性
				u32BufSize = 3840*2160*2,--静态属性
				u32Profile = 0, --Profile: 0: Baseline, 1: MP, 2: HP
				u32PicWidth = 720,
				u32PicHeight = 576,
				--u32PicWidth = 1920,
				--u32PicHeight = 1080,
				bByFrame = true,--静态属性
				u32BFrameNum = 0,--静态属性
				u32RefNum = 0--静态属性
			}
		},

		bitrateOpts = {
            mode = "H264CBR",
            attr = {
                u32Gop = 60,
				u32StatTime = 1,
				u32SrcFrmRate = 30,
				fr32DstFrmRate = 30,
	
				u32MaxQp = 48,   --vbr
				u32MinQp = 10,  --vbr
				u32MinIQp = 20,
				u32MaxBitRate = 8000,   --vbr
	
				u32BitRate = 4000, --cbr 
				u32FluctuateLevel = 1, -- cbr
            }
		},

		currentBrMode = "CBR"
	}

	local r = setmetatable( chanObj, channel_meta )
	_M.channels[ chanId ] = r
	return r
end

return _M