require "himpp"
local _dbg = require "coaf.debug"
local capture = require "soc.capture"
local codec, mod_OSD
local index_stream = {[1] = "main", [2] = "sub", [4] = "preview"}

local SYS = HiMPP.SYS
local VPSS = HiMPP.VPSS

local _M = {
    MAX_CHANNELS = 256,
    channels = {},
}

local function modNumber(num, mod)
    if num % mod == 0 then
        return num
    end
    return modNumber(num - 1, mod)
end

local channel_meta = {}
function channel_meta:Init()
    if self.initialized then
        return true
    end

    if not self.captureChannel then
        _dbg.Error("VPSS", "Please Init capture before VPSS!!")
        return false
    end

    self.captureChannel:Init()

    VPSS.ForceFreeGroup(self.id - 1)
    self.vpssGroup = VPSS.GetGroup(self.id - 1)
    if not self.vpssGroup then
        _dbg.Error("VPSS", "Faile to create VPSS group, for VPSSChannel ID = ", self.id)
        return
    end

    local r, errmsg = self.vpssGroup:Create({
		u32MaxW = 3840,-- NOTE: 是否一定要与 capture 捕获范围相同？
		u32MaxH = 2160,
		enPixFmt = "YUV_SEMIPLANAR_420",
		--bNrEn = true,
		enDieMode = "NODIE"
	})
	if not r then
		_dbg.Error( "VPSS", "Fail to initialize VPSS group, for VPSSChannel ID=" .. self.id .. ", error:", errmsg )
		self.vpssGroup = nil
		return
	end

    r, errmsg = SYS.Bind(self.captureChannel.vi_channel, self.vpssGroup)
    if not r then
        _dbg.Error( "VPSS", "Fail to bind VI channel to VPSS group, for VPSSChannel ID=" .. self.id .. ", reason:", errmsg )
		self.vpssGroup:Destroy()
		self.vpssGroup = nil
        return
    end
    self.initialized = true
    return true
end

function channel_meta:resizeVpss(check)
    for i, v in ipairs(codec.vi_vpss_list) do
        local stream = index_stream[i]
        if stream then
            local encWidth = _G.codec_cfg[stream].ve.u32PicWidth == 1 and _G.capture_cfg[1].stDestSize[1] or _G.codec_cfg[stream].ve.u32PicWidth
            local encHeight = _G.codec_cfg[stream].ve.u32PicHeight == 1 and _G.capture_cfg[1].stDestSize[2] or _G.codec_cfg[stream].ve.u32PicHeight
            if i == 2 then
                encWidth = math.min(_G.capture_cfg[1].stDestSize[1], 1920)
                encHeight = math.min(_G.capture_cfg[1].stDestSize[2], 1080)
            end
        
            local isWidthBigger = codec.WidthIsBigger()
            if isWidthBigger ~= (encWidth > encHeight) then
                local tmp = encWidth
                encWidth = encHeight
                encHeight = tmp
            end
            v:SetMode({
                enChnMode = "USER",
                u32Width = encWidth + 12,
                u32Height = encHeight + 12,
                -- bDouble = false,
                stFrameRate = {
                    s32SrcFrmRate = -1, s32DstFrmRate = -1
                },
                -- stAspectRatio = {
                --     enMode = "NONE", u32BgColor = 0, stVideoRect = {0,0,0,0}
                -- },
                enPixelFormat = "YUV_SEMIPLANAR_420",
                enCompressMode = "NONE"
            })
        end
    end
end

function channel_meta:setCrop(cfg)
    cfg = cfg or _G.vpss_cfg
    local is_ok = self.vpssGroup:SetCrop(cfg.enable and true or false, "ABS",{
        s32X = modNumber(math.floor(cfg.s32X * _G.capture_cfg[1].stDestSize[1] / 100), 2),
        s32Y = modNumber(math.floor(cfg.s32Y * _G.capture_cfg[1].stDestSize[2] / 100), 2),
        u32Width = modNumber(math.floor(cfg.u32Width * _G.capture_cfg[1].stDestSize[1]  / 100), 4),
        u32Height = modNumber(math.floor(cfg.u32Height * _G.capture_cfg[1].stDestSize[2] / 100),4)
    })
    if not is_ok then
        _dbg.Error("VPSS", "set crop failed !")
        return false
    end

    self:resizeVpss()

    mod_OSD.ResolutionChanged(1)
    mod_OSD.ResolutionChanged(2)
    return true
end

function channel_meta:getCrop()

end

function channel_meta:Destroy()
    self.vpssGroup:Stop()
    SYS.UnBind(self.captureChannel.vi_channel, self.vpssGroup)
    self.vpssGroup:Destroy()
    self.vpssGroup = nil
    --NOTE: 这里是否还需要对 capture 进行操作？
    self.initialized = nil
end

function channel_meta:Start()
    if not self.initialized then
        self:Init()
    end

    self.captureChannel:Start()

    if self.initialized then
        local is_ok = self.vpssGroup:Start()
        if is_ok ~= 0 then
            _dbg.Error("VPSS", "Fail to start VPSS group, VPSSChannel (ID=" .. self.id .. "), error:", string.format("%08X", is_ok) )
        end
    end
end

function channel_meta:Stop()
    if self.initialized then
        self.vpssGroup:Stop()
    end
end

function channel_meta:GetVPSSGroup()
    return self.vpssGroup
end

channel_meta.__index = channel_meta

function _M.Init()
    --NOTE: 好像没什么要做的
    -- _M.attr = capture.vi_attr
    codec = require "codec"
    mod_OSD = require "mod_OSD"
end

function _M.Destroy()
    for _, v in pairs(_M.channels) do
        v:Destroy()
    end
end

function _M.CreateChannel(chnId)
    chnId = chnId or 1
    if chnId < 1 or chnId > _M.MAX_CHANNELS then
        _dbg.Error( "VPSS", "Invalid channel ID passed. It's only support 1-" .. _M.MAX_CHANNELS .. " channels." )
        return
    end

    if _M.channels[chnId] then
        return _M.channels[chnId]
    end

    local obj = {
        id = chnId,
        captureChannel = capture.CreateChannel(chnId)
    }

    local meta = setmetatable(obj, channel_meta)
    _M.channels[chnId] = meta
    return meta
end

return _M