local _dbg = require "coaf.debug"
local cjson = require "cjson"
local pl = require "pl.pretty"
local coaf = require "coaf.core"
local thread = require "coaf.thread"
local run_daemon = require "coaf.run_daemon"


local _M = {
    id_seed = 0,
    PICPATH = "/data/storage/img/"
}

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)
function _M.GetProxy(objPath, service, loc, timeout, addr)
	local proxy
	local cli

	cli = client.New( service, loc, addr, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
    end
	return proxy
end


function _M.init()
    _dbg.Info("==pic======init")
    _M.id_seed = 0
    local k,v
    for k,v in pairs(cfg_pics ) do
        if v.id > _M.id_seed then
                _M.id_seed = v.id
        end
    end
    run_daemon:Init()
end

local function IdToUrl(id)
    --根据图片id，获取图片url
    for k,v in pairs(cfg_pics) do
        if tonumber(v.id) == tonumber(id) then
            return v.url
        end
    end
    return nil
end

local convertId, isConverting = 1
local convertList = {}
local function checkConvertList()
    if isConverting then
        return
    end
    local _, v = next(convertList)
    if v then
        isConverting = true
        v:Start()
    end
end
local function ConvertIMG(src_file)
	local newUrl = src_file .. ".rgba"
	local cmd = string.format("convert %s -depth 8 %s", src_file, newUrl)
	os.execute(cmd)

    local newId = convertId + 1
    local task = run_daemon.Create({
        name = "convert",
        run = "/usr/bin/lua",
        args = {"/usr/local/bin/convert.lua", newUrl, "rgba"},
        on_terminate = function()
            convertList[newId] = nil
            isConverting = false
            checkConvertList()
        end,
    })

    if task then
        convertList[newId] = task
        convertId = convertId + 1
    end

    coaf.ScheduleDelayedTask(function()
        checkConvertList()
    end) 
	return newUrl
end

function _M.CheckIdentifying()
    return not isConverting
end

function _M.AddPic(details)
    local _, picType = details.url:match("(.+)%.(.+)")
    if not picType or (picType ~= "jpg" and picType ~= "jpeg" and picType ~= "png") then
        return false, "invalidPicTypeError"
    end

    local count = 0
    local Cfg = cfg_pics
    for k,v in pairs(Cfg) do
        count = count + 1
        if count > 9 then
            return false, "limitPicError"
        end
        if v.name == details.name then
            _dbg.Error("picManager", "该图片名字已经存在")
            return false, "invalidPicNameError"
        end
        if v.url == details.url then
            _dbg.Error("picManager", "该图片已经存在于设备")
            return false, "samePicUrlError"
        end
    end

    _M.id_seed = _M.id_seed + 1 
    details.id = _M.id_seed 
    local src_file = _M.PICPATH .. details.url

    local size_w, size_h 
    local f = io.popen( "/usr/bin/identify '" .. src_file .. "'")
    if f then
    	local line = f:read("*l")
    	f:close()
        if line then
    		size_w, size_h = line:match(".-([0-9]+)[xX]([0-9]+)")
    		size_w = tonumber(size_w)
    		size_h = tonumber(size_h)
            if not size_w or not size_h then
                _dbg.Error("无法获取图片的宽高")
    			return false,"decodePicError"
    		end
        else
            _dbg.Error("执行identify file 无 line")
    		return false, "identifyLineError"
    	end
    else
        _dbg.Error("执行identify file 报错")
    	return false, "identifyError"
    end
    details.size_w = size_w
    details.size_h = size_h
    details.rgba = ConvertIMG(src_file)
    table.insert(Cfg, details)
    cfg_pics:__TRIGGER(true)
    return true
end

local function CallOsd(output,typ,url,position)
    ---通知 重新处理背景图或者迭加图
    local mod_relationImg = require "mod_relationImg"
    local proxy
    if typ == "bg" then
        local outputid
        if output == "1" or output == 1 then
            outputid = "hd0"
        elseif output == "2" or output == 2 then
            outputid = "hd1"
		end
		if cfg_imgRelation["output"..output][typ].enable and (cfg_imgRelation["output"..output][typ].enable == 0 or cfg_imgRelation["output"..output][typ].enable == "0" ) then
            return true           
        end
        proxy = mod_relationImg.GetProxy( "/output/"..outputid, "codec" )
    elseif typ == "osd" then
        proxy = mod_relationImg.GetProxy( "/osd", "OSD" )
    end
    if not proxy then
        _dbg.Error("picManager", "无法连接OSD服务")
		return false, "serverOSDError"
    else
        local r, done,err
        if typ == "bg" then
            r, done, err = proxy.setBackGround(url)
        elseif typ == "osd" then
            r, done, err = proxy.setLogo(tonumber(output),url,position)
        end
        proxy:Destroy()
        if not r or not done then
            _dbg.Error("picManager", "设置 输出的叠加/背景图片失败!!!!err:",err)
            return false,"setBGError"
        end
    end
    _dbg.Info("通知 重新处理背景图或者迭加图 ok")
    return true
end

function _M.ModifyPic(tab, norun)
    if 1 == 1 then
        return false, "非法调用"
    end
    if not norun then
        norun = false
    end
    local url
    if tonumber(tab.enable)==0 or tonumber(tab.id) == 0 then
        url = ""
    else
        url = IdToUrl(tonumber(tab.id))
    end
    local output = tab.output
    local type = tab.type

    if not norun then
        ---通知osd服务
        -- print("output~~~~~~~~~", output)
        local r,err = CallOsd(output,type,url,tab.position)
        if not r then
            return false,err
        end
    end
    
    cfg_imgRelation["output"..output][type].url = url
    cfg_imgRelation["output"..output][type].position = tab.position
    cfg_imgRelation:__TRIGGER(true)
    return true
end

--图片管理 页面查询使用
function _M.GetPics(type)
    local datas = {}
    if type then
        for k,v in pairs(cfg_pics) do
            if v.type == type then
                if type == "bg"  then
                    table.insert(datas, table.copy(v))
                elseif type == "osd" then
                    table.insert(datas, table.copy(v))
                end
            end
        end
    else
        datas = table.copy(cfg_pics)
    end

    for _, v in ipairs(datas) do
        v.url = "/img/" .. v.url
    end
    return datas
end

function _M.GetPicByid(id)
    if not id then
        return nil
    end
    local k,v
    for k,v in pairs(cfg_pics) do
        if v.id == id then
            return v
        end
    end
    _dbg.Info("picManager", "GetPicByid no found!")
    return nil
end

-- type,可选参数osd/bg
--根据输出的1，2 及类型来查看 所有图片
--被选中的图片则enable=1
function _M.GetOutputPics(output,type)
    local pic
    local position 
    if tonumber(output) == 1 then
        if type == "bg" then
            pic=cfg_imgRelation.output1.bg.url
        elseif type == "osd" then
            pic=cfg_imgRelation.output1.osd.url

            position=cfg_imgRelation.output1.osd.position
        end
    else
        if type == "bg" then
            pic=cfg_imgRelation.output2.bg.url
        elseif type == "osd" then
            pic=cfg_imgRelation.output2.osd.url
            position=cfg_imgRelation.output2.osd.position
        end
    end
    local data = {}
    for k,pictabs in pairs(cfg_pics) do
        pictabs.enable = 0
        if pictabs.type == type then
            if pictabs.url == pic  then
                pictabs.enable=1
            end
            table.insert(data,pictabs)
        end
    end

    return data,position
end


--首页的输出 对应的图片地址
function _M.GetBGPic(output)
    local pic = ""
    if tonumber(output) == 1 then
        pic=cfg_imgRelation.output1.bg.url
    else
        pic=cfg_imgRelation.output2.bg.url
    end
    return pic
end

function _M.RemovePic(id)
    local Cfg = cfg_pics
    local img_name

    --TODO: 如果正在叠加这个图片，那么不能删除
    for k,v in pairs(Cfg) do
        if tonumber(v.id) == tonumber(id) then
            img_name=v.url
            -- local mod_relationImg = require "mod_relationImg"
            -- local used_enable = mod_relationImg.CheckPicUsed(img_name)
            -- if used_enable == 1 then
            --     _dbg.Error("图片被叠加，正在使用,不能删除!",img_name)
            --     return false , "图片已经被使用,不能删除!"
            -- end
            local OSDProxy = _M.GetProxy("/osd", "codec")
            if not OSDProxy then
                return false, "Get OSD Proxy Failed."
            end

            local is_ok, skip = OSDProxy.RemoveOSDPic(id)
            if not is_ok or not skip then
                return false, not is_ok and "OSD Proxy Request Failed." or "Using this picture so can't delete."
            end
            table.remove(Cfg,k)
            break
        end
    end
    cfg_pics:__TRIGGER(true)
    img_name = _M.PICPATH .. img_name

    coaf.ScheduleDelayedTask(function()
        os.execute("/bin/rm -rf " .. img_name)
        os.execute("/bin/rm -rf " .. img_name..".rgba")
    end)
    
    return true
end

--根据分辨率 对图片进行处理 --bg
function _M.ResizePic(rate,picName,newfile)
    _dbg.Info("convert -quality 100 -resize '".. rate .. "!' " .._M.PICPATH..  picName .."  " .. newfile )
    local r, code = os.execute("convert -quality 100 -resize '".. rate .. "!' " .._M.PICPATH..  picName .."  " .. newfile )
    if not r then
        return false,code
    end
    return true
end


--图片转RGBA
function _M.ConvertPic(picName,newfile)
    _dbg.Info("convert  " .. picName .. " -depth 8 " .. "  " .. newfile)
    local r, code = os.execute("convert  " .. picName .. " -depth 8" .. " " .. newfile)
    if not r then
        return false,code
    end
    return true
end

--获取指定图片的大小
function _M.GetPicRate(src_file)
    local datas = {}
    for k,v in pairs(cfg_pics) do
        if v.url == src_file then
            return v.size_w,v.size_h
        end
    end
    return nil,nil,"findPicError"
end

function _M.RemoveAllPic()
    local Cfg = cfg_pics
    for k,v in pairs(Cfg) do      
        _M.RemovePic(tonumber(v.id))
    end
    return true
end

function _M.GetIMGDesc(fileID)
    for k, v in pairs(cfg_pics) do
        if tonumber(fileID) == tonumber(v.id) then
            return v.size_w, v.size_h, v.rgba, "/img/" .. v.url, v.name
        end
    end
end

return _M
