#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

package.path = package.path .. ";../?.lua"

--Why this? Since the OnvifSvc module only support single thread
--driven method.

local socket = require "patch_socket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local roll = require "coaf.log.rollfile"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /data/log/" )
	local logF = roll.Create("/var/log/onvif.log")
	if not logF or not logF:init() then
		_dbg.Error("ONVIF", "Fail to open log file: /data/log/onvif.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "onvif" )

_G.print = function(...)
	_dbg.Info1( "ONVIF", ... )
end

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "ONVIF" )
	stdio.Start()
end

require "onvif_def"
require "OnvifSvc"

local global_info = require "globalInfo"
--------
-- Make the device info as global.
_G.GLOBAL_INFO = global_info
_G.REBOOT_REQUIRE = false

--------
-- local ptz_mod = require "ptzModule"
local onvif_mod
local cosvc_ONVIF
local period_timer
local idle_task
--------

service.InitSocket()
client.InitSocket( socket, adapter )

local function periodCheck()
	-- ptz_mod.Check() -- To check and try create
end

local function idleCheck()
	OnvifSvc.Polling(10)	--For 10ms timeout polling.
	if _G.REBOOT_REQUIRE then
		-- reset.Reboot()
		os.execute( "/sbin/reboot" )
	end
end
function split(str, sep)
    local sep, fields = sep or "\t", {}
    local pattern = string.format("([^%s]+)", sep)
    str:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end

local function Main()
	local onvif_mod = require "onvifModule"

	cosvc_ONVIF = service.New( "onvif", "unix:/var/run/onvif.sock" )
	if cosvc_ONVIF then
		_dbg.Info( "ONVIF", "CoService of 'onvif' is created." )
		cosvc_ONVIF:CreateCoServerProxy()
	else
		_dbg.Error( "ONVIF", "Fail created CoService!" )
	end

	onvif_mod.Init( cosvc_ONVIF )
	-- ptz_mod.Init( cosvc_ONVIF, coaf.CoAF_LOOP )

	-- idle_task = coaf.NewIdle( idleCheck )
	local thread = require "coaf.thread"
	thread.NewTimer(function()
		idleCheck()
	end, 0.05, 0.05):start()
	period_timer = coaf.NewTimer( periodCheck, 0.01, 3 )
	if idle_task then
		idle_task:start()
	end
	if period_timer then
		period_timer:start()
	end
end

--------
coaf.Start(Main)
--------
_dbg.Info( "ONVIF", "ONVIF service is terminated." )

