local _M = {}
local GetProxy = require "GetProxy"

local AllAudioSources = {
	[1] = {
		token = "000",
		Channels = 2
	}
}

local AllVideoSources = {
	[1] = {
		token = "000",
		Framerate = 60,
		Resolution = { Width = 1920, Height = 1080 },
		Imaging = { Brightness = 0, ColorSaturation = 0, Contrast = 0 },
	}
}

local AllVideoSourceConfigs = {
	[1] = {
		token = "000",
		Name = "VideoSourceInput",
		UseCount = 2,
		SourceToken = "000",
		Bounds = {
			x = 0, y = 0, width=1920, height=1080
		},
		Extension = {
			Rotate = {
				Mode = tt__RotateMode__OFF, Degree = 0
			}
		}
	}
}

local AllAudioSourceConfigs = {
	[1] = {
		token = "000",
		Name = "AudioSourceInput",
		UseCount = 2,
		SourceToken = "000",
	}
}

function _M.UpdateAVSourcesAndConfigs()
	local proxy = GetProxy("/capture", "codec")
	local is_ok, data = proxy.GetInfo()
	if is_ok and data and data[1] then
		local info = data[1]
		AllVideoSources[1].Framerate = info.sourceFrameRate
		local w, h = info.sourceSize:match("(%d+)x(%d+)")
		AllVideoSources[1].Resolution = { Width = w, Height = h }
		AllVideoSources[1].Imaging = { 
			Brightness = info.brightness,
			ColorSaturation = info.saturation,
			Contrast = info.contrast,
		}
		
		AllVideoSourceConfigs[1].Bounds = { x = 0, y = 0, width = w, height = h }
		
		local rmf = gets and gets["avio.video.RMF"] or "0"
		if rmf == "0" then
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__OFF, Degree = 0 }
		elseif rmf == "mirror" or rmf == "flip" then
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__AUTO, Degree = (rmf == "flip" and 180 or 0) }
		else
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__ON, Degree = tonumber(rmf) }
		end

		if info.rotate == 0 then
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__OFF, Degree = 0}
		else
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__ON, Degree = tonumber(info.rotate) }
		end
	end
	local signals = nil--busOprs.GetSignalStatus()
	if signals then
		local cc, gets = busOprs.Get( {
			["avio.video.brightness"] = 128,
			["avio.video.contrast"] = 128,
			["avio.video.saturation"] = 128,
			["avio.video.hue"] = 128,
			["avio.video.RMF"] = "0"
		} )

		AllVideoSources[1].Framerate = math.floor(signals.video["video.frame_rate"]) or 0
		if AllVideoSources[1].Framerate <= 0 then
			AllVideoSources[1].Framerate = 50 --TODO: Here.
		end
		
		AllVideoSources[1].Resolution = { Width = signals.video["video.width"] or 0, Height = signals.video["video.height"] or 0 }
		AllVideoSources[1].Imaging = { 
			Brightness = gets and gets["avio.video.brightness"] or 128,
			ColorSaturation = gets and gets["avio.video.saturation"] or 128,
			Contrast = gets and gets["avio.video.contrast"] or 128,
		}
		
		AllVideoSourceConfigs[1].Bounds = { x = 0, y = 0, width=signals.video["video.width"] or 0, height = signals.video["video.height"] or 0 }
		
		local rmf = gets and gets["avio.video.RMF"] or "0"
		if rmf == "0" then
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__OFF, Degree = 0 }
		elseif rmf == "mirror" or rmf == "flip" then
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__AUTO, Degree = (rmf == "flip" and 180 or 0) }
		else
			AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__ON, Degree = tonumber(rmf) }
		end
	end
end
----------------------------------------------------------

local FixedProfiles = {
	[1] = {
		token = "Profile_Main_00",
		Name = "MainStream_Profile",
		fixed = 1,
		
		VideoSourceConfiguration = AllVideoSourceConfigs[1],
		AudioSourceConfiguration = AllAudioSourceConfigs[1],
		
		VideoEncoderConfiguration = {
			Name = "MainVideoEncoder",
			UseCount = 1,
			token = "000",
			Encoding = tt__VideoEncoding__H264,
			
			Resolution = {
				Width = 1920,
				Height = 1080
			},
			
			Quality = 5,
			
			RateControl = { 
				FrameRateLimit = 30,
				EncodingInterval = 1,
				BitrateLimit = 2000
			},

			H264 = {
				GovLength = 30,
				H264Profile = tt__H264Profile__High
			},

			Multicast = {
				Address = {
					Type = tt__IPType__IPv4,
					IPv4Address = "224.0.0.1"
				},
				Port = 50000,
				TTL = 127,
				AutoStart = 0,
			},

			SessionTimeout = 0x7fffffff,
		},
		
		AudioEncoderConfiguration = {
			Name = "MainAudioEncoder",
			UseCount = 1,
			token = "000",
			Encoding = tt__AudioEncoding__AAC,
			Bitrate = 64000,
			SampleRate = 48000,
			Multicast = {
				Address = {
					Type = tt__IPType__IPv4,
					IPv4Address = "224.0.0.1"
				},
				Port = 60000,
				TTL = 127,
				AutoStart = 0,
			},
			
			SessionTimeout = 0x7fffffff,
		},
	},
	
	[2] = {
		token = "Profile_Sub_00",
		Name = "SubStream_Profile",
		fixed = 1,
		
		VideoSourceConfiguration = AllVideoSourceConfigs[1],
		AudioSourceConfiguration = AllAudioSourceConfigs[1],
		
		VideoEncoderConfiguration = {
			Name = "SubVideoEncoder",
			UseCount = 1,
			token = "001",
			Encoding = tt__VideoEncoding__H264,
			
			Resolution = {
				Width = 960,
				Height = 540
			},
			
			Quality = 5,
			
			RateControl = { 
				FrameRateLimit = 30,
				EncodingInterval = 1,
				BitrateLimit = 512
			},

			H264 = {
				GovLength = 30,
				H264Profile = tt__H264Profile__High
			},

			Multicast = {
				Address = {
					Type = tt__IPType__IPv4,
					IPv4Address = "224.0.0.1"
				},
				Port = 50002,
				TTL = 127,
				AutoStart = 0,
			},

			SessionTimeout = 0x7fffffff,
		},
		
		AudioEncoderConfiguration = {
			Name = "SubAudioEncoder",
			UseCount = 1,
			token = "001",
			Encoding = tt__AudioEncoding__AAC,
			Bitrate = 64000,
			SampleRate = 48000,
			Multicast = {
				Address = {
					Type = tt__IPType__IPv4,
					IPv4Address = "224.0.0.1"
				},
				Port = 60002,
				TTL = 127,
				AutoStart = 0,
			},
			
			SessionTimeout = 0x7fffffff,
		},
	}
}


function _M.UpdateAVEncoderConfigs(spec)
	local proxy = GetProxy("/codec", "codec")
	local is_ok, data = proxy.GetInfo()
	proxy:Destroy()
	if not is_ok then
		return
	end
	if not spec or spec == 1 then --Update main stream
		local info = data.main
		
		local isH264 = false
		if info.type == "H264" then
			FixedProfiles[1].VideoEncoderConfiguration.Encoding = tt__VideoEncoding__H264
			isH264 = true
		elseif info.type == "H265" then
			FixedProfiles[1].VideoEncoderConfiguration.Encoding = tt__VideoEncoding__H265
		end
		
		FixedProfiles[1].VideoEncoderConfiguration.Resolution = {
			Width = info.ve.u32PicWidth == 1 and info.capSize[1] or info.ve.u32PicWidth,
			Height = info.ve.u32PicHeight == 1 and info.capSize[2] or info.ve.u32PicHeight
		}
		

		FixedProfiles[1].VideoEncoderConfiguration.RateControl.FrameRateLimit = info.rc.fr32DstFrmRate
		FixedProfiles[1].VideoEncoderConfiguration.RateControl.EncodingInterval = 0
		
		FixedProfiles[1].VideoEncoderConfiguration.RateControl.BitrateLimit = info.rc.u32BitRate
		FixedProfiles[1].VideoEncoderConfiguration.Quality = 50
		
		if isH264 then
			FixedProfiles[1].VideoEncoderConfiguration.H264.GovLength = info.rc.u32Gop
			local x = tonumber(info.ve.u32Profile)
			if x == 0 then
				FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Baseline
			elseif x == 1 then
				FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Main
			elseif x == 2 then
				FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__High
			end		

		end
		
		-- if streamSt["audio.codec"] == "AAC" or streamSt["audio.codec"] == "MPEG4-GENERIC" then
		-- 	FixedProfiles[1].AudioEncoderConfiguration.Encoding = tt__AudioEncoding__AAC
		-- elseif streamSt["audio.codec"] == "G711" or streamSt["audio.codec"] == "PCMU" or streamSt["audio.codec"] == "PCMA" then
		-- 	FixedProfiles[1].AudioEncoderConfiguration.Encoding = _G.tt__AudioEncoding__G711
		-- else
		-- 	FixedProfiles[1].AudioEncoderConfiguration.Encoding = _G.tt__AudioEncoding__G711	--TODO:
		-- end
		-- FixedProfiles[1].AudioEncoderConfiguration.Bitrate = ( streamSt["audio.bitrate"] or 0 )/1000
		-- FixedProfiles[1].AudioEncoderConfiguration.SampleRate = streamSt["audio.sampling"] or 0

		local streamSt = nil-- busOprs.GetStreamStatus( "00" )
		if streamSt then
			local cc, gets = busOprs.Get({
				["encoder.stream[0].video.framerate"] = 0,
				["encoder.stream[0].video.bitrate"] = 0,
				["encoder.stream[0].video.profile"] = "baseline",
				["encoder.stream[0].video.gop_size"] = 30,
				["encoder.stream[0].video.quality"] = 0,
			})
			
			local isH264 = false
			if streamSt["video.codec"] == "H264" or streamSt["video.codec"] == "h264" or streamSt["video.codec"] == "H.264" or streamSt["video.codec"] == "h.264" then
				FixedProfiles[1].VideoEncoderConfiguration.Encoding = tt__VideoEncoding__H264
				isH264 = true
			elseif streamSt["video.codec"] == "JPEG" or streamSt["video.codec"] == "jpeg" or streamSt["video.codec"] == "MJPEG" or streamSt["video.codec"] == "mjpeg" then
				FixedProfiles[1].VideoEncoderConfiguration.Encoding = tt__VideoEncoding__JPEG
			end
			
			FixedProfiles[1].VideoEncoderConfiguration.Resolution = {
				Width = streamSt["video.width"] or 0,
				Height = streamSt["video.height"] or 0
			}
			
			local x
			x = gets and gets["encoder.stream[0].video.framerate"] or 0
			if x <= 0 then
				FixedProfiles[1].VideoEncoderConfiguration.RateControl.FrameRateLimit = AllVideoSources[1].Framerate
				FixedProfiles[1].VideoEncoderConfiguration.RateControl.EncodingInterval = 1
			elseif x == 0.5 then
				FixedProfiles[1].VideoEncoderConfiguration.RateControl.EncodingInterval = 2
				if AllVideoSources[1].Framerate <= 30 then
					FixedProfiles[1].VideoEncoderConfiguration.RateControl.FrameRateLimit = AllVideoSources[1].Framerate
				else
					FixedProfiles[1].VideoEncoderConfiguration.RateControl.FrameRateLimit = math.floor(AllVideoSources[1].Framerate/2)
				end
			else
				FixedProfiles[1].VideoEncoderConfiguration.RateControl.FrameRateLimit = x
				FixedProfiles[1].VideoEncoderConfiguration.RateControl.EncodingInterval = 0
			end
			
			FixedProfiles[1].VideoEncoderConfiguration.RateControl.BitrateLimit = (gets and gets["encoder.stream[0].video.bitrate"] or 0)/1000
			FixedProfiles[1].VideoEncoderConfiguration.Quality = gets and gets["encoder.stream[0].video.quality"] or 50
			
			if isH264 then
				FixedProfiles[1].VideoEncoderConfiguration.H264.GovLength = gets and gets["encoder.stream[0].video.gop_size"] or 30
				x = gets and gets["encoder.stream[0].video.profile"] or "baseline"
				if x == "base" or x == "baseline" then
					FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Baseline
				elseif x == "main" then
					FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Main
				elseif x == "high" then
					FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__High
				else
					FixedProfiles[1].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Baseline
				end				
			end
			
			if streamSt["audio.codec"] == "AAC" or streamSt["audio.codec"] == "MPEG4-GENERIC" then
				FixedProfiles[1].AudioEncoderConfiguration.Encoding = tt__AudioEncoding__AAC
			elseif streamSt["audio.codec"] == "G711" or streamSt["audio.codec"] == "PCMU" or streamSt["audio.codec"] == "PCMA" then
				FixedProfiles[1].AudioEncoderConfiguration.Encoding = _G.tt__AudioEncoding__G711
			else
				FixedProfiles[1].AudioEncoderConfiguration.Encoding = _G.tt__AudioEncoding__G711	--TODO:
			end
			FixedProfiles[1].AudioEncoderConfiguration.Bitrate = ( streamSt["audio.bitrate"] or 0 )/1000
			FixedProfiles[1].AudioEncoderConfiguration.SampleRate = streamSt["audio.sampling"] or 0
		end
	end
	
	if not spec or spec == 2 then --Update sub stream
		local info = data.sub
		
		local isH264 = false
		if info.type == "H264" then
			FixedProfiles[2].VideoEncoderConfiguration.Encoding = tt__VideoEncoding__H264
			isH264 = true
		elseif info.type == "H265" then
			FixedProfiles[2].VideoEncoderConfiguration.Encoding = tt__VideoEncoding__H265
		end
		
		FixedProfiles[2].VideoEncoderConfiguration.Resolution = {
			Width = info.ve.u32PicWidth == 1 and info.capSize[1] or info.ve.u32PicWidth,
			Height = info.ve.u32PicHeight == 1 and info.capSize[2] or info.ve.u32PicHeight
		}
		

		FixedProfiles[2].VideoEncoderConfiguration.RateControl.FrameRateLimit = info.rc.fr32DstFrmRate
		FixedProfiles[2].VideoEncoderConfiguration.RateControl.EncodingInterval = 0
		
		FixedProfiles[2].VideoEncoderConfiguration.RateControl.BitrateLimit = info.rc.u32BitRate
		FixedProfiles[2].VideoEncoderConfiguration.Quality = 50
		
		if isH264 then
			FixedProfiles[2].VideoEncoderConfiguration.H264.GovLength = info.rc.u32Gop
			local x = tonumber(info.ve.u32Profile)
			if x == 0 then
				FixedProfiles[2].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Baseline
			elseif x == 1 then
				FixedProfiles[2].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__Main
			elseif x == 2 then
				FixedProfiles[2].VideoEncoderConfiguration.H264.H264Profile = tt__H264Profile__High
			end		

		end
		
		-- if streamSt["audio.codec"] == "AAC" or streamSt["audio.codec"] == "MPEG4-GENERIC" then
		-- 	FixedProfiles[1].AudioEncoderConfiguration.Encoding = tt__AudioEncoding__AAC
		-- elseif streamSt["audio.codec"] == "G711" or streamSt["audio.codec"] == "PCMU" or streamSt["audio.codec"] == "PCMA" then
		-- 	FixedProfiles[1].AudioEncoderConfiguration.Encoding = _G.tt__AudioEncoding__G711
		-- else
		-- 	FixedProfiles[1].AudioEncoderConfiguration.Encoding = _G.tt__AudioEncoding__G711	--TODO:
		-- end
		-- FixedProfiles[1].AudioEncoderConfiguration.Bitrate = ( streamSt["audio.bitrate"] or 0 )/1000
		-- FixedProfiles[1].AudioEncoderConfiguration.SampleRate = streamSt["audio.sampling"] or 0

	end
end

function _M.GetAllProfiles()
	local ptzConfigs = require "ptzConfigs"
	local i
	for i=1, #FixedProfiles do
		FixedProfiles[i].PTZConfiguration = ptzConfigs.GetConfig("000")
	end
	_M.UpdateAVSourcesAndConfigs()
	_M.UpdateAVEncoderConfigs()
	return FixedProfiles
end

-----------------------------------------

function _M.GetProfile(token)
	local ptzConfigs = require "ptzConfigs"

	local i
	for i=1, #FixedProfiles do
		if FixedProfiles[i].token == token then
			FixedProfiles[i].PTZConfiguration = ptzConfigs.GetConfig("000")
			_M.UpdateAVSourcesAndConfigs()
			_M.UpdateAVEncoderConfigs(i)
			return FixedProfiles[i]
		end
	end
	return nil
end

-----------------------------------------
function _M.GetAllVideoSources()
	_M.UpdateAVSourcesAndConfigs()
	return AllVideoSources
end

-----------------------------------------
function _M.GetAllAudioSources()
	_M.UpdateAVSourcesAndConfigs()
	return AllAudioSources
end

-----------------------------------------
function _M.GetAllVideoSourceConfigs()
	_M.UpdateAVSourcesAndConfigs()
	return AllVideoSourceConfigs
end

-----------------------------------------
function _M.GetVideoSourceConfig(token)
	local i
	_M.UpdateAVSourcesAndConfigs()
	for i=1, #AllVideoSourceConfigs do
		if AllVideoSourceConfigs[i].token == token then
			return AllVideoSourceConfigs[i]
		end
	end
	return nil
end

-----------------------------------------
function _M.GetAllAudioSourceConfigs()
	_M.UpdateAVSourcesAndConfigs()
	return AllAudioSourceConfigs
end

-----------------------------------------
function _M.GetAudioSourceConfig(token)
	local i
	_M.UpdateAVSourcesAndConfigs()
	for i=1, #AllAudioSourceConfigs do
		if AllAudioSourceConfigs[i].token == token then
			return AllAudioSourceConfigs[i]
		end
	end
	return nil
end

-----------------------------------------
return _M

