#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local roll = require "coaf.log.rollfile"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /data/log/" )
	local logF = roll.Create("/var/log/ptz.log")
	if not logF or not logF:init() then
		_dbg.Error("PTZ", "Fail to open log file: /var/log/ptz.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "ptz" )

_G.print = function(...)
	_dbg.Info1( "PTZ", ... )
end

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "PTZ" )
	stdio.Start()
end

--------
local ptz_mod = require "ptzModule"
local period_timer
--------

service.InitSocket()
client.InitSocket( nil, adapter )

local function periodCheck()
	ptz_mod.Check() -- To check and try create
end

function split(str, sep)
    local sep, fields = sep or "\t", {}
    local pattern = string.format("([^%s]+)", sep)
    str:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end

local function Main()
	cosvc_ONVIF = service.New( "ptz", "unix:/var/run/ptz.sock" )
	if cosvc_ONVIF then
		_dbg.Info( "PTZ", "CoService of 'ptz' is created." )
		cosvc_ONVIF:CreateCoServerProxy()
	else
		_dbg.Error( "PTZ", "Fail created CoService!" )
	end

	ptz_mod.Init( cosvc_ONVIF, coaf.CoAF_LOOP )

	period_timer = coaf.NewTimer( periodCheck, 0.01, 3 )
	if period_timer then
		period_timer:start()
	end
end

--------
coaf.Start(Main)
--------
_dbg.Info( "PTZ", "PTZ service is terminated." )

