print("Running File __tds__GetSystemDateAndTime.lua")
local function ParseTime(str)
	local y,m,d,h,i,s = str:match( "(%d+)%-(%d+)%-(%d+)%s+(%d+):(%d+):(%d+)" )
	return {
		year = tonumber(y),
		month = tonumber(m),
		day = tonumber(d),
		hour = tonumber(h),
		min = tonumber(i),
		sec = tonumber(s)
	}
end

local local_time = nil--ParseTime( timeinfo["local"] )
local local_f = io.popen([[/bin/date +%F\ %T]])
if local_f then
	local l = local_f:read("l")
	local_f:close()
	local_time = ParseTime( l )
end

local utc_time = nil--ParseTime( timeinfo["utc"] )
local utc_f = io.popen([[/bin/date -u +%F\ %T]])
if utc_f then
	local l = utc_f:read("l")
	utc_f:close()
	
	utc_time = ParseTime( l )
end

local tz
local GetProxy = require "GetProxy"
local proxy = GetProxy("/datetime", "systemctrl")
if proxy then
	local is_ok, location, diff = proxy.GetLocation()
	proxy:Destroy()
	if is_ok then
		local hour, min, sign
		if diff > 0 then
			sign = "+"
		else
			sign = "-"
		end

		diff = math.abs(diff)
		if diff ~= math.ceil(diff) then
			hour = math.floor( diff )
			min = math.abs(60 * (diff%1))
		else
			min = 0
			hour = diff
		end
		tz = "UTC" .. sign .. string.format("%02d", hour) .. ":" .. string.format("%02d", min)
		print("==========tz = ", tz)
	end
end

return {
	SystemDateAndTime = {
		DateTimeType = tt__SetDateTimeType__Manual,
		DaylightSavings = 0,
		TimeZone = { TZ = tz },
		UTCDateTime = {
			Date = {
				Year = utc_time.year,
				Month = utc_time.month,
				Day = utc_time.day,
			},
			
			Time = {
				Hour = utc_time.hour,
				Minute = utc_time.min,
				Second = utc_time.sec
			}
		},
		
		LocalDateTime = {
			Date = {
				Year = local_time.year,
				Month = local_time.month,
				Day = local_time.day,
			},
			
			Time = {
				Hour = local_time.hour,
				Minute = local_time.min,
				Second = local_time.sec
			}
		}
	}
}

--NOTE: SVC_OK = 0, SVC_NOIMPL = 1

