print("Running File __trt__GetStreamUri.lua")
local args = {...}
local request = args[1]
if not request then
	error( "Invalid request argument passed!" )
end

local multicast = (request.StreamSetup.Stream == tt__StreamType__RTP_Multicast)

local isMainStream = false
if not request.ProfileToken or request.ProfileToken:match( "Profile_Main" ) then
	isMainStream = true
end

local svcPort
local sessId

local GetProxy = require "GetProxy"
local proxy = GetProxy("/streamer", "codec")
local mainData, subData
if proxy then
	local is_ok, data = proxy.GetInfo()
	if is_ok then
		for _, v in ipairs(data.main) do
			if v.type == "rtsp" then
				mainData = v
				break
			end
		end

		for _, v in ipairs(data.sub) do
			if v.type == "rtsp" then
				subData = v
				break
			end
		end
	end
	proxy:Destroy()
end

if isMainStream then
	svcPort = mainData and mainData.port
	sessId = mainData and mainData.session
else
	svcPort = subData and subData.port
	sessId = subData and subData.session
end

if request.StreamSetup.Transport.Protocol == tt__TransportProtocol__UDP then
	local rtspUri = "rtsp://" .. GLOBAL_INFO.GetHostAddress( svcPort ) .. "/" .. sessId 
	
	if multicast then
		rtspUri = rtspUri .. "/ssm"
	end
	return {
		MediaUri = {
			Uri = rtspUri,
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
elseif request.StreamSetup.Transport.Protocol == tt__TransportProtocol__TCP then
	local rtspUri = "rtsp://" .. GLOBAL_INFO.GetHostAddress( svcPort ) .. "/" .. sessId 
	
	if multicast then
		rtspUri = rtspUri .. "/ssm"
	end
	return {
		MediaUri = {
			Uri = rtspUri,
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
elseif request.StreamSetup.Transport.Protocol == tt__TransportProtocol__RTSP then
	local rtspUri = "rtsp://" .. GLOBAL_INFO.GetHostAddress( svcPort ) .. "/" .. sessId 
	
	if multicast then
		rtspUri = rtspUri .. "/ssm"
	end
	
	return {
		MediaUri = {
			Uri = rtspUri,
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
elseif request.StreamSetup.Transport.Protocol == tt__TransportProtocol__HTTP then
	return {
		MediaUri = {
			Uri = "", --TODO
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
else
	return {
		MediaUri = {
			Uri = "", --TODO
			InvalidAfterConnect = 1,
			InvalidAfterReboot = 1,
			Timeout = 0
		}
	}
end

--TODO: Check the 'request' arguments, and fill the 'response' arguments, see <same-name>.txt file for reference.

--[==[ *ABSTRACT* The 'request' arguments:
{
	StreamSetup: (!)
	{
		Stream: (!)
		(
			tt__StreamType__RTP_Unicast: 0,
			tt__StreamType__RTP_Multicast: 1,
		)
		Transport: (!)
		{
			Protocol: (!)
			(
				tt__TransportProtocol__UDP: 0,
				tt__TransportProtocol__TCP: 1,
				tt__TransportProtocol__RTSP: 2,
				tt__TransportProtocol__HTTP: 3,
			)
			Tunnel: 
			{...}
		}
		__size: <int>
		__any: <_XML>
		__anyAttribute: <_XML>
	}
	ProfileToken: <char*> (!)
}
]==]

--[==[ *ABSTRACT* The 'response' arguments:
{
	MediaUri: (!)
	{
		Uri: <char*> (!)
		InvalidAfterConnect: (!)
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		InvalidAfterReboot: (!)
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		Timeout: <LONG64> (!)
		__size: <int>
		__any: <_XML>
		__anyAttribute: <_XML>
	}
}
]==]


