local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local client = require "coaf.client"

local LED = require "ui_leds"
local SW = require "ui_switchs"

local _M = {
	lastAlarmTime = {}
}

local testing = false

local function restoreCallback()
	if not testing then
		local cli = client.New( "systemctrl" )
		if cli then
			local proxy = cli:CreateProxy( "/", true )	--exclusive.
			if proxy then
				proxy.RestoreFactory()
				proxy:Destroy()
			end
			cli:Destroy()
		end
	end
end


function _M.Init()
	LED.Init()
	SW.Init()
	
	
	SW.CreateSW( "restore", { --GPIO0_0
		pingroup = 0x12150000,
		pin = 0,
		holdTimes = 24,
		callback = restoreCallback
	})
	
	LED.CreateLight( "run", { --GPIO0_2
		pingroup = 0x12150000,
		pin = 2
	})

end

function _M.EnterResetStatus()
	LED.Force( "signal1", LED.BLINK_VERYFAST )
	LED.Force( "signal2", LED.BLINK_VERYFAST )
	LED.Force( "txd", LED.BLINK_VERYFAST )
	LED.Force( "rxd", LED.BLINK_VERYFAST )
	LED.SetLightStatus( "beep", LED.ON, 4 )
end

function _M.EnterRestoreStatus()
	LED.Force( "signal1", LED.BLINK_VERYFAST )
	LED.Force( "signal2", LED.BLINK_VERYFAST )
	LED.Force( "txd", LED.OFF )
	LED.Force( "rxd", LED.OFF )
end

function _M.EnterRebootStatus()
	LED.Force( "signal1", LED.BLINK_VERYFAST )
	LED.Force( "signal2", LED.BLINK_VERYFAST )
	LED.Force( "txd", LED.OFF )
	LED.Force( "rxd", LED.OFF )
	LED.SetLightStatus( "beep", LED.ON, 8 )
end

function _M.NormalStatus()
	LED.NoForce( "signal1" )
	LED.NoForce( "signal2" )
	LED.NoForce( "txd" )
	LED.NoForce( "rxd" )
end

function _M.ForceLED( led, status, offAfter )
	if type(status) == "string" then
		LED.Force( led, LED[status] or LED.OFF, offAfter )
	else
		LED.Force( led, status, offAfter )
	end
end

function _M.SetLED( led, status, offAfter )
	if type(status) == "string" then
		LED.SetLightStatus( led, LED[status] or LED.OFF, offAfter )
	else
		LED.SetLightStatus( led, status, offAfter )
	end
end

function _M.Testing(onOff)
	testing = onOff
end

function _M.ChangeAlarmPolarity(chnId, pol)
	if chnId == 1 then
		SW.ChangeSWPolarity( "alarm1", pol )
	else
		SW.ChangeSWPolarity( "alarm2", pol )
	end
end

function _M.TestAlarm(chnId)
	-- alarmCallback( chnId == 2 and "alarm2" or "alarm1" )
	return true
end

return _M

