-- local pt = require 'pl.pretty'
local cjson = require "cjson"
local websocket_coaf = require "websocket_coaf"

local _M = {
    clients = {},
    data = {}
}

function _M.get_module_data(modules)
    local result = {
        code = 200,
        message = 'success',
        data = {}
    }

    local update = {}  --NOTE:这里放需要去更新的数据模块名

    if modules then
        for _, v in ipairs(modules) do
            result.data[v] = _M.data[v]
            if not _M.data[v] or not next(_M.data[v]) then
                update[v] = true
            end
        end
    else
        result.data = _M.data
    end

    for module in pairs(update) do
        websocket_coaf.request_data(module)
        result.data[module] = _M.data[module]
    end

    return cjson.encode(result)
end

local function error_json()
    local result = {
        code = 500,
        message = 'json 参数错误!',
        data = {}
    }
    return cjson.encode(result)
end

local function create_new_id()
    local template ="xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
    local d = io.open("/dev/urandom", "r"):read(4)
    math.randomseed(os.time() + d:byte(1) + (d:byte(2) * 256) + (d:byte(3) * 65536) + (d:byte(4) * 4294967296))
    return string.upper(string.gsub(template, "x", function (c)
        local v = (c == "x") and math.random(0, 0xf) or math.random(8, 0xb)
        return string.format("%x", v)
    end
    ))
end

function _M.send_msg(msg, client_id)
    local aims = {}
    if client_id then
        aims[client_id] = _M.clients[client_id]
    else
        aims = _M.clients
    end

    for _, v in pairs(aims) do
        v.ws:send(msg)
    end
end

local function handle_request(request, client_id)
    local modules
    if type(request.modules) == "string" then
        modules = {request.modules}
    elseif type(request.modules) == "table" then
        modules = request.modules
    end

    _M.send_msg(_M.get_module_data(modules), client_id)
end

local function echo_handler(ws)
    local client_id = create_new_id()

    _M.clients[client_id] = {
        ws = ws,
        ws_send_default_num = 0
    }

    _M.send_msg(_M.get_module_data(), client_id)

    ws:on_message(function(_, message)
        local ok, request = xpcall(cjson.decode, debug.traceback, message)
        if not ok then 
            _M.send_msg(error_json(), client_id)
            return
        end

        if type(request) == "table" then 
            handle_request(request, client_id)
        else
            _M.send_msg(error_json(), client_id)
        end
    end)

    ws:on_close(function()
        ws:close()
        _M.clients[client_id] = nil
        collectgarbage()
    end)
end

function _M.Init()
    _M.wbserver = require'websocket'.server.ev.listen
    {
        port = 14001,
        default = echo_handler
    }
end

return _M