--Requirements for upgrading
local BASED_VERSION_NUM='2.00.0001'
local NEW_VERSION_NUM='0230'
local NEW_VERSION='2.00.0230'
local ROOT_REQ_SPACES=-552  --KB
local DATA_REQ_SPACES=0  --KB
local BASED_DEVICE_TYPE=''
local BASED_PRODUCT_ID='S2_SYN'
local client = require "coaf.client"
local _dbg = require "coaf.debug"
local adapter = require "coaf.cli_adapter.simple"

client.InitSocket(nil, adapter)

--------------------------------------
local function getDownloaderProxies()
	local proxy
	local cli
	cli = client.New( "downloader", "CO") --5 seconds timeout.
	if not cli then
		_dbg.Error( "UPGRADE.CO", "Fail to create connection to service 'local/"..service.."'" )
		return nil
	end
	proxy = cli:CreateProxy("/", true) --Exclusive
	if not proxy then
		_dbg.Error( "UPGRADE.CO", "Fail to create object proxy for '/".."/".."@local/".."downloader".."'" )
		cli:Destroy()
		return nil
	end
	return proxy
end

local function sleep(n)
	if ngx and ngx.sleep then
		ngx.sleep(n)
	else
		local r, coaf = pcall( require, "coaf.core" )
		if r and coaf then
			coaf.Sleep(n)
		else
			local socket
			r, socket = pcall( require, "socket" )
			if r and socket then
				socket.sleep(n)
			else
				os.execute( "/bin/usleep " .. math.ceil(n*1000000) )
			end
		end
	end
end
---- Up before this are arguments auto generated by upgrading scripts ---
local function UpgradePreCheck( tmpLoc )
	local x, features = pcall(require, "features")
	if x then
		if not features then
			if _G.DEVICE_TYPE and BASED_DEVICE_TYPE ~= _G.DEVICE_TYPE then
				return false, "device-type-error"
			end
		else
			if features.DEVICE_TYPE and BASED_DEVICE_TYPE ~= features.DEVICE_TYPE then
				return false, "device-type-error"
			end
		end
	end

	require "version"
	if SOFTWARE_VERSION then
		local n_bin_base=tonumber(BASED_VERSION_NUM)
		local n_bin_new=tonumber(NEW_VERSION_NUM)
		local n_dev_rev = SOFTWARE_VERSION:match("%x*%.%x*%.(.+)")
		n_dev_rev = tonumber(n_dev_rev)
		if n_bin_new and n_bin_base and n_dev_rev then
			if n_bin_new >= n_dev_rev and n_bin_base > n_dev_rev then
				return false,"bin-base-version-is-higher"
			elseif n_bin_new < n_dev_rev and n_bin_base < n_dev_rev then
				return false,"version-too-old"
			end
		end
	end

	local copack_f = io.open( tmpLoc .. "/_COPACK/co.bin", "r" )
	local copack_upgrade=false
	if copack_f then
		copack_f:close()
		copack_upgrade = true
	end

	if copack_upgrade then
		local dl = getDownloaderProxies()
		if not dl then
			return false, "precheck.co-error"
		end

		os.execute( "/bin/mkdir -p /tmp/co-package" )
		os.execute( "/bin/mv -f " .. tmpLoc .. "/_COPACK/co.bin /tmp/co-package/co.bin" )

		local n
		local did, done, err, status

		local busy 
		local errors

        did, done, err = dl.StartDownload( "firmware", "/co-package/co.bin", "co.bin" )
        if not did or not done then
            _dbg.Error( "UPGRADE.CO", "Start download error for #", n, ", message is:", err )
            errors = err or 'ERROR'
            dl:Destroy()
            dl = nil
        else
            busy = true
        end

		while busy do
            if dl then
                did, status, err = dl.GetDownloadStatus( "firmware", "co.bin" )
                if did and status and status.status == "finished" then
                --Try again to confirm that.
                    sleep(0.2)
                    did, status, err = dl.GetDownloadStatus( "firmware", "co.bin" )
                end

                if did and status then
                    if status.status == "done" or status.status == "finished" then
                        busy = nil
                        dl:Destroy()
                        dl = nil
                    elseif status.status ~= "pending" and status.status ~= "downloading" then
                        busy = nil
                        errors = status.status
                        dl:Destroy()
                        dl = nil
                        _dbg.Error( "UPGRADE.CO", "Download error for #", n, ", status:", status.status, status.message )
                    end
                else
                    busy = nil
                    errors = err or "ERROR"
                    dl:Destroy()
                    dl = nil
                    _dbg.Error( "UPGRADE.CO", "Download fail, status:", did, status, err )
                end
            end

			if busy then
				sleep(0.5)
			end
		end

		if errors then
			return false, "precheck.co-download"
		end
	end

	return true, "done"
end

return UpgradePreCheck

