#!/bin/sh

export PROD_ID="S2_SYN"
export TARGET_TMP_DIR="/tmp/.upgrade_tmp"

export SWVER="2.00.0230"
export LUA_VERSION="5.3"

export ROOT_REQ_KB="-552"
export DATA_REQ_KB="0"
export UPGRADE_RETURNS=1

#!/bin/sh
export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
mkdir -p /data/update_log
mkdir -p $TARGET_TMP_DIR/_UPGRADE

updatetime=`date "+%Y-%m-%d %H:%M:%S"`
updatetimecode=`date "+%Y%m%d%H%M"`
WITH_WARNING=0
WITH_ERROR=0
logfile="/data/update_log/$SWVER-$updatetimecode.log"

echo "Upgrade to Software version $SWVER..." > $logfile
echo "Upgrade time: $updatetime" >> $logfile

#NOTE: Must disable watch-dog while upgrading!
if [ -x /sbin/wdog ]; then
	/sbin/wdog disable
	echo "Disabled watch-dog while upgrading." >> $logfile
fi

echo "------------------------------------" >> $logfile

if [ $# -gt 0 ]; then
    cd $1
else
    cd /data/update
fi

tar xzf ./patch.tar.gz -C $TARGET_TMP_DIR/_UPGRADE 
if [ $? -ne 0 ]; then
	echo "Upgrading fail! Error to extract the patch file!" >> $logfile
	rm -Rf $TARGET_TMP_DIR
	exit 1
fi

rm -Rf ./patch.tar.gz
cd $TARGET_TMP_DIR	#Change to target position

getDiskFree()
{
	#Because the K2E busybox system only support human format
	local freeSpaces=`df -h $1 | sed 1d | awk '{print $4}'`
	if [ "${freeSpaces/G/}" != "${freeSpaces}" ]; then
		#Is GB
		freeSpaces=${freeSpaces/G/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024 * 1024))"`
	elif [ "${freeSpaces/M/}" != "${freeSpaces}" ]; then
		#Is MB
		freeSpaces=${freeSpaces/M/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024))"`
	elif [ "${freeSpaces/K/}" != "${freeSpaces}" ]; then
		#Is KB
		freeSpaces=${freeSpaces/K/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces))"`
	fi
	echo $freeSpaces
	exit 0
}

root_free_sz=`getDiskFree /`
data_free_sz=`getDiskFree /data`

root_free_sz=`expr $root_free_sz - 1024`	#Keep 1MB at least
data_free_sz=`expr $data_free_sz - 1024`	#Keep 1MB at least

echo "ROOT: Require $ROOT_REQ_KB KB, valid $root_free_sz KB." >> $logfile
echo "DATA: Require $DATA_REQ_KB KB, valid $data_free_sz KB." >> $logfile

if [ $root_free_sz -lt $ROOT_REQ_KB -o $data_free_sz -lt $DATA_REQ_KB ]; then
	echo "ERROR: Insufficent disk spaces" >> $logfile
	exit 1
fi

upgrade()
{
	if [ -d "$TARGET_TMP_DIR/_UPGRADE/fakeroot" ]; then
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/fakeroot/"* / >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/datafs" ]; then
		if [ ! -d /data ]; then
			mkdir /data
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/datafs/"* /data/ >> $logfile 2>&1
	fi
	return 0
}

drop()
{
	if [ "$1" = "fakeroot" ]; then
		if [ -e "$2" ]; then
			rm -Rf "$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	elif [ "$1" = "datafs" ]; then
		if [ -e "/data$2" ]; then
			rm -Rf "/data$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [/data$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	fi
    return 0
}

## The upgrading content below: (generated by release tool)

drop fakeroot "/data/web/html/static/lang/en/audioVideoForm.json"
drop fakeroot "/data/web/html/static/lang/en/codec.json"
drop fakeroot "/data/web/html/static/lang/en/decoder.json"
drop fakeroot "/data/web/html/static/lang/en/driverMode.json"
drop fakeroot "/data/web/html/static/lang/en/encoderInfo.json"
drop fakeroot "/data/web/html/static/lang/en/flow.json"
drop fakeroot "/data/web/html/static/lang/en.json"
drop fakeroot "/data/web/html/static/lang/en/NDIManual.json"
drop fakeroot "/data/web/html/static/lang/en/paneModules.json"
drop fakeroot "/data/web/html/static/lang/en/software.json"
drop fakeroot "/data/web/html/static/lang/en/streamServerForm.json"
drop fakeroot "/data/web/html/static/lang/en/strForm.json"
drop fakeroot "/data/web/html/static/lang/en/superMode.json"
drop fakeroot "/data/web/html/static/lang/en/systemStatus.json"
drop fakeroot "/data/web/html/static/lang/en/videoAudioModules.json"
drop fakeroot "/data/web/html/static/lang/en/videoWebrtc.json"
drop fakeroot "/data/web/html/static/lang/en/warning.json"
drop fakeroot "/data/web/html/static/lang/tw/audioVideoForm.json"
drop fakeroot "/data/web/html/static/lang/tw/codec.json"
drop fakeroot "/data/web/html/static/lang/tw/driverMode.json"
drop fakeroot "/data/web/html/static/lang/tw/encoderInfo.json"
drop fakeroot "/data/web/html/static/lang/tw.json"
drop fakeroot "/data/web/html/static/lang/tw/paneModules.json"
drop fakeroot "/data/web/html/static/lang/tw/software.json"
drop fakeroot "/data/web/html/static/lang/tw/streamServerForm.json"
drop fakeroot "/data/web/html/static/lang/tw/superMode.json"
drop fakeroot "/data/web/html/static/lang/tw/systemStatus.json"
drop fakeroot "/data/web/html/static/lang/tw/tsForm.json"
drop fakeroot "/data/web/html/static/lang/tw/videoAudioModules.json"
drop fakeroot "/data/web/html/static/lang/tw/videoWebrtc.json"
drop fakeroot "/data/web/html/static/lang/tw/warning.json"
drop fakeroot "/data/web/html/static/lang/zh/audioVideoForm.json"
drop fakeroot "/data/web/html/static/lang/zh/codec.json"
drop fakeroot "/data/web/html/static/lang/zh/decoder.json"
drop fakeroot "/data/web/html/static/lang/zh/driverMode.json"
drop fakeroot "/data/web/html/static/lang/zh/encoderInfo.json"
drop fakeroot "/data/web/html/static/lang/zh.json"
drop fakeroot "/data/web/html/static/lang/zh/NDIManual.json"
drop fakeroot "/data/web/html/static/lang/zh/paneModules.json"
drop fakeroot "/data/web/html/static/lang/zh/software.json"
drop fakeroot "/data/web/html/static/lang/zh/streamServerForm.json"
drop fakeroot "/data/web/html/static/lang/zh/superMode.json"
drop fakeroot "/data/web/html/static/lang/zh/systemStatus.json"
drop fakeroot "/data/web/html/static/lang/zh/videoAudioModules.json"
drop fakeroot "/data/web/html/static/lang/zh/videoWebrtc.json"
drop fakeroot "/data/web/html/static/lang/zh/warning.json"
drop fakeroot "/data/web/html/static/mock/config/features/overlaySet.json"
drop fakeroot "/data/web/html/static/mock/config/features/parameter.json"
drop fakeroot "/data/web/html/static/mock/config/features/pattern.json"
drop fakeroot "/data/web/html/static/mock/config/features/sourceType.json"
drop fakeroot "/data/web/html/static/mock/config/lang/en.json"
drop fakeroot "/data/web/html/static/mock/config/lang/tw.json"
drop fakeroot "/data/web/html/static/mock/config/lang/zh.json"
drop fakeroot "/data/web/html/static/mock/config/nav/nav.json"
drop fakeroot "/data/web/html/static/mock/configure/audio/get.json"
drop fakeroot "/data/web/html/static/mock/configure/audio/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/cropper/get.json"
drop fakeroot "/data/web/html/static/mock/configure/cropper/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/imgman/add.json"
drop fakeroot "/data/web/html/static/mock/configure/imgman/get1.json"
drop fakeroot "/data/web/html/static/mock/configure/imgman/get.json"
drop fakeroot "/data/web/html/static/mock/configure/imgman/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/imgman/remove.json"
drop fakeroot "/data/web/html/static/mock/configure/onvif/get.json"
drop fakeroot "/data/web/html/static/mock/configure/onvif/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/osd/add.json"
drop fakeroot "/data/web/html/static/mock/configure/osd/get.json"
drop fakeroot "/data/web/html/static/mock/configure/osd/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/osd/remove.json"
drop fakeroot "/data/web/html/static/mock/configure/preset/apply.json"
drop fakeroot "/data/web/html/static/mock/configure/preset/get.json"
drop fakeroot "/data/web/html/static/mock/configure/preset/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/preset/remove.json"
drop fakeroot "/data/web/html/static/mock/configure/ptz/get.json"
drop fakeroot "/data/web/html/static/mock/configure/ptz/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/ptz/ptzControl.json"
drop fakeroot "/data/web/html/static/mock/configure/ptz/ptzList.json"
drop fakeroot "/data/web/html/static/mock/configure/serial/get.json"
drop fakeroot "/data/web/html/static/mock/configure/serial/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/serial/serialList.json"
drop fakeroot "/data/web/html/static/mock/configure/serverSet/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/serverSet/onvif.json"
drop fakeroot "/data/web/html/static/mock/configure/serverSet/sip_gb.json"
drop fakeroot "/data/web/html/static/mock/configure/sip/get.json"
drop fakeroot "/data/web/html/static/mock/configure/sip/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/txtman/add.json"
drop fakeroot "/data/web/html/static/mock/configure/txtman/get.json"
drop fakeroot "/data/web/html/static/mock/configure/txtman/modify.json"
drop fakeroot "/data/web/html/static/mock/configure/txtman/remove.json"
drop fakeroot "/data/web/html/static/mock/configure/video/get.json"
drop fakeroot "/data/web/html/static/mock/configure/video/modify.json"
drop fakeroot "/data/web/html/static/mock/nav.json"
drop fakeroot "/data/web/html/static/mock/superMode/get.json"
drop fakeroot "/data/web/html/static/mock/wifi/connect.json"
drop fakeroot "/data/web/html/static/mock/wifi/disconnect.json"
drop fakeroot "/data/web/html/static/mock/wifi/get.json"
drop fakeroot "/data/web/html/static/mock/wifi/hotspot.json"
drop fakeroot "/data/web/html/static/mock/wifi/modify.json"
drop fakeroot "/data/web/html/static/mock/wifi/remove.json"
drop fakeroot "/data/web/html/static/mock/wireless/get.json"
drop fakeroot "/data/web/html/static/mock/wireless/modify.json"
drop fakeroot "/usr/local/bin/onvif/test.lua"
upgrade
chown root:root /root
chmod 777 /var/lib
#!/bin/bash

## Upgrading done at here.

if [ -x /usr/sbin/flash_erase -a -x /usr/sbin/nandwrite -a -f /usr/local/bin/uImage_hi3521d ]; then
	NEED_UPDATE_BOOT=y
	# if [ "${PROD_ID:0:2}" = "S2" ]; then
	# 	cat "/proc/cmdline" | grep "mem=512M" > /dev/null 2>&1
	# 	if [ $? -ne 0 ]; then
	# 		NEED_UPDATE_BOOT=y
	# 	fi
	# elif [ "${PROD_ID:0:5}" = "S2OEM" ]; then
	# 	cat "/proc/cmdline" | grep "mem=512M" > /dev/null 2>&1
	# 	if [ $? -ne 0 ]; then
	# 		NEED_UPDATE_BOOT=y
	# 	fi
	# elif [ "${PROD_ID:0:5}" = "S2PRO" ]; then
	# 	cat "/proc/cmdline" | grep "mem=512M" > /dev/null 2>&1
	# 	if [ $? -ne 0 ]; then
	# 		NEED_UPDATE_BOOT=y
	# 	fi
	# fi
	if [ ! -z "$NEED_UPDATE_BOOT" ]; then
		/usr/sbin/flash_erase -q /dev/mtd1 0 32
		if [ $? -ne 0 ]; then
			/usr/sbin/flash_erase -q /dev/mtd1 0 16
		fi
		/usr/sbin/nandwrite -p /dev/mtd1 /usr/local/bin/uImage_hi3521d
	fi
fi

#Fix timezone problem
if [ -e /etc/TZ ]; then
	rm -f /etc/TZ
fi

#Update version.
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/share/lua/$LUA_VERSION/version.lua >> $logfile 2>&1

echo "------------------------------------" >> $logfile
echo "Finished upgrading. [$WITH_ERROR errors, $WITH_WARNING warnnings.]" >> $logfile

rm -Rf $TARGET_TMP_DIR/_UPGRADE
sync
sync
exit $UPGRADE_RETURNS

rm /data/configs/aenc_config.conf
